-- *************************************************************************************************
--   SWISScenter Source                                                              Robert Taylor
-- *************************************************************************************************

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `certificates`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE certificates (
  cert_id          int unsigned auto_increment primary key not null,
  name             varchar(7) not null,
  rank             int not null,
  description      varchar(200) null,
  scheme           text
) ENGINE=MyISAM;

INSERT INTO certificates (name, rank, scheme, description) VALUES ('G',      10, 'MPAA','General Audiences');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('Uc',     10, 'BBFC','Suitable for pre-school');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('U',      20, 'BBFC','Minimum age 4 years');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('PG',     30, 'BBFC','Parental guidance recommended');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('12A',    35, 'BBFC','Minimum age 12 years (unless accompanied)');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('12',     40, 'BBFC','Minimum age 12 years');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('PG-13',  50, 'MPAA','Parents Strongly Cautioned');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('15',     60, 'BBFC','Minimum age 15 years');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('R',      70, 'MPAA','Restricted');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('NC-17',  80, 'MPAA','Not suitable for viewers under 17');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('18',     90, 'BBFC','Minimum age 18 years');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('XXX',   100, 'MPAA','Adult');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('R18',   100, 'BBFC','Restricted distribution, minimum age 18 years');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('PG',     30, 'MPAA','Parental guidance recommended');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('UR',      5, 'MPAA','Un-Rated Family content');
INSERT INTO certificates (name, rank, scheme, description) VALUES ('NR',     90, 'MPAA','Not Rated Adult content');

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `categories`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE categories (
  cat_id          int auto_increment not null primary key,
  cat_name        varchar(100) not null unique,
  download_info   varchar(1) default 'N'
) ENGINE=MyISAM;

INSERT INTO categories (cat_name,download_info) VALUES ( 'General'           ,'N');
INSERT INTO categories (cat_name,download_info) VALUES ( 'Music Videos'      ,'N');
INSERT INTO categories (cat_name,download_info) VALUES ( 'Audio Books'       ,'N');
INSERT INTO categories (cat_name,download_info) VALUES ( 'Films'             ,'Y');
INSERT INTO categories (cat_name,download_info) VALUES ( 'Language Learning' ,'N');
INSERT INTO categories (cat_name,download_info) VALUES ( 'TV Series'         ,'N');

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `users`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE users (
  user_id         int(10) unsigned not null auto_increment,
  name            text,
  maxcert         int unsigned default 1,
  pin             varchar(10) default null
  ,
  PRIMARY KEY  (user_id),
  FOREIGN KEY  (maxcert) REFERENCES certificates (cert_id) ON DELETE SET NULL
) ENGINE=MyISAM;

CREATE TEMPORARY TABLE tt_cert AS SELECT * FROM certificates ORDER BY rank DESC LIMIT 1;
INSERT INTO users (user_id,name) VALUES (1,'Default');
UPDATE users,tt_cert SET users.maxcert = tt_cert.cert_id;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `media_types`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE media_types (
  media_id     int(10) unsigned  not null auto_increment,
  media_name   varchar(20),
  media_table  varchar(20) null
  ,
  PRIMARY KEY  (media_id)
) ENGINE=MyISAM;

INSERT INTO media_types (media_id,media_name,media_table) VALUES (1,'Music','mp3s');
INSERT INTO media_types (media_id,media_name,media_table) VALUES (2,'Photo','photos');
INSERT INTO media_types (media_id,media_name,media_table) VALUES (3,'Video','movies');
INSERT INTO media_types (media_id,media_name,media_table) VALUES (4,'Radio','');

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `media_locations`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE media_locations (
  location_id     int(10) unsigned not null auto_increment,
  name            text,
  media_type	    int(10) unsigned,
  download_info   char(1) default 'Y',
  cat_id          int unsigned not null default 1,
  unrated         int unsigned not null default 1,
  percent_scanned integer
  ,
  PRIMARY KEY  (location_id),
  FOREIGN KEY  (media_type) REFERENCES media_types (media_id) ON DELETE CASCADE,
  FOREIGN KEY  (cat_id)     REFERENCES categories  (cat_id)   ON DELETE SET DEFAULT,
  FOREIGN KEY  (unrated)    REFERENCES certificates(cert_id)  ON DELETE SET DEFAULT
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `messages`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE messages (
  message_id      int(10) unsigned NOT NULL auto_increment,
  title           varchar(255),
  added           varchar(20) NOT NULL,
  message_text    text,
  status          int unsigned default 0
  ,
  PRIMARY KEY  (message_id)
) ENGINE=MyISAM;

insert into messages (title,message_text,added) values ('Welcome to the Swisscenter','This is the messages section, where you will be informed of new features and updates to the SwissCenter interface whenever you perform an automatic update.',now());

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `art_files`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE art_files (
  filename     varchar(100) NOT NULL default ''
  ,
  PRIMARY KEY  (filename)
) ENGINE=MyISAM;

INSERT INTO art_files (filename) VALUES ('folder.jpg');
INSERT INTO art_files (filename) VALUES ('folder.gif');
INSERT INTO art_files (filename) VALUES ('folder.png');

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `clients`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE clients (
  ip_address       varchar(100) NOT NULL default '',
  box_id           varchar(100),
  user_id          int(10) unsigned,
  agent_string     text,
  device_type      text,
  last_seen        datetime     default null,
  screen_type      text         default null,
  screen_x_res     int unsigned default null,
  screen_y_res     int unsigned default null,
  browser_x_res    int unsigned default null,
  browser_y_res    int unsigned default null,
  aspect           text         default null,
  port             integer
  ,
  PRIMARY KEY  (ip_address),
  FOREIGN KEY (user_id) references users (user_id)
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `system_prefs`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE system_prefs (
  name     varchar(100) NOT NULL default '',
  value    text
  ,
  PRIMARY KEY  (name)
) ENGINE=MyISAM;

INSERT INTO system_prefs (name,value) VALUES ('CACHE_MAXSIZE_MB','10');

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `user_prefs`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE user_prefs (
  user_id  int(10) unsigned NOT NULL auto_increment,
  name     varchar(100) NOT NULL default '',
  value    text
  ,
  PRIMARY KEY  (user_id,name),
  FOREIGN KEY  (user_id) REFERENCES users (user_id) ON DELETE CASCADE
) ENGINE=MyISAM;


INSERT INTO user_prefs (user_id,name,value) VALUES (1,'WEATHER_HOME','UKXX0022');
INSERT INTO user_prefs (user_id,name,value) VALUES (1,'WEATHER_UNITS','m');
INSERT INTO user_prefs (user_id,name,value) VALUES (1,'STYLE','KDE');

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `movies`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE movies (
  file_id             int(10) unsigned not null auto_increment,
  dirname             text,
  filename            text,
  title               text,
  size                int(11) default null,
  length              int(11) default null,
  lengthstring        text,
  verified            char(1),
  year                varchar(4),
  details_available   varchar(1),
  match_pc            int(10),
  discovered          datetime default null,
  location_id         int unsigned,
  certificate         int unsigned null,
  synopsis            text
  ,
  PRIMARY KEY  (file_id),
  FOREIGN KEY  (location_id) REFERENCES media_locations (location_id) ON DELETE CASCADE,
  FOREIGN KEY  (certificate) REFERENCES certificates(cert_id) ON DELETE SET NULL,
  KEY title    (title(50))
) ENGINE=MyISAM;

CREATE UNIQUE INDEX movies_fsp_u1 ON movies (dirname(250),filename(250));

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `actors`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE actors (
  actor_id     int(10) unsigned  NOT NULL auto_increment,
  actor_name   text
  ,
  PRIMARY KEY  (actor_id),
  UNIQUE (actor_name(100))
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `directors`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE directors (
  director_id     int(10) unsigned  NOT NULL auto_increment,
  director_name   text
  ,
  PRIMARY KEY  (director_id),
  UNIQUE (director_name(100))
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `genres`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE genres (
  genre_id     int(10) unsigned  NOT NULL auto_increment,
  genre_name   text
  ,
  PRIMARY KEY  (genre_id),
  UNIQUE (genre_name(100))
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `actors_in_movie`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE actors_in_movie (
  movie_id     int(10) unsigned,
  actor_id     int(10) unsigned
  ,
  PRIMARY KEY (movie_id, actor_id),
  FOREIGN KEY  (actor_id) REFERENCES actors (actor_id) ON DELETE CASCADE
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `directors_of_movie`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE directors_of_movie (
  movie_id     int(10) unsigned,
  director_id  int(10) unsigned
  ,
  PRIMARY KEY (movie_id, director_id),
  FOREIGN KEY  (director_id) REFERENCES directors (director_id) ON DELETE CASCADE
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `genres_of_movie`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE genres_of_movie (
  movie_id     int(10) unsigned,
  genre_id     int(10) unsigned
  ,
  PRIMARY KEY (movie_id, genre_id),
  FOREIGN KEY  (genre_id) REFERENCES genres (genre_id) ON DELETE CASCADE
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `mp3s`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE mp3s (
  file_id          int(10) unsigned not null auto_increment,
  dirname          text,
  filename         text,
  size int(11)     default null,
  length int(11)   default null,
  lengthstring     text,
  bitrate          text,
  version          text,
  title            text,
  artist           text,
  album            text,
  year             text,
  track            varchar(50) default null,
  genre            text,
  verified         char(1),
  discovered       datetime default null,
  location_id      int unsigned,
  certificate      int unsigned null,
  bitrate_mode     text default null
  ,
  PRIMARY KEY  (file_id),
  FOREIGN KEY  (location_id) REFERENCES media_locations (location_id) ON DELETE CASCADE,
  FOREIGN KEY (certificate)  REFERENCES certificates(cert_id)         ON DELETE SET NULL,
  KEY title    (title(50)),
  KEY artist   (artist(50)),
  KEY album    (album(50)),
  KEY genre    (genre(50)),
  KEY year     (year(50)),
  KEY dirname  (dirname(255)),
  KEY filename (dirname(255))
) ENGINE=MyISAM;

CREATE UNIQUE INDEX mp3s_fsp_u1   ON mp3s   (dirname(250),filename(250));

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `mp3_albumart`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE mp3_albumart (
  file_id          int unsigned not null,
  image            mediumblob not null
  ,
  FOREIGN KEY (file_id) references mp3s (file_id)
) ENGINE=MyISAM;

CREATE INDEX mp3s_art_n1 ON mp3_albumart (file_id);

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `photos`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE photos (
  file_id             int(10) unsigned not null auto_increment,
  dirname             text,
  filename            text,
  size                int(11) default null,
  width               int(11) default null,
  height              int(11) default null,
  date_modified       varchar(50),
  date_created        varchar(50),
  verified            char(1) default null,
  discovered          datetime default null,
  location_id         int unsigned,
  certificate         int unsigned null,
  exif_exposure_mode  text default null,
  exif_exposure_time  text default null,
  exif_fnumber        text default null,
  exif_focal_length   text default null,
  exif_image_source   text default null,
  exif_make           text default null,
  exif_model          text default null,
  exif_orientation    text default null,
  exif_white_balance  text default null,
  exif_flash          text default null,
  exif_iso            text default null,
  exif_light_source   text default null,
  exif_exposure_prog  text default null,
  exif_meter_mode     text default null,
  exif_capture_type   text default null
  ,
  PRIMARY KEY  (file_id),
  FOREIGN KEY  (location_id) REFERENCES media_locations (location_id) ON DELETE CASCADE,
  FOREIGN KEY (certificate)  REFERENCES certificates(cert_id)         ON DELETE SET NULL,
  KEY dirname  (dirname(255)),
  KEY filename (dirname(255))
) ENGINE=MyISAM;

CREATE UNIQUE INDEX photos_fsp_u1 ON photos (dirname(250),filename(250));

-- -------------------------------------------------------------------------------------------------
-- Create table to hold photo "albums" information.
-- -------------------------------------------------------------------------------------------------

CREATE TABLE photo_albums (
  file_id          int(10) unsigned not null auto_increment,
  dirname          text,
  title            text,
  verified         char(1),
  discovered       datetime default null,
  location_id      int unsigned,
  certificate      int unsigned null
  ,
  PRIMARY KEY  (file_id),
  FOREIGN KEY  (certificate) REFERENCES certificates(cert_id) ON DELETE SET NULL,
  FOREIGN KEY  (location_id) REFERENCES media_locations (location_id) ON DELETE CASCADE,
  KEY title    (title(50))
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `viewings`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE viewings
( user_id          int(10) unsigned NOT NULL
, media_type       int(10) unsigned NOT NULL
, media_id         int(10) unsigned NOT NULL
, last_viewed      datetime NOT NULL
, total_viewings   int(10) unsigned default 0
) ENGINE=MyISAM;

CREATE INDEX viewings_n1 ON viewings (media_id);

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `viewings`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE rss_subscriptions (
  id               int auto_increment,
  type             int not null,
  url              text not null,
  title            varchar(50) not null,
  update_frequency int not null default 60,
  last_update      datetime default null
  ,
  PRIMARY KEY  (id)
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `viewings`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE rss_items (
  id               int auto_increment,
  subscription_id  int not null,
  title            text not null,
  url              text,
  description      text not null,
  published_date   datetime not null,
  timestamp        int not null,
  guid             text,
  linked_file      text
  ,
  PRIMARY KEY  (id)
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `weather`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE weather (
  url varchar(255) NOT NULL default '',
  xml text NOT NULL,
  requested bigint(20) NOT NULL default '0',
  type varchar(10) default NULL
  ,
  PRIMARY KEY  (url,type)
) ENGINE=MyISAM;

-- -------------------------------------------------------------------------------------------------
-- Table structure for table `cities`
-- -------------------------------------------------------------------------------------------------

CREATE TABLE cities (
  name varchar(100) NOT NULL default '',
  twc_code varchar(50)
  ,
  PRIMARY KEY  (name)
) ENGINE=MyISAM;

INSERT INTO cities (name) VALUES ('Aachen');
INSERT INTO cities (name) VALUES ('Aalborg');
INSERT INTO cities (name) VALUES ('Aappilattoq');
INSERT INTO cities (name) VALUES ('Aarhus');
INSERT INTO cities (name) VALUES ('Aasiaat');
INSERT INTO cities (name) VALUES ('Aba');
INSERT INTO cities (name) VALUES ('Abadan');
INSERT INTO cities (name) VALUES ('Abakaliki');
INSERT INTO cities (name) VALUES ('Abakan');
INSERT INTO cities (name) VALUES ('Abaokoro');
INSERT INTO cities (name) VALUES ('Abbotsford');
INSERT INTO cities (name) VALUES ('Abeche');
INSERT INTO cities (name) VALUES ('Abengourou');
INSERT INTO cities (name) VALUES ('Abeokuta');
INSERT INTO cities (name) VALUES ('Aberdare');
INSERT INTO cities (name) VALUES ('Aberdeen');
INSERT INTO cities (name) VALUES ('Abha');
INSERT INTO cities (name) VALUES ('Abidjan');
INSERT INTO cities (name) VALUES ('Abilene');
INSERT INTO cities (name) VALUES ('Abingdon');
INSERT INTO cities (name) VALUES ('Abomey');
INSERT INTO cities (name) VALUES ('Abottabad');
INSERT INTO cities (name) VALUES ('Abovyan');
INSERT INTO cities (name) VALUES ('Abu Dhabi');
INSERT INTO cities (name) VALUES ('Abuja');
INSERT INTO cities (name) VALUES ('Acapulco');
INSERT INTO cities (name) VALUES ('Acarigua');
INSERT INTO cities (name) VALUES ('Accra');
INSERT INTO cities (name) VALUES ('Accrington');
INSERT INTO cities (name) VALUES ('Acheng');
INSERT INTO cities (name) VALUES ('Achinsk');
INSERT INTO cities (name) VALUES ('Ad Da\'ein');
INSERT INTO cities (name) VALUES ('ad-Dammam');
INSERT INTO cities (name) VALUES ('Adamstown');
INSERT INTO cities (name) VALUES ('Adana');
INSERT INTO cities (name) VALUES ('Adapazari');
INSERT INTO cities (name) VALUES ('AddisAbaba');
INSERT INTO cities (name) VALUES ('Adelaide');
INSERT INTO cities (name) VALUES ('Aden');
INSERT INTO cities (name) VALUES ('Adiyaman');
INSERT INTO cities (name) VALUES ('Ado');
INSERT INTO cities (name) VALUES ('Afgoye');
INSERT INTO cities (name) VALUES ('Afyonkarahisar');
INSERT INTO cities (name) VALUES ('Agadez');
INSERT INTO cities (name) VALUES ('Agadir');
INSERT INTO cities (name) VALUES ('Agana');
INSERT INTO cities (name) VALUES ('Agra');
INSERT INTO cities (name) VALUES ('Aguadilla');
INSERT INTO cities (name) VALUES ('Aguascalientes');
INSERT INTO cities (name) VALUES ('Ahmadabad');
INSERT INTO cities (name) VALUES ('Ahmadnagar');
INSERT INTO cities (name) VALUES ('Ahmadpur East');
INSERT INTO cities (name) VALUES ('Ahvaz');
INSERT INTO cities (name) VALUES ('Airai');
INSERT INTO cities (name) VALUES ('Aix-en-Provence');
INSERT INTO cities (name) VALUES ('Aizawl');
INSERT INTO cities (name) VALUES ('Ajdabiyah');
INSERT INTO cities (name) VALUES ('Ajman');
INSERT INTO cities (name) VALUES ('Ajmer');
INSERT INTO cities (name) VALUES ('Akita');
INSERT INTO cities (name) VALUES ('Akmechet (Kyzyl-Orda)');
INSERT INTO cities (name) VALUES ('Akola');
INSERT INTO cities (name) VALUES ('Akron');
INSERT INTO cities (name) VALUES ('Aksaray');
INSERT INTO cities (name) VALUES ('Aksu');
INSERT INTO cities (name) VALUES ('Akunnaaq');
INSERT INTO cities (name) VALUES ('Akure');
INSERT INTO cities (name) VALUES ('Akureyri');
INSERT INTO cities (name) VALUES ('Akyab');
INSERT INTO cities (name) VALUES ('Al Ain');
INSERT INTO cities (name) VALUES ('Al Muharraq');
INSERT INTO cities (name) VALUES ('Al-Aziziya');
INSERT INTO cities (name) VALUES ('al-Bayda');
INSERT INTO cities (name) VALUES ('al-Hawiyah');
INSERT INTO cities (name) VALUES ('al-Hufuf');
INSERT INTO cities (name) VALUES ('al-Kut');
INSERT INTO cities (name) VALUES ('al-Najaf');
INSERT INTO cities (name) VALUES ('al-Qurayyat');
INSERT INTO cities (name) VALUES ('Alacant');
INSERT INTO cities (name) VALUES ('Alagoinhas');
INSERT INTO cities (name) VALUES ('Alameda');
INSERT INTO cities (name) VALUES ('Alanya');
INSERT INTO cities (name) VALUES ('Albacete');
INSERT INTO cities (name) VALUES ('Albany');
INSERT INTO cities (name) VALUES ('Albuquerque');
INSERT INTO cities (name) VALUES ('Alchevsk');
INSERT INTO cities (name) VALUES ('Aleppo (Halab)');
INSERT INTO cities (name) VALUES ('Alessandria');
INSERT INTO cities (name) VALUES ('Alexandria');
INSERT INTO cities (name) VALUES ('Alfred-Faure');
INSERT INTO cities (name) VALUES ('Algeciras');
INSERT INTO cities (name) VALUES ('Algiers');
INSERT INTO cities (name) VALUES ('Alhambra');
INSERT INTO cities (name) VALUES ('Aligarh');
INSERT INTO cities (name) VALUES ('Aliso Viejo');
INSERT INTO cities (name) VALUES ('Allahabad');
INSERT INTO cities (name) VALUES ('Allentown');
INSERT INTO cities (name) VALUES ('AlluitsupPaa');
INSERT INTO cities (name) VALUES ('Almaty');
INSERT INTO cities (name) VALUES ('Almere');
INSERT INTO cities (name) VALUES ('Almeria');
INSERT INTO cities (name) VALUES ('Almetyevsk');
INSERT INTO cities (name) VALUES ('Alofi');
INSERT INTO cities (name) VALUES ('AlorSetar');
INSERT INTO cities (name) VALUES ('Altoona');
INSERT INTO cities (name) VALUES ('Alvorada');
INSERT INTO cities (name) VALUES ('Alwar');
INSERT INTO cities (name) VALUES ('Alytus');
INSERT INTO cities (name) VALUES ('Amadora');
INSERT INTO cities (name) VALUES ('Amagasaki');
INSERT INTO cities (name) VALUES ('Amaigbo');
INSERT INTO cities (name) VALUES ('Amarah');
INSERT INTO cities (name) VALUES ('Amarillo');
INSERT INTO cities (name) VALUES ('Ambato');
INSERT INTO cities (name) VALUES ('Ambon');
INSERT INTO cities (name) VALUES ('American Fork');
INSERT INTO cities (name) VALUES ('Americana');
INSERT INTO cities (name) VALUES ('Amersfoort');
INSERT INTO cities (name) VALUES ('Ames');
INSERT INTO cities (name) VALUES ('Amiens');
INSERT INTO cities (name) VALUES ('Amman');
INSERT INTO cities (name) VALUES ('Amol');
INSERT INTO cities (name) VALUES ('Amparai');
INSERT INTO cities (name) VALUES ('Amravati');
INSERT INTO cities (name) VALUES ('Amritsar');
INSERT INTO cities (name) VALUES ('Amsterdam');
INSERT INTO cities (name) VALUES ('Amundsen-Scott');
INSERT INTO cities (name) VALUES ('Anaheim');
INSERT INTO cities (name) VALUES ('Ananindeua');
INSERT INTO cities (name) VALUES ('Anantapur');
INSERT INTO cities (name) VALUES ('Anapolis');
INSERT INTO cities (name) VALUES ('Anbu');
INSERT INTO cities (name) VALUES ('Anchorage');
INSERT INTO cities (name) VALUES ('Ancona');
INSERT INTO cities (name) VALUES ('Anda');
INSERT INTO cities (name) VALUES ('AndersonAirForceBase');
INSERT INTO cities (name) VALUES ('Andimeshk');
INSERT INTO cities (name) VALUES ('Andizhan');
INSERT INTO cities (name) VALUES ('Andong');
INSERT INTO cities (name) VALUES ('AndorralaVella');
INSERT INTO cities (name) VALUES ('Andover');
INSERT INTO cities (name) VALUES ('Andria');
INSERT INTO cities (name) VALUES ('Aneho');
INSERT INTO cities (name) VALUES ('Angarsk');
INSERT INTO cities (name) VALUES ('Angeles');
INSERT INTO cities (name) VALUES ('Angers');
INSERT INTO cities (name) VALUES ('Angoche');
INSERT INTO cities (name) VALUES ('AngradosReis');
INSERT INTO cities (name) VALUES ('Angren');
INSERT INTO cities (name) VALUES ('Ankang');
INSERT INTO cities (name) VALUES ('Ankara');
INSERT INTO cities (name) VALUES ('Ann Arbor');
INSERT INTO cities (name) VALUES ('Annaba');
INSERT INTO cities (name) VALUES ('Annapolis');
INSERT INTO cities (name) VALUES ('Annecy');
INSERT INTO cities (name) VALUES ('Anqing');
INSERT INTO cities (name) VALUES ('Anqiu');
INSERT INTO cities (name) VALUES ('Ansan');
INSERT INTO cities (name) VALUES ('AnseRoyal');
INSERT INTO cities (name) VALUES ('Anshan');
INSERT INTO cities (name) VALUES ('Anshun');
INSERT INTO cities (name) VALUES ('Antakya');
INSERT INTO cities (name) VALUES ('Antalya');
INSERT INTO cities (name) VALUES ('Antananarivo');
INSERT INTO cities (name) VALUES ('Antioch');
INSERT INTO cities (name) VALUES ('Antofagasta');
INSERT INTO cities (name) VALUES ('Antsirabe');
INSERT INTO cities (name) VALUES ('Antsiranana');
INSERT INTO cities (name) VALUES ('Antwerp');
INSERT INTO cities (name) VALUES ('Anyang');
INSERT INTO cities (name) VALUES ('Anzhero-Sudzhensk');
INSERT INTO cities (name) VALUES ('Aomen');
INSERT INTO cities (name) VALUES ('Aomori');
INSERT INTO cities (name) VALUES ('AparecidadeGoiania');
INSERT INTO cities (name) VALUES ('Apeldoorn');
INSERT INTO cities (name) VALUES ('Apia');
INSERT INTO cities (name) VALUES ('Apple Valley');
INSERT INTO cities (name) VALUES ('Appleton');
INSERT INTO cities (name) VALUES ('ApraHarbor');
INSERT INTO cities (name) VALUES ('Apucarana');
INSERT INTO cities (name) VALUES ('Aqaba');
INSERT INTO cities (name) VALUES ('Aqtau');
INSERT INTO cities (name) VALUES ('Aqtobe');
INSERT INTO cities (name) VALUES ('Araar');
INSERT INTO cities (name) VALUES ('Aracaju');
INSERT INTO cities (name) VALUES ('Aracatuba');
INSERT INTO cities (name) VALUES ('Arad');
INSERT INTO cities (name) VALUES ('Araguaina');
INSERT INTO cities (name) VALUES ('Arak');
INSERT INTO cities (name) VALUES ('Arapiraca');
INSERT INTO cities (name) VALUES ('Araraquara');
INSERT INTO cities (name) VALUES ('ArbaMinch');
INSERT INTO cities (name) VALUES ('Arcadia');
INSERT INTO cities (name) VALUES ('Arctowski');
INSERT INTO cities (name) VALUES ('Ardabil');
INSERT INTO cities (name) VALUES ('Arecibo');
INSERT INTO cities (name) VALUES ('Arequipa');
INSERT INTO cities (name) VALUES ('Arezzo');
INSERT INTO cities (name) VALUES ('Arhangelsk');
INSERT INTO cities (name) VALUES ('Arica');
INSERT INTO cities (name) VALUES ('Arima');
INSERT INTO cities (name) VALUES ('Arles');
INSERT INTO cities (name) VALUES ('Arlington');
INSERT INTO cities (name) VALUES ('Arlit');
INSERT INTO cities (name) VALUES ('Armavir');
INSERT INTO cities (name) VALUES ('Armenia');
INSERT INTO cities (name) VALUES ('Arnhem');
INSERT INTO cities (name) VALUES ('Arraijan');
INSERT INTO cities (name) VALUES ('Arsuk');
INSERT INTO cities (name) VALUES ('Artigas');
INSERT INTO cities (name) VALUES ('Arusha');
INSERT INTO cities (name) VALUES ('Arvada');
INSERT INTO cities (name) VALUES ('Arzamas');
INSERT INTO cities (name) VALUES ('as-Salimiyah');
INSERT INTO cities (name) VALUES ('as-Samawah');
INSERT INTO cities (name) VALUES ('as-Shihr');
INSERT INTO cities (name) VALUES ('Asahikawa');
INSERT INTO cities (name) VALUES ('Asansol');
INSERT INTO cities (name) VALUES ('Asau');
INSERT INTO cities (name) VALUES ('Asfi');
INSERT INTO cities (name) VALUES ('Asheville');
INSERT INTO cities (name) VALUES ('Ashford');
INSERT INTO cities (name) VALUES ('Ashikaga');
INSERT INTO cities (name) VALUES ('Ashqabat');
INSERT INTO cities (name) VALUES ('Ashqelon');
INSERT INTO cities (name) VALUES ('Asmara');
INSERT INTO cities (name) VALUES ('Asseb');
INSERT INTO cities (name) VALUES ('Assela');
INSERT INTO cities (name) VALUES ('Assomada');
INSERT INTO cities (name) VALUES ('Astana');
INSERT INTO cities (name) VALUES ('Astrakhan');
INSERT INTO cities (name) VALUES ('Asuncion');
INSERT INTO cities (name) VALUES ('Aswan');
INSERT INTO cities (name) VALUES ('Asyut');
INSERT INTO cities (name) VALUES ('at-Taif');
INSERT INTO cities (name) VALUES ('Atafu');
INSERT INTO cities (name) VALUES ('Atakpame');
INSERT INTO cities (name) VALUES ('Atammik');
INSERT INTO cities (name) VALUES ('Atbarah');
INSERT INTO cities (name) VALUES ('Athens');
INSERT INTO cities (name) VALUES ('Athens-Clarke');
INSERT INTO cities (name) VALUES ('Atlanta');
INSERT INTO cities (name) VALUES ('Atlantic City');
INSERT INTO cities (name) VALUES ('Attu');
INSERT INTO cities (name) VALUES ('Atyrau');
INSERT INTO cities (name) VALUES ('Auburn');
INSERT INTO cities (name) VALUES ('Auckland');
INSERT INTO cities (name) VALUES ('Augsburg');
INSERT INTO cities (name) VALUES ('Augusta');
INSERT INTO cities (name) VALUES ('Augusta-Richmond');
INSERT INTO cities (name) VALUES ('Auki');
INSERT INTO cities (name) VALUES ('Aurora');
INSERT INTO cities (name) VALUES ('Austin');
INSERT INTO cities (name) VALUES ('Avarua');
INSERT INTO cities (name) VALUES ('Avignon');
INSERT INTO cities (name) VALUES ('Awassa');
INSERT INTO cities (name) VALUES ('Awbari');
INSERT INTO cities (name) VALUES ('Awka');
INSERT INTO cities (name) VALUES ('Ayacucho');
INSERT INTO cities (name) VALUES ('Aydin');
INSERT INTO cities (name) VALUES ('Ayorou');
INSERT INTO cities (name) VALUES ('Ayr');
INSERT INTO cities (name) VALUES ('Babol');
INSERT INTO cities (name) VALUES ('Babruysk');
INSERT INTO cities (name) VALUES ('Bacau');
INSERT INTO cities (name) VALUES ('Bacolod');
INSERT INTO cities (name) VALUES ('Badajoz');
INSERT INTO cities (name) VALUES ('Badaojiang');
INSERT INTO cities (name) VALUES ('Bafata');
INSERT INTO cities (name) VALUES ('Bafilo');
INSERT INTO cities (name) VALUES ('Bafoussam');
INSERT INTO cities (name) VALUES ('Baghdad');
INSERT INTO cities (name) VALUES ('Bago');
INSERT INTO cities (name) VALUES ('Baguio');
INSERT INTO cities (name) VALUES ('Bahawalnagar');
INSERT INTO cities (name) VALUES ('Bahawalpur');
INSERT INTO cities (name) VALUES ('BahiaBlanca');
INSERT INTO cities (name) VALUES ('BahirDar');
INSERT INTO cities (name) VALUES ('BaiaMare');
INSERT INTO cities (name) VALUES ('Baicheng');
INSERT INTO cities (name) VALUES ('Baie-Mahault');
INSERT INTO cities (name) VALUES ('Bairiki');
INSERT INTO cities (name) VALUES ('Baiyin');
INSERT INTO cities (name) VALUES ('Bajil');
INSERT INTO cities (name) VALUES ('BajosdeHaina');
INSERT INTO cities (name) VALUES ('Bakau');
INSERT INTO cities (name) VALUES ('Bakersfield');
INSERT INTO cities (name) VALUES ('Baku');
INSERT INTO cities (name) VALUES ('Balakovo');
INSERT INTO cities (name) VALUES ('Balashiha');
INSERT INTO cities (name) VALUES ('Balashov');
INSERT INTO cities (name) VALUES ('Baldwin Park');
INSERT INTO cities (name) VALUES ('Balikesir');
INSERT INTO cities (name) VALUES ('Balikpapan');
INSERT INTO cities (name) VALUES ('Baliuag');
INSERT INTO cities (name) VALUES ('Balkanabat');
INSERT INTO cities (name) VALUES ('Bally');
INSERT INTO cities (name) VALUES ('Ballymena');
INSERT INTO cities (name) VALUES ('Balti');
INSERT INTO cities (name) VALUES ('Baltimore');
INSERT INTO cities (name) VALUES ('Balykchy');
INSERT INTO cities (name) VALUES ('Balzers');
INSERT INTO cities (name) VALUES ('Bama');
INSERT INTO cities (name) VALUES ('Bamako');
INSERT INTO cities (name) VALUES ('Bambari');
INSERT INTO cities (name) VALUES ('Bamenda');
INSERT INTO cities (name) VALUES ('Banbury');
INSERT INTO cities (name) VALUES ('Banda Aceh');
INSERT INTO cities (name) VALUES ('Bandar Lampung');
INSERT INTO cities (name) VALUES ('Bandar-eAbbas');
INSERT INTO cities (name) VALUES ('Bandar-eAnzali');
INSERT INTO cities (name) VALUES ('Bandar-eMahshahr');
INSERT INTO cities (name) VALUES ('BandarPenggaram');
INSERT INTO cities (name) VALUES ('BandarSeriBegawan');
INSERT INTO cities (name) VALUES ('Bandirma');
INSERT INTO cities (name) VALUES ('Bandundu');
INSERT INTO cities (name) VALUES ('Bandung');
INSERT INTO cities (name) VALUES ('Banfora');
INSERT INTO cities (name) VALUES ('Bangalore');
INSERT INTO cities (name) VALUES ('Bangkok');
INSERT INTO cities (name) VALUES ('Bangui');
INSERT INTO cities (name) VALUES ('Banha');
INSERT INTO cities (name) VALUES ('Banjarmasin');
INSERT INTO cities (name) VALUES ('BanjaLuka');
INSERT INTO cities (name) VALUES ('Banjul');
INSERT INTO cities (name) VALUES ('Bantam');
INSERT INTO cities (name) VALUES ('Bantou');
INSERT INTO cities (name) VALUES ('Baoding');
INSERT INTO cities (name) VALUES ('Baoji');
INSERT INTO cities (name) VALUES ('Baoshan');
INSERT INTO cities (name) VALUES ('Baotou');
INSERT INTO cities (name) VALUES ('Baqubah');
INSERT INTO cities (name) VALUES ('Baranagar');
INSERT INTO cities (name) VALUES ('Baranovici');
INSERT INTO cities (name) VALUES ('Barasat');
INSERT INTO cities (name) VALUES ('Barbacena');
INSERT INTO cities (name) VALUES ('Barcelona');
INSERT INTO cities (name) VALUES ('Barddhaman');
INSERT INTO cities (name) VALUES ('Bareli');
INSERT INTO cities (name) VALUES ('Barentsburg');
INSERT INTO cities (name) VALUES ('Bari');
INSERT INTO cities (name) VALUES ('Barinas');
INSERT INTO cities (name) VALUES ('Barisal');
INSERT INTO cities (name) VALUES ('Barletta');
INSERT INTO cities (name) VALUES ('Barnaul');
INSERT INTO cities (name) VALUES ('Barnstable Town');
INSERT INTO cities (name) VALUES ('Barnstaple');
INSERT INTO cities (name) VALUES ('Barqa');
INSERT INTO cities (name) VALUES ('Barquisimeto');
INSERT INTO cities (name) VALUES ('Barrancabermeja');
INSERT INTO cities (name) VALUES ('Barranquilla');
INSERT INTO cities (name) VALUES ('BarraMansa');
INSERT INTO cities (name) VALUES ('Barreiras');
INSERT INTO cities (name) VALUES ('Barrigada');
INSERT INTO cities (name) VALUES ('Barrow-in-Furness');
INSERT INTO cities (name) VALUES ('Barry');
INSERT INTO cities (name) VALUES ('Barueri');
INSERT INTO cities (name) VALUES ('Basel');
INSERT INTO cities (name) VALUES ('Basildon');
INSERT INTO cities (name) VALUES ('Basingstoke');
INSERT INTO cities (name) VALUES ('Basrah');
INSERT INTO cities (name) VALUES ('Bassar');
INSERT INTO cities (name) VALUES ('Basseterre');
INSERT INTO cities (name) VALUES ('Bata');
INSERT INTO cities (name) VALUES ('Batam');
INSERT INTO cities (name) VALUES ('Batdambang');
INSERT INTO cities (name) VALUES ('Bath');
INSERT INTO cities (name) VALUES ('Bathinda');
INSERT INTO cities (name) VALUES ('Batman');
INSERT INTO cities (name) VALUES ('Batnah');
INSERT INTO cities (name) VALUES ('Baton Rouge');
INSERT INTO cities (name) VALUES ('Battle Creek');
INSERT INTO cities (name) VALUES ('Batumi');
INSERT INTO cities (name) VALUES ('Baucau');
INSERT INTO cities (name) VALUES ('Bauchi');
INSERT INTO cities (name) VALUES ('Bauru');
INSERT INTO cities (name) VALUES ('Bayamo');
INSERT INTO cities (name) VALUES ('Bayamon');
INSERT INTO cities (name) VALUES ('Baydabo');
INSERT INTO cities (name) VALUES ('Be\'erSheva');
INSERT INTO cities (name) VALUES ('Beaumont');
INSERT INTO cities (name) VALUES ('Beauvais');
INSERT INTO cities (name) VALUES ('Beaverton');
INSERT INTO cities (name) VALUES ('Bedford');
INSERT INTO cities (name) VALUES ('Bedworth');
INSERT INTO cities (name) VALUES ('Behbahan');
INSERT INTO cities (name) VALUES ('Beian');
INSERT INTO cities (name) VALUES ('Beibei');
INSERT INTO cities (name) VALUES ('Beihai');
INSERT INTO cities (name) VALUES ('Beijing');
INSERT INTO cities (name) VALUES ('Beipiao');
INSERT INTO cities (name) VALUES ('Beira');
INSERT INTO cities (name) VALUES ('Beirut');
INSERT INTO cities (name) VALUES ('Bejaia');
INSERT INTO cities (name) VALUES ('Bekobod');
INSERT INTO cities (name) VALUES ('Belem');
INSERT INTO cities (name) VALUES ('Belfast');
INSERT INTO cities (name) VALUES ('BelfordRoxo');
INSERT INTO cities (name) VALUES ('Belfort');
INSERT INTO cities (name) VALUES ('Belgaum');
INSERT INTO cities (name) VALUES ('Belgorod');
INSERT INTO cities (name) VALUES ('Belgrade');
INSERT INTO cities (name) VALUES ('Belize');
INSERT INTO cities (name) VALUES ('Bellary');
INSERT INTO cities (name) VALUES ('Bellevue');
INSERT INTO cities (name) VALUES ('Bellflower');
INSERT INTO cities (name) VALUES ('Bellingham');
INSERT INTO cities (name) VALUES ('Bellingshausen');
INSERT INTO cities (name) VALUES ('Bello');
INSERT INTO cities (name) VALUES ('Belmopan');
INSERT INTO cities (name) VALUES ('BeloHorizonte');
INSERT INTO cities (name) VALUES ('Bend');
INSERT INTO cities (name) VALUES ('Bengbu');
INSERT INTO cities (name) VALUES ('Benghazi');
INSERT INTO cities (name) VALUES ('Bengkulu');
INSERT INTO cities (name) VALUES ('Benguela');
INSERT INTO cities (name) VALUES ('Beni Mellal');
INSERT INTO cities (name) VALUES ('Beni Suef');
INSERT INTO cities (name) VALUES ('Benin');
INSERT INTO cities (name) VALUES ('Benoni');
INSERT INTO cities (name) VALUES ('Benxi');
INSERT INTO cities (name) VALUES ('Berbera');
INSERT INTO cities (name) VALUES ('Berberati');
INSERT INTO cities (name) VALUES ('Berdyansk');
INSERT INTO cities (name) VALUES ('Berezniki');
INSERT INTO cities (name) VALUES ('Bergen');
INSERT INTO cities (name) VALUES ('Bergisch Gladbach');
INSERT INTO cities (name) VALUES ('Berkeley');
INSERT INTO cities (name) VALUES ('Berlin');
INSERT INTO cities (name) VALUES ('Bern');
INSERT INTO cities (name) VALUES ('BernardoO\'Higgins');
INSERT INTO cities (name) VALUES ('Bertoua');
INSERT INTO cities (name) VALUES ('Berwick-upon-Tweed');
INSERT INTO cities (name) VALUES ('Besancon');
INSERT INTO cities (name) VALUES ('Bethlehem');
INSERT INTO cities (name) VALUES ('Betim');
INSERT INTO cities (name) VALUES ('Bexhil');
INSERT INTO cities (name) VALUES ('Beziers');
INSERT INTO cities (name) VALUES ('Bhagalpur');
INSERT INTO cities (name) VALUES ('Bharatpur');
INSERT INTO cities (name) VALUES ('Bhatpara');
INSERT INTO cities (name) VALUES ('Bhavnagar');
INSERT INTO cities (name) VALUES ('Bhilai');
INSERT INTO cities (name) VALUES ('Bhilwara');
INSERT INTO cities (name) VALUES ('Bhiwandi');
INSERT INTO cities (name) VALUES ('Bhopal');
INSERT INTO cities (name) VALUES ('Bhubaneswar');
INSERT INTO cities (name) VALUES ('Bialystok');
INSERT INTO cities (name) VALUES ('Bida');
INSERT INTO cities (name) VALUES ('Bielefeld');
INSERT INTO cities (name) VALUES ('Bielsko-Biala');
INSERT INTO cities (name) VALUES ('BienHoa');
INSERT INTO cities (name) VALUES ('BigBend');
INSERT INTO cities (name) VALUES ('Bihac');
INSERT INTO cities (name) VALUES ('Bihar');
INSERT INTO cities (name) VALUES ('Bijapur');
INSERT INTO cities (name) VALUES ('Bikaner');
INSERT INTO cities (name) VALUES ('Bikenibeu');
INSERT INTO cities (name) VALUES ('Bila Tserkva');
INSERT INTO cities (name) VALUES ('Bilaspur');
INSERT INTO cities (name) VALUES ('Bilbao');
INSERT INTO cities (name) VALUES ('Bilbays');
INSERT INTO cities (name) VALUES ('Billings');
INSERT INTO cities (name) VALUES ('Biloxi');
INSERT INTO cities (name) VALUES ('Bilqas');
INSERT INTO cities (name) VALUES ('Binghamton');
INSERT INTO cities (name) VALUES ('Binoinano');
INSERT INTO cities (name) VALUES ('Binzhou');
INSERT INTO cities (name) VALUES ('Biratnagar');
INSERT INTO cities (name) VALUES ('Birganj');
INSERT INTO cities (name) VALUES ('Birjand');
INSERT INTO cities (name) VALUES ('Birmingham');
INSERT INTO cities (name) VALUES ('BirniNKonni');
INSERT INTO cities (name) VALUES ('Bishkek');
INSERT INTO cities (name) VALUES ('Biskara');
INSERT INTO cities (name) VALUES ('Bismarck');
INSERT INTO cities (name) VALUES ('Bismil');
INSERT INTO cities (name) VALUES ('Bissau');
INSERT INTO cities (name) VALUES ('Bitola');
INSERT INTO cities (name) VALUES ('Bitung');
INSERT INTO cities (name) VALUES ('Biysk');
INSERT INTO cities (name) VALUES ('Bizerte');
INSERT INTO cities (name) VALUES ('Blackburn');
INSERT INTO cities (name) VALUES ('Blackpool');
INSERT INTO cities (name) VALUES ('Blacksburg');
INSERT INTO cities (name) VALUES ('Blagoveshchensk');
INSERT INTO cities (name) VALUES ('Blantyre');
INSERT INTO cities (name) VALUES ('Bletchley');
INSERT INTO cities (name) VALUES ('Blida');
INSERT INTO cities (name) VALUES ('Blitar');
INSERT INTO cities (name) VALUES ('Bloemfontein');
INSERT INTO cities (name) VALUES ('Bloomington');
INSERT INTO cities (name) VALUES ('Bluefields');
INSERT INTO cities (name) VALUES ('Blumenau');
INSERT INTO cities (name) VALUES ('Bo');
INSERT INTO cities (name) VALUES ('BoaVista');
INSERT INTO cities (name) VALUES ('BoboDioulasso');
INSERT INTO cities (name) VALUES ('Boca Raton');
INSERT INTO cities (name) VALUES ('Bochum');
INSERT INTO cities (name) VALUES ('BoddenTown');
INSERT INTO cities (name) VALUES ('Bognor Regis');
INSERT INTO cities (name) VALUES ('Bogor');
INSERT INTO cities (name) VALUES ('Bogora');
INSERT INTO cities (name) VALUES ('Bogota');
INSERT INTO cities (name) VALUES ('Bohicon');
INSERT INTO cities (name) VALUES ('Boise City');
INSERT INTO cities (name) VALUES ('Bojnurd');
INSERT INTO cities (name) VALUES ('Bokaro');
INSERT INTO cities (name) VALUES ('Bolama');
INSERT INTO cities (name) VALUES ('Bolgatanga');
INSERT INTO cities (name) VALUES ('Bologna');
INSERT INTO cities (name) VALUES ('Bolton');
INSERT INTO cities (name) VALUES ('Bolzano');
INSERT INTO cities (name) VALUES ('Boma');
INSERT INTO cities (name) VALUES ('Bonao');
INSERT INTO cities (name) VALUES ('Bonn');
INSERT INTO cities (name) VALUES ('Bonriki');
INSERT INTO cities (name) VALUES ('Bontang');
INSERT INTO cities (name) VALUES ('Bordeaux');
INSERT INTO cities (name) VALUES ('BorgoMaggiore');
INSERT INTO cities (name) VALUES ('Borisov');
INSERT INTO cities (name) VALUES ('Borujerd');
INSERT INTO cities (name) VALUES ('Boshan');
INSERT INTO cities (name) VALUES ('Bossangoa');
INSERT INTO cities (name) VALUES ('Bossier City');
INSERT INTO cities (name) VALUES ('Boston');
INSERT INTO cities (name) VALUES ('Botosani');
INSERT INTO cities (name) VALUES ('Botshabelo');
INSERT INTO cities (name) VALUES ('Bottrop');
INSERT INTO cities (name) VALUES ('Botucatu');
INSERT INTO cities (name) VALUES ('Bouak');
INSERT INTO cities (name) VALUES ('Bouar');
INSERT INTO cities (name) VALUES ('Boulder');
INSERT INTO cities (name) VALUES ('Boulogne-Billancourt');
INSERT INTO cities (name) VALUES ('Bountiful');
INSERT INTO cities (name) VALUES ('Bourail');
INSERT INTO cities (name) VALUES ('Bourges');
INSERT INTO cities (name) VALUES ('Bournemouth');
INSERT INTO cities (name) VALUES ('Bowling Green');
INSERT INTO cities (name) VALUES ('Boynton Beach');
INSERT INTO cities (name) VALUES ('Bozhou');
INSERT INTO cities (name) VALUES ('Bradenton');
INSERT INTO cities (name) VALUES ('Bradford');
INSERT INTO cities (name) VALUES ('Braga');
INSERT INTO cities (name) VALUES ('BragancaPaulista');
INSERT INTO cities (name) VALUES ('Brahamanbaria');
INSERT INTO cities (name) VALUES ('Brahmapur');
INSERT INTO cities (name) VALUES ('Braila');
INSERT INTO cities (name) VALUES ('Brampton');
INSERT INTO cities (name) VALUES ('Brandon');
INSERT INTO cities (name) VALUES ('Brasilia');
INSERT INTO cities (name) VALUES ('Brasov');
INSERT INTO cities (name) VALUES ('Bratislava');
INSERT INTO cities (name) VALUES ('Bratsk');
INSERT INTO cities (name) VALUES ('Braunschweig');
INSERT INTO cities (name) VALUES ('Bray');
INSERT INTO cities (name) VALUES ('Brazzaville');
INSERT INTO cities (name) VALUES ('Breda');
INSERT INTO cities (name) VALUES ('Bremen');
INSERT INTO cities (name) VALUES ('Bremerhaven');
INSERT INTO cities (name) VALUES ('Brescia');
INSERT INTO cities (name) VALUES ('Brest');
INSERT INTO cities (name) VALUES ('Bria');
INSERT INTO cities (name) VALUES ('Bridgend');
INSERT INTO cities (name) VALUES ('Bridgeport');
INSERT INTO cities (name) VALUES ('Bridgetown');
INSERT INTO cities (name) VALUES ('Bridgwater');
INSERT INTO cities (name) VALUES ('Bridlington');
INSERT INTO cities (name) VALUES ('Brighton');
INSERT INTO cities (name) VALUES ('Brikama');
INSERT INTO cities (name) VALUES ('Brindisi');
INSERT INTO cities (name) VALUES ('Brisbane');
INSERT INTO cities (name) VALUES ('Bristol');
INSERT INTO cities (name) VALUES ('Brno');
INSERT INTO cities (name) VALUES ('Brownsville');
INSERT INTO cities (name) VALUES ('Brugges');
INSERT INTO cities (name) VALUES ('Brussels');
INSERT INTO cities (name) VALUES ('Bryan');
INSERT INTO cities (name) VALUES ('Bryansk');
INSERT INTO cities (name) VALUES ('Buariki');
INSERT INTO cities (name) VALUES ('Bucaramanga');
INSERT INTO cities (name) VALUES ('Buchanan');
INSERT INTO cities (name) VALUES ('Bucuresti');
INSERT INTO cities (name) VALUES ('Budapest');
INSERT INTO cities (name) VALUES ('Buena Park');
INSERT INTO cities (name) VALUES ('Buenaventura');
INSERT INTO cities (name) VALUES ('BuenosAires');
INSERT INTO cities (name) VALUES ('Buffalo');
INSERT INTO cities (name) VALUES ('Buga');
INSERT INTO cities (name) VALUES ('Bugama');
INSERT INTO cities (name) VALUES ('Bugulma');
INSERT INTO cities (name) VALUES ('Buhe');
INSERT INTO cities (name) VALUES ('Bujumbura');
INSERT INTO cities (name) VALUES ('Bukan');
INSERT INTO cities (name) VALUES ('Bukavu');
INSERT INTO cities (name) VALUES ('Bukhara');
INSERT INTO cities (name) VALUES ('Bulawayo');
INSERT INTO cities (name) VALUES ('BungerOasis');
INSERT INTO cities (name) VALUES ('Bunia');
INSERT INTO cities (name) VALUES ('BuonMeThuot');
INSERT INTO cities (name) VALUES ('Buraydah');
INSERT INTO cities (name) VALUES ('Buraymi');
INSERT INTO cities (name) VALUES ('Burbank');
INSERT INTO cities (name) VALUES ('Burewala');
INSERT INTO cities (name) VALUES ('Burgas');
INSERT INTO cities (name) VALUES ('Burgess Hill');
INSERT INTO cities (name) VALUES ('Burgos');
INSERT INTO cities (name) VALUES ('Burlington');
INSERT INTO cities (name) VALUES ('Burnaby');
INSERT INTO cities (name) VALUES ('Burnley');
INSERT INTO cities (name) VALUES ('Bursa');
INSERT INTO cities (name) VALUES ('Burton-upon-Trent');
INSERT INTO cities (name) VALUES ('Bury Saint Edmunds');
INSERT INTO cities (name) VALUES ('Bushehr');
INSERT INTO cities (name) VALUES ('Butare');
INSERT INTO cities (name) VALUES ('Butaritari');
INSERT INTO cities (name) VALUES ('Butembo');
INSERT INTO cities (name) VALUES ('Butuan');
INSERT INTO cities (name) VALUES ('Buzau');
INSERT INTO cities (name) VALUES ('Bua');
INSERT INTO cities (name) VALUES ('Bydgoszcz');
INSERT INTO cities (name) VALUES ('Byera');
INSERT INTO cities (name) VALUES ('Caaguazu');
INSERT INTO cities (name) VALUES ('Cabanatuan');
INSERT INTO cities (name) VALUES ('Cabimas');
INSERT INTO cities (name) VALUES ('Cabinda');
INSERT INTO cities (name) VALUES ('CabodeSantoAgostinho');
INSERT INTO cities (name) VALUES ('CaboFrio');
INSERT INTO cities (name) VALUES ('Cachoeirinha');
INSERT INTO cities (name) VALUES ('CachoeirodeItapemirim');
INSERT INTO cities (name) VALUES ('Cadiz');
INSERT INTO cities (name) VALUES ('Caen');
INSERT INTO cities (name) VALUES ('Caerphily');
INSERT INTO cities (name) VALUES ('Cagayan');
INSERT INTO cities (name) VALUES ('Cagliari');
INSERT INTO cities (name) VALUES ('Cairns');
INSERT INTO cities (name) VALUES ('Cairo');
INSERT INTO cities (name) VALUES ('Cajamarca');
INSERT INTO cities (name) VALUES ('Calabar');
INSERT INTO cities (name) VALUES ('Calabozo');
INSERT INTO cities (name) VALUES ('Calais');
INSERT INTO cities (name) VALUES ('Calama');
INSERT INTO cities (name) VALUES ('Calamba');
INSERT INTO cities (name) VALUES ('Calbayog');
INSERT INTO cities (name) VALUES ('Calcutta');
INSERT INTO cities (name) VALUES ('Calexico');
INSERT INTO cities (name) VALUES ('Calgary');
INSERT INTO cities (name) VALUES ('Cali');
INSERT INTO cities (name) VALUES ('Camacari');
INSERT INTO cities (name) VALUES ('Camaguey');
INSERT INTO cities (name) VALUES ('Camaragibe');
INSERT INTO cities (name) VALUES ('Camarillo');
INSERT INTO cities (name) VALUES ('Cambourne-Redruth');
INSERT INTO cities (name) VALUES ('Cambridge');
INSERT INTO cities (name) VALUES ('Camden');
INSERT INTO cities (name) VALUES ('Campeche');
INSERT INTO cities (name) VALUES ('Campinas');
INSERT INTO cities (name) VALUES ('CampinaGrande');
INSERT INTO cities (name) VALUES ('Campos');
INSERT INTO cities (name) VALUES ('CampoGrande');
INSERT INTO cities (name) VALUES ('CamPha');
INSERT INTO cities (name) VALUES ('CamRanh');
INSERT INTO cities (name) VALUES ('Canberra');
INSERT INTO cities (name) VALUES ('Cancun');
INSERT INTO cities (name) VALUES ('Cangzhou');
INSERT INTO cities (name) VALUES ('Cannes');
INSERT INTO cities (name) VALUES ('Canoas');
INSERT INTO cities (name) VALUES ('Canterbury');
INSERT INTO cities (name) VALUES ('Canton');
INSERT INTO cities (name) VALUES ('CanTho');
INSERT INTO cities (name) VALUES ('Cap-Haitien');
INSERT INTO cities (name) VALUES ('Cape Breton');
INSERT INTO cities (name) VALUES ('Cape Coral');
INSERT INTO cities (name) VALUES ('Cape Town');
INSERT INTO cities (name) VALUES ('Capesterre-Belle-Eau');
INSERT INTO cities (name) VALUES ('Caracas');
INSERT INTO cities (name) VALUES ('Carapicuiba');
INSERT INTO cities (name) VALUES ('Cardenas');
INSERT INTO cities (name) VALUES ('Cardiff');
INSERT INTO cities (name) VALUES ('Cariacica');
INSERT INTO cities (name) VALUES ('Carlisle');
INSERT INTO cities (name) VALUES ('Carlsbad');
INSERT INTO cities (name) VALUES ('Carltonville');
INSERT INTO cities (name) VALUES ('Carnot');
INSERT INTO cities (name) VALUES ('Carolina');
INSERT INTO cities (name) VALUES ('Carora');
INSERT INTO cities (name) VALUES ('Carrollton');
INSERT INTO cities (name) VALUES ('Carson');
INSERT INTO cities (name) VALUES ('Carson City');
INSERT INTO cities (name) VALUES ('Cartagena');
INSERT INTO cities (name) VALUES ('Cartago');
INSERT INTO cities (name) VALUES ('Caruaru');
INSERT INTO cities (name) VALUES ('Carupano');
INSERT INTO cities (name) VALUES ('Cary');
INSERT INTO cities (name) VALUES ('Casablanca');
INSERT INTO cities (name) VALUES ('Casas Adobes');
INSERT INTO cities (name) VALUES ('Cascavel');
INSERT INTO cities (name) VALUES ('Casey');
INSERT INTO cities (name) VALUES ('Casper');
INSERT INTO cities (name) VALUES ('Castanhal');
INSERT INTO cities (name) VALUES ('Castello');
INSERT INTO cities (name) VALUES ('Castletown');
INSERT INTO cities (name) VALUES ('Castries');
INSERT INTO cities (name) VALUES ('Catalina Foothills');
INSERT INTO cities (name) VALUES ('Catamarca');
INSERT INTO cities (name) VALUES ('Catanduva');
INSERT INTO cities (name) VALUES ('Catania');
INSERT INTO cities (name) VALUES ('Catanzaro');
INSERT INTO cities (name) VALUES ('Caucaia');
INSERT INTO cities (name) VALUES ('Caxias');
INSERT INTO cities (name) VALUES ('CaxiasdoSul');
INSERT INTO cities (name) VALUES ('Cayenne');
INSERT INTO cities (name) VALUES ('Cayey');
INSERT INTO cities (name) VALUES ('Cebu');
INSERT INTO cities (name) VALUES ('Cedar Rapids');
INSERT INTO cities (name) VALUES ('Celaya');
INSERT INTO cities (name) VALUES ('Cerritos');
INSERT INTO cities (name) VALUES ('Cesena');
INSERT INTO cities (name) VALUES ('Ceske Budejovice');
INSERT INTO cities (name) VALUES ('Ceyhan');
INSERT INTO cities (name) VALUES ('Chacabuco');
INSERT INTO cities (name) VALUES ('Chachak');
INSERT INTO cities (name) VALUES ('ChalanKanoa');
INSERT INTO cities (name) VALUES ('Chambery');
INSERT INTO cities (name) VALUES ('Champaign');
INSERT INTO cities (name) VALUES ('Chandigarh');
INSERT INTO cities (name) VALUES ('Chandler');
INSERT INTO cities (name) VALUES ('Chandrapur');
INSERT INTO cities (name) VALUES ('Changchun');
INSERT INTO cities (name) VALUES ('Changde');
INSERT INTO cities (name) VALUES ('Changhwa');
INSERT INTO cities (name) VALUES ('Changji');
INSERT INTO cities (name) VALUES ('Changsha');
INSERT INTO cities (name) VALUES ('Changweon');
INSERT INTO cities (name) VALUES ('Changzhi');
INSERT INTO cities (name) VALUES ('Changzhou');
INSERT INTO cities (name) VALUES ('Chaohu');
INSERT INTO cities (name) VALUES ('Chaoyang');
INSERT INTO cities (name) VALUES ('Chaozhou');
INSERT INTO cities (name) VALUES ('Chapeco');
INSERT INTO cities (name) VALUES ('Chapel Hill');
INSERT INTO cities (name) VALUES ('Charikar');
INSERT INTO cities (name) VALUES ('Charleroi');
INSERT INTO cities (name) VALUES ('Charlesbourg');
INSERT INTO cities (name) VALUES ('Charleston');
INSERT INTO cities (name) VALUES ('Charlestown');
INSERT INTO cities (name) VALUES ('Charlotte');
INSERT INTO cities (name) VALUES ('Charlottesville');
INSERT INTO cities (name) VALUES ('Charlottetown');
INSERT INTO cities (name) VALUES ('CharlotteAmalie');
INSERT INTO cities (name) VALUES ('Chattanooga');
INSERT INTO cities (name) VALUES ('Chavakachcheri');
INSERT INTO cities (name) VALUES ('Cheboksary');
INSERT INTO cities (name) VALUES ('Chechon');
INSERT INTO cities (name) VALUES ('Cheju');
INSERT INTO cities (name) VALUES ('Chelmsford');
INSERT INTO cities (name) VALUES ('Cheltenham');
INSERT INTO cities (name) VALUES ('Chelyabinsk');
INSERT INTO cities (name) VALUES ('Chemnitz');
INSERT INTO cities (name) VALUES ('Chengde');
INSERT INTO cities (name) VALUES ('Chengdu');
INSERT INTO cities (name) VALUES ('Chenghai');
INSERT INTO cities (name) VALUES ('Chennai (Madras)');
INSERT INTO cities (name) VALUES ('Chenzhou');
INSERT INTO cities (name) VALUES ('Cheonan');
INSERT INTO cities (name) VALUES ('Cheongju');
INSERT INTO cities (name) VALUES ('Cheonju');
INSERT INTO cities (name) VALUES ('Cherepovec');
INSERT INTO cities (name) VALUES ('Cherkasy');
INSERT INTO cities (name) VALUES ('Cherkessk');
INSERT INTO cities (name) VALUES ('Chernihiv');
INSERT INTO cities (name) VALUES ('Chernivtsi');
INSERT INTO cities (name) VALUES ('Chesapeake');
INSERT INTO cities (name) VALUES ('Chester');
INSERT INTO cities (name) VALUES ('Chetumal');
INSERT INTO cities (name) VALUES ('Cheyenne');
INSERT INTO cities (name) VALUES ('ChiangMai');
INSERT INTO cities (name) VALUES ('Chiayi');
INSERT INTO cities (name) VALUES ('Chiba');
INSERT INTO cities (name) VALUES ('Chicago');
INSERT INTO cities (name) VALUES ('Chichester');
INSERT INTO cities (name) VALUES ('Chiclayo');
INSERT INTO cities (name) VALUES ('Chico');
INSERT INTO cities (name) VALUES ('Chicopee');
INSERT INTO cities (name) VALUES ('Chicoutimi');
INSERT INTO cities (name) VALUES ('Chifeng');
INSERT INTO cities (name) VALUES ('Chihuahua');
INSERT INTO cities (name) VALUES ('Chilln');
INSERT INTO cities (name) VALUES ('Chilpancingo');
INSERT INTO cities (name) VALUES ('Chimbote');
INSERT INTO cities (name) VALUES ('Chimkent');
INSERT INTO cities (name) VALUES ('Chimoio');
INSERT INTO cities (name) VALUES ('Chinandega');
INSERT INTO cities (name) VALUES ('ChinchaAlta');
INSERT INTO cities (name) VALUES ('Chingola');
INSERT INTO cities (name) VALUES ('Chinhoyi');
INSERT INTO cities (name) VALUES ('Chiniot');
INSERT INTO cities (name) VALUES ('Chinju');
INSERT INTO cities (name) VALUES ('Chino');
INSERT INTO cities (name) VALUES ('Chino Hills');
INSERT INTO cities (name) VALUES ('Chipata');
INSERT INTO cities (name) VALUES ('Chippenham');
INSERT INTO cities (name) VALUES ('Chishtian Mandi');
INSERT INTO cities (name) VALUES ('Chisinau');
INSERT INTO cities (name) VALUES ('Chita');
INSERT INTO cities (name) VALUES ('Chittagong');
INSERT INTO cities (name) VALUES ('Chizhou');
INSERT INTO cities (name) VALUES ('Chol');
INSERT INTO cities (name) VALUES ('Cholet');
INSERT INTO cities (name) VALUES ('Choluteca');
INSERT INTO cities (name) VALUES ('Chongjin');
INSERT INTO cities (name) VALUES ('Chongqing');
INSERT INTO cities (name) VALUES ('ChonBuri');
INSERT INTO cities (name) VALUES ('Chorley');
INSERT INTO cities (name) VALUES ('Christchurch');
INSERT INTO cities (name) VALUES ('Christiansted');
INSERT INTO cities (name) VALUES ('Chula Vista');
INSERT INTO cities (name) VALUES ('Chuncheng');
INSERT INTO cities (name) VALUES ('Chuncheon');
INSERT INTO cities (name) VALUES ('Chungho');
INSERT INTO cities (name) VALUES ('Chungju');
INSERT INTO cities (name) VALUES ('Chungli');
INSERT INTO cities (name) VALUES ('Chuzhou');
INSERT INTO cities (name) VALUES ('Cianjur');
INSERT INTO cities (name) VALUES ('Cibadak');
INSERT INTO cities (name) VALUES ('Cicero');
INSERT INTO cities (name) VALUES ('CiegodeAvila');
INSERT INTO cities (name) VALUES ('Cienfuegos');
INSERT INTO cities (name) VALUES ('Cikampek');
INSERT INTO cities (name) VALUES ('Cilacap');
INSERT INTO cities (name) VALUES ('Cilegon');
INSERT INTO cities (name) VALUES ('Cincinnati');
INSERT INTO cities (name) VALUES ('Cirebon');
INSERT INTO cities (name) VALUES ('Ciudad Bolivar');
INSERT INTO cities (name) VALUES ('Ciudad del Carmen');
INSERT INTO cities (name) VALUES ('Ciudad Guayana');
INSERT INTO cities (name) VALUES ('Ciudad Jurez');
INSERT INTO cities (name) VALUES ('Ciudad Obregon');
INSERT INTO cities (name) VALUES ('Ciudad Valles');
INSERT INTO cities (name) VALUES ('Ciudad Victoria');
INSERT INTO cities (name) VALUES ('CiudaddelEste');
INSERT INTO cities (name) VALUES ('Clacton-on-Sea');
INSERT INTO cities (name) VALUES ('Clarksville');
INSERT INTO cities (name) VALUES ('Clay Cross-North Wingfield');
INSERT INTO cities (name) VALUES ('Clearwater');
INSERT INTO cities (name) VALUES ('Cleethorpes');
INSERT INTO cities (name) VALUES ('Clermont-Ferrand');
INSERT INTO cities (name) VALUES ('Cleveland');
INSERT INTO cities (name) VALUES ('Clifton');
INSERT INTO cities (name) VALUES ('Clinton');
INSERT INTO cities (name) VALUES ('Cluj-Napoca');
INSERT INTO cities (name) VALUES ('Coalville');
INSERT INTO cities (name) VALUES ('Coatzacoalcos');
INSERT INTO cities (name) VALUES ('Cochabamba');
INSERT INTO cities (name) VALUES ('CockburnHarbour');
INSERT INTO cities (name) VALUES ('CockburnTown');
INSERT INTO cities (name) VALUES ('Coimbra');
INSERT INTO cities (name) VALUES ('Cojbalsan');
INSERT INTO cities (name) VALUES ('Colchester');
INSERT INTO cities (name) VALUES ('Coleraine');
INSERT INTO cities (name) VALUES ('Colima');
INSERT INTO cities (name) VALUES ('College Station');
INSERT INTO cities (name) VALUES ('Colmar');
INSERT INTO cities (name) VALUES ('Colombo');
INSERT INTO cities (name) VALUES ('Colon');
INSERT INTO cities (name) VALUES ('Colonia');
INSERT INTO cities (name) VALUES ('Colorado Springs');
INSERT INTO cities (name) VALUES ('Columbia');
INSERT INTO cities (name) VALUES ('Columbus');
INSERT INTO cities (name) VALUES ('Colwyn Bay');
INSERT INTO cities (name) VALUES ('ComodoroRivadavia');
INSERT INTO cities (name) VALUES ('Compton');
INSERT INTO cities (name) VALUES ('Conakry');
INSERT INTO cities (name) VALUES ('Concepcin');
INSERT INTO cities (name) VALUES ('Concord');
INSERT INTO cities (name) VALUES ('Concordia');
INSERT INTO cities (name) VALUES ('Congleton');
INSERT INTO cities (name) VALUES ('ConselheiroLafaiete');
INSERT INTO cities (name) VALUES ('Constanta');
INSERT INTO cities (name) VALUES ('Constantine');
INSERT INTO cities (name) VALUES ('Contagem');
INSERT INTO cities (name) VALUES ('Copenhagen');
INSERT INTO cities (name) VALUES ('Copiap');
INSERT INTO cities (name) VALUES ('Coquimbo');
INSERT INTO cities (name) VALUES ('Coquitlam');
INSERT INTO cities (name) VALUES ('Coral Springs');
INSERT INTO cities (name) VALUES ('Corby');
INSERT INTO cities (name) VALUES ('Cordoba');
INSERT INTO cities (name) VALUES ('Cork');
INSERT INTO cities (name) VALUES ('Corlu');
INSERT INTO cities (name) VALUES ('Coro');
INSERT INTO cities (name) VALUES ('Corona');
INSERT INTO cities (name) VALUES ('CoronelOviedo');
INSERT INTO cities (name) VALUES ('Corpus Christi');
INSERT INTO cities (name) VALUES ('Corrientes');
INSERT INTO cities (name) VALUES ('Corriverton');
INSERT INTO cities (name) VALUES ('Corum');
INSERT INTO cities (name) VALUES ('Corvallis');
INSERT INTO cities (name) VALUES ('Costa Mesa');
INSERT INTO cities (name) VALUES ('Cotabato');
INSERT INTO cities (name) VALUES ('Cotia');
INSERT INTO cities (name) VALUES ('Cotonou');
INSERT INTO cities (name) VALUES ('Cottbus');
INSERT INTO cities (name) VALUES ('Council Bluffs');
INSERT INTO cities (name) VALUES ('Coventry');
INSERT INTO cities (name) VALUES ('Craiova');
INSERT INTO cities (name) VALUES ('Crewe');
INSERT INTO cities (name) VALUES ('Criciuma');
INSERT INTO cities (name) VALUES ('CruzBay');
INSERT INTO cities (name) VALUES ('Cua');
INSERT INTO cities (name) VALUES ('Cuautla');
INSERT INTO cities (name) VALUES ('Cubatao');
INSERT INTO cities (name) VALUES ('Cucuta');
INSERT INTO cities (name) VALUES ('Cuenca');
INSERT INTO cities (name) VALUES ('Cuernavaca');
INSERT INTO cities (name) VALUES ('Cuiaba');
INSERT INTO cities (name) VALUES ('Culiacan');
INSERT INTO cities (name) VALUES ('Cumana');
INSERT INTO cities (name) VALUES ('Curic');
INSERT INTO cities (name) VALUES ('Curitiba');
INSERT INTO cities (name) VALUES ('Cusco');
INSERT INTO cities (name) VALUES ('Cwmbran');
INSERT INTO cities (name) VALUES ('Czestochowa');
INSERT INTO cities (name) VALUES ('Daan');
INSERT INTO cities (name) VALUES ('Dabou');
INSERT INTO cities (name) VALUES ('Dachang');
INSERT INTO cities (name) VALUES ('Dadu');
INSERT INTO cities (name) VALUES ('Dagupan');
INSERT INTO cities (name) VALUES ('Dakar');
INSERT INTO cities (name) VALUES ('Dakhla');
INSERT INTO cities (name) VALUES ('Dakoro');
INSERT INTO cities (name) VALUES ('Dali');
INSERT INTO cities (name) VALUES ('Dalian');
INSERT INTO cities (name) VALUES ('Daliang');
INSERT INTO cities (name) VALUES ('Dallas');
INSERT INTO cities (name) VALUES ('Daloa');
INSERT INTO cities (name) VALUES ('Daly City');
INSERT INTO cities (name) VALUES ('Damanhur');
INSERT INTO cities (name) VALUES ('Damascus');
INSERT INTO cities (name) VALUES ('Damaturu');
INSERT INTO cities (name) VALUES ('Danang');
INSERT INTO cities (name) VALUES ('Danao');
INSERT INTO cities (name) VALUES ('Dandong');
INSERT INTO cities (name) VALUES ('Dangriga');
INSERT INTO cities (name) VALUES ('Danville');
INSERT INTO cities (name) VALUES ('Dapaong');
INSERT INTO cities (name) VALUES ('Dara');
INSERT INTO cities (name) VALUES ('Darbhanga');
INSERT INTO cities (name) VALUES ('Dare');
INSERT INTO cities (name) VALUES ('Darhan');
INSERT INTO cities (name) VALUES ('Darlington');
INSERT INTO cities (name) VALUES ('Darmstadt');
INSERT INTO cities (name) VALUES ('Darnah');
INSERT INTO cities (name) VALUES ('Darwen');
INSERT INTO cities (name) VALUES ('DaresSalaam');
INSERT INTO cities (name) VALUES ('Dashkovuz');
INSERT INTO cities (name) VALUES ('Daska');
INSERT INTO cities (name) VALUES ('Datong');
INSERT INTO cities (name) VALUES ('Daugavpils');
INSERT INTO cities (name) VALUES ('Davanagere');
INSERT INTO cities (name) VALUES ('Davao');
INSERT INTO cities (name) VALUES ('Davenport');
INSERT INTO cities (name) VALUES ('David');
INSERT INTO cities (name) VALUES ('Davie');
INSERT INTO cities (name) VALUES ('Davis');
INSERT INTO cities (name) VALUES ('Dawei');
INSERT INTO cities (name) VALUES ('Dawukou');
INSERT INTO cities (name) VALUES ('Daxian');
INSERT INTO cities (name) VALUES ('Dayraz-Zawr');
INSERT INTO cities (name) VALUES ('Dayton');
INSERT INTO cities (name) VALUES ('Daytona Beach');
INSERT INTO cities (name) VALUES ('DaLat');
INSERT INTO cities (name) VALUES ('Deal');
INSERT INTO cities (name) VALUES ('Dearborn');
INSERT INTO cities (name) VALUES ('Debrecen');
INSERT INTO cities (name) VALUES ('Decatur');
INSERT INTO cities (name) VALUES ('Dehra Dun');
INSERT INTO cities (name) VALUES ('Dehui');
INSERT INTO cities (name) VALUES ('Delano');
INSERT INTO cities (name) VALUES ('Delft');
INSERT INTO cities (name) VALUES ('Delhi');
INSERT INTO cities (name) VALUES ('Delicias');
INSERT INTO cities (name) VALUES ('Delray Beach');
INSERT INTO cities (name) VALUES ('Deltona');
INSERT INTO cities (name) VALUES ('Denizli');
INSERT INTO cities (name) VALUES ('Denpasar');
INSERT INTO cities (name) VALUES ('Denton');
INSERT INTO cities (name) VALUES ('Denver');
INSERT INTO cities (name) VALUES ('Depok');
INSERT INTO cities (name) VALUES ('Dera Ghazi Khan');
INSERT INTO cities (name) VALUES ('Dera Ismail Khan');
INSERT INTO cities (name) VALUES ('Derbent');
INSERT INTO cities (name) VALUES ('Derby');
INSERT INTO cities (name) VALUES ('Des Moines');
INSERT INTO cities (name) VALUES ('Dese');
INSERT INTO cities (name) VALUES ('Detroit');
INSERT INTO cities (name) VALUES ('Dewas');
INSERT INTO cities (name) VALUES ('Deyang');
INSERT INTO cities (name) VALUES ('Dezful');
INSERT INTO cities (name) VALUES ('Dezhou');
INSERT INTO cities (name) VALUES ('Dhaka');
INSERT INTO cities (name) VALUES ('Dhamar');
INSERT INTO cities (name) VALUES ('Dhanbad');
INSERT INTO cities (name) VALUES ('Dharan');
INSERT INTO cities (name) VALUES ('Dhidhdhoo');
INSERT INTO cities (name) VALUES ('Dhule');
INSERT INTO cities (name) VALUES ('Diadema');
INSERT INTO cities (name) VALUES ('Diamond Bar');
INSERT INTO cities (name) VALUES ('Didao');
INSERT INTO cities (name) VALUES ('Dijon');
INSERT INTO cities (name) VALUES ('Dili');
INSERT INTO cities (name) VALUES ('Dimitrovgrad');
INSERT INTO cities (name) VALUES ('Dinajpur');
INSERT INTO cities (name) VALUES ('Dindigul');
INSERT INTO cities (name) VALUES ('Dingzhou');
INSERT INTO cities (name) VALUES ('Diourbel');
INSERT INTO cities (name) VALUES ('Dire Dawa');
INSERT INTO cities (name) VALUES ('Disuq');
INSERT INTO cities (name) VALUES ('Divinopolis');
INSERT INTO cities (name) VALUES ('Divo');
INSERT INTO cities (name) VALUES ('Diwaniyah');
INSERT INTO cities (name) VALUES ('Diyarbakir');
INSERT INTO cities (name) VALUES ('Djelfa');
INSERT INTO cities (name) VALUES ('Djenne');
INSERT INTO cities (name) VALUES ('Djibouti');
INSERT INTO cities (name) VALUES ('Djougou');
INSERT INTO cities (name) VALUES ('Dniprodzerzhynsk');
INSERT INTO cities (name) VALUES ('Dnipropetrovsk');
INSERT INTO cities (name) VALUES ('Dobric');
INSERT INTO cities (name) VALUES ('Dodoma');
INSERT INTO cities (name) VALUES ('Dogondoutchi');
INSERT INTO cities (name) VALUES ('Doha');
INSERT INTO cities (name) VALUES ('DomeFuji');
INSERT INTO cities (name) VALUES ('Donetsk');
INSERT INTO cities (name) VALUES ('Dongchang');
INSERT INTO cities (name) VALUES ('Dongguan');
INSERT INTO cities (name) VALUES ('Donghai');
INSERT INTO cities (name) VALUES ('Donghe');
INSERT INTO cities (name) VALUES ('Dongling');
INSERT INTO cities (name) VALUES ('Dongosaru');
INSERT INTO cities (name) VALUES ('Dongtai');
INSERT INTO cities (name) VALUES ('Dongying');
INSERT INTO cities (name) VALUES ('Donostia');
INSERT INTO cities (name) VALUES ('Dordrecht');
INSERT INTO cities (name) VALUES ('Dornbirn');
INSERT INTO cities (name) VALUES ('Dortmund');
INSERT INTO cities (name) VALUES ('Dosso');
INSERT INTO cities (name) VALUES ('DosQuebradas');
INSERT INTO cities (name) VALUES ('Dothan');
INSERT INTO cities (name) VALUES ('Douala');
INSERT INTO cities (name) VALUES ('Douglas');
INSERT INTO cities (name) VALUES ('Dourados');
INSERT INTO cities (name) VALUES ('Dover');
INSERT INTO cities (name) VALUES ('Downey');
INSERT INTO cities (name) VALUES ('Drammen');
INSERT INTO cities (name) VALUES ('Draper');
INSERT INTO cities (name) VALUES ('Dresden');
INSERT INTO cities (name) VALUES ('Drogheda');
INSERT INTO cities (name) VALUES ('Dubai');
INSERT INTO cities (name) VALUES ('Dublin');
INSERT INTO cities (name) VALUES ('Dubrovnik');
INSERT INTO cities (name) VALUES ('Dubuque');
INSERT INTO cities (name) VALUES ('Duisburg');
INSERT INTO cities (name) VALUES ('Duluth');
INSERT INTO cities (name) VALUES ('Dumaguete');
INSERT INTO cities (name) VALUES ('Dumai');
INSERT INTO cities (name) VALUES ('Dumfries');
INSERT INTO cities (name) VALUES ('Dumontd\'Urville');
INSERT INTO cities (name) VALUES ('Dundalk');
INSERT INTO cities (name) VALUES ('Dundee');
INSERT INTO cities (name) VALUES ('Dunedin');
INSERT INTO cities (name) VALUES ('Dunfermline');
INSERT INTO cities (name) VALUES ('Dunhua');
INSERT INTO cities (name) VALUES ('Dunkerque');
INSERT INTO cities (name) VALUES ('DuquedeCaxias');
INSERT INTO cities (name) VALUES ('Durango');
INSERT INTO cities (name) VALUES ('Durban');
INSERT INTO cities (name) VALUES ('Durgapur');
INSERT INTO cities (name) VALUES ('Durham');
INSERT INTO cities (name) VALUES ('Durres');
INSERT INTO cities (name) VALUES ('Dushanbe');
INSERT INTO cities (name) VALUES ('Dusseldorf');
INSERT INTO cities (name) VALUES ('Dzaoudzi');
INSERT INTO cities (name) VALUES ('Dzerzhinsk');
INSERT INTO cities (name) VALUES ('Eagan');
INSERT INTO cities (name) VALUES ('East End-Long Look');
INSERT INTO cities (name) VALUES ('East Lansing');
INSERT INTO cities (name) VALUES ('East London');
INSERT INTO cities (name) VALUES ('East Los Angeles');
INSERT INTO cities (name) VALUES ('Eastbourne');
INSERT INTO cities (name) VALUES ('Eastleigh');
INSERT INTO cities (name) VALUES ('Eaton Socon-Saint Neots');
INSERT INTO cities (name) VALUES ('Eau Claire');
INSERT INTO cities (name) VALUES ('Ebeye');
INSERT INTO cities (name) VALUES ('Ecatepec');
INSERT INTO cities (name) VALUES ('Ed Damazin');
INSERT INTO cities (name) VALUES ('Ede');
INSERT INTO cities (name) VALUES ('Eden Prairie');
INSERT INTO cities (name) VALUES ('Edfu');
INSERT INTO cities (name) VALUES ('Edinburgh');
INSERT INTO cities (name) VALUES ('Edirne');
INSERT INTO cities (name) VALUES ('Edison');
INSERT INTO cities (name) VALUES ('Edmonton');
INSERT INTO cities (name) VALUES ('Eda');
INSERT INTO cities (name) VALUES ('Eindhoven');
INSERT INTO cities (name) VALUES ('Ekibastuz');
INSERT INTO cities (name) VALUES ('El Araich');
INSERT INTO cities (name) VALUES ('El Arish');
INSERT INTO cities (name) VALUES ('El Cajon');
INSERT INTO cities (name) VALUES ('El Geneina');
INSERT INTO cities (name) VALUES ('El Jadida');
INSERT INTO cities (name) VALUES ('El Mahalla el Kubra');
INSERT INTO cities (name) VALUES ('El Managel');
INSERT INTO cities (name) VALUES ('El Minya');
INSERT INTO cities (name) VALUES ('El Monte');
INSERT INTO cities (name) VALUES ('El Obeid');
INSERT INTO cities (name) VALUES ('El Paso');
INSERT INTO cities (name) VALUES ('El Tigre');
INSERT INTO cities (name) VALUES ('Elat');
INSERT INTO cities (name) VALUES ('Elazig');
INSERT INTO cities (name) VALUES ('ElaNguema');
INSERT INTO cities (name) VALUES ('Elbasan');
INSERT INTO cities (name) VALUES ('Elblag');
INSERT INTO cities (name) VALUES ('Eldoret');
INSERT INTO cities (name) VALUES ('Elektrostal');
INSERT INTO cities (name) VALUES ('Elgin');
INSERT INTO cities (name) VALUES ('Elista');
INSERT INTO cities (name) VALUES ('Elizabeth');
INSERT INTO cities (name) VALUES ('Elkhart');
INSERT INTO cities (name) VALUES ('Elx');
INSERT INTO cities (name) VALUES ('ElProgreso');
INSERT INTO cities (name) VALUES ('Embu');
INSERT INTO cities (name) VALUES ('Emmen');
INSERT INTO cities (name) VALUES ('Encarnacion');
INSERT INTO cities (name) VALUES ('Encinitas');
INSERT INTO cities (name) VALUES ('Engels');
INSERT INTO cities (name) VALUES ('EnglishHarbour');
INSERT INTO cities (name) VALUES ('Enid');
INSERT INTO cities (name) VALUES ('Enschede');
INSERT INTO cities (name) VALUES ('Ensenada');
INSERT INTO cities (name) VALUES ('Entebbe');
INSERT INTO cities (name) VALUES ('Enugu');
INSERT INTO cities (name) VALUES ('Envigado');
INSERT INTO cities (name) VALUES ('Eqalugaarsuit');
INSERT INTO cities (name) VALUES ('Erdenet');
INSERT INTO cities (name) VALUES ('Erfurt');
INSERT INTO cities (name) VALUES ('Erie');
INSERT INTO cities (name) VALUES ('Erlangen');
INSERT INTO cities (name) VALUES ('Erzincan');
INSERT INTO cities (name) VALUES ('Erzurum');
INSERT INTO cities (name) VALUES ('Esbjerg');
INSERT INTO cities (name) VALUES ('Eschen');
INSERT INTO cities (name) VALUES ('Escondido');
INSERT INTO cities (name) VALUES ('Escuintla');
INSERT INTO cities (name) VALUES ('Esfahan');
INSERT INTO cities (name) VALUES ('Eskisehir');
INSERT INTO cities (name) VALUES ('Esmeraldas');
INSERT INTO cities (name) VALUES ('Espoo');
INSERT INTO cities (name) VALUES ('Essen');
INSERT INTO cities (name) VALUES ('Esteli');
INSERT INTO cities (name) VALUES ('Etawah');
INSERT INTO cities (name) VALUES ('Eugene');
INSERT INTO cities (name) VALUES ('Euijeongbu');
INSERT INTO cities (name) VALUES ('Evansville');
INSERT INTO cities (name) VALUES ('Everett');
INSERT INTO cities (name) VALUES ('Evreux');
INSERT INTO cities (name) VALUES ('Exeter');
INSERT INTO cities (name) VALUES ('Exmouth');
INSERT INTO cities (name) VALUES ('Eydhafushi');
INSERT INTO cities (name) VALUES ('Ezhou');
INSERT INTO cities (name) VALUES ('Faaa');
INSERT INTO cities (name) VALUES ('Fairfield');
INSERT INTO cities (name) VALUES ('Faisalabad');
INSERT INTO cities (name) VALUES ('Faiyum');
INSERT INTO cities (name) VALUES ('Fakaofo');
INSERT INTO cities (name) VALUES ('Falkirk');
INSERT INTO cities (name) VALUES ('Fallujah');
INSERT INTO cities (name) VALUES ('Fangaua');
INSERT INTO cities (name) VALUES ('Farafenni');
INSERT INTO cities (name) VALUES ('Fareham');
INSERT INTO cities (name) VALUES ('Farghona');
INSERT INTO cities (name) VALUES ('Fargo');
INSERT INTO cities (name) VALUES ('Farmington Hills');
INSERT INTO cities (name) VALUES ('Farrukhabad');
INSERT INTO cities (name) VALUES ('Fashir');
INSERT INTO cities (name) VALUES ('Fayaoue');
INSERT INTO cities (name) VALUES ('Fayetteville');
INSERT INTO cities (name) VALUES ('FeiradeSantana');
INSERT INTO cities (name) VALUES ('Felixtowe');
INSERT INTO cities (name) VALUES ('Fengcheng');
INSERT INTO cities (name) VALUES ('Fengshan');
INSERT INTO cities (name) VALUES ('Fengyuan');
INSERT INTO cities (name) VALUES ('Ferndown');
INSERT INTO cities (name) VALUES ('Ferrara');
INSERT INTO cities (name) VALUES ('FerrazdeVasconcelos');
INSERT INTO cities (name) VALUES ('Feydhoo');
INSERT INTO cities (name) VALUES ('Fez');
INSERT INTO cities (name) VALUES ('Fianarantsoa');
INSERT INTO cities (name) VALUES ('Firenze (Florence)');
INSERT INTO cities (name) VALUES ('Firozabad');
INSERT INTO cities (name) VALUES ('Flagstaff');
INSERT INTO cities (name) VALUES ('Fleetwood');
INSERT INTO cities (name) VALUES ('Flint');
INSERT INTO cities (name) VALUES ('Florence-Graham');
INSERT INTO cities (name) VALUES ('Florencia');
INSERT INTO cities (name) VALUES ('Florianopolis');
INSERT INTO cities (name) VALUES ('Floridablanca');
INSERT INTO cities (name) VALUES ('Foggia');
INSERT INTO cities (name) VALUES ('Folkestone');
INSERT INTO cities (name) VALUES ('Fond du Lac');
INSERT INTO cities (name) VALUES ('Fontana');
INSERT INTO cities (name) VALUES ('Forli');
INSERT INTO cities (name) VALUES ('Formosa');
INSERT INTO cities (name) VALUES ('Fort Collins');
INSERT INTO cities (name) VALUES ('Fort Lauderdale');
INSERT INTO cities (name) VALUES ('Fort Myers');
INSERT INTO cities (name) VALUES ('Fort Smith');
INSERT INTO cities (name) VALUES ('Fort Wayne');
INSERT INTO cities (name) VALUES ('Fort Worth');
INSERT INTO cities (name) VALUES ('Fort-de-France');
INSERT INTO cities (name) VALUES ('Fortaleza');
INSERT INTO cities (name) VALUES ('Foshan');
INSERT INTO cities (name) VALUES ('Fountain Valley');
INSERT INTO cities (name) VALUES ('FozdoIguacu');
INSERT INTO cities (name) VALUES ('Franca');
INSERT INTO cities (name) VALUES ('FranciscoMorato');
INSERT INTO cities (name) VALUES ('Francistown');
INSERT INTO cities (name) VALUES ('FrancodaRocha');
INSERT INTO cities (name) VALUES ('Frankfort');
INSERT INTO cities (name) VALUES ('Frankfurt');
INSERT INTO cities (name) VALUES ('Fredericton');
INSERT INTO cities (name) VALUES ('Freeport');
INSERT INTO cities (name) VALUES ('Freetown');
INSERT INTO cities (name) VALUES ('Freiburg');
INSERT INTO cities (name) VALUES ('Fremont');
INSERT INTO cities (name) VALUES ('Fresno');
INSERT INTO cities (name) VALUES ('Fujairah');
INSERT INTO cities (name) VALUES ('Fuji');
INSERT INTO cities (name) VALUES ('Fukui');
INSERT INTO cities (name) VALUES ('Fukuoka');
INSERT INTO cities (name) VALUES ('Fukushima');
INSERT INTO cities (name) VALUES ('Fukuyama');
INSERT INTO cities (name) VALUES ('Fuling');
INSERT INTO cities (name) VALUES ('Fullerton');
INSERT INTO cities (name) VALUES ('Funabashi');
INSERT INTO cities (name) VALUES ('Funchal');
INSERT INTO cities (name) VALUES ('Funtua');
INSERT INTO cities (name) VALUES ('Furth');
INSERT INTO cities (name) VALUES ('Fushun');
INSERT INTO cities (name) VALUES ('Fuvahmulah');
INSERT INTO cities (name) VALUES ('Fuxin');
INSERT INTO cities (name) VALUES ('Fuyang');
INSERT INTO cities (name) VALUES ('Fuyu');
INSERT INTO cities (name) VALUES ('Fuzhou');
INSERT INTO cities (name) VALUES ('Gabes');
INSERT INTO cities (name) VALUES ('Gaborone');
INSERT INTO cities (name) VALUES ('Gabu');
INSERT INTO cities (name) VALUES ('Gadhdhoo');
INSERT INTO cities (name) VALUES ('Gagnoa');
INSERT INTO cities (name) VALUES ('Gainesville');
INSERT INTO cities (name) VALUES ('Galati');
INSERT INTO cities (name) VALUES ('Galle');
INSERT INTO cities (name) VALUES ('Galveston');
INSERT INTO cities (name) VALUES ('Galway');
INSERT INTO cities (name) VALUES ('Ganganagar');
INSERT INTO cities (name) VALUES ('Gangdong');
INSERT INTO cities (name) VALUES ('Ganzhou');
INSERT INTO cities (name) VALUES ('Gaomi');
INSERT INTO cities (name) VALUES ('Gaozhou');
INSERT INTO cities (name) VALUES ('Gapan');
INSERT INTO cities (name) VALUES ('Garanhuns');
INSERT INTO cities (name) VALUES ('Garapan');
INSERT INTO cities (name) VALUES ('Gardabaer');
INSERT INTO cities (name) VALUES ('Garden Grove');
INSERT INTO cities (name) VALUES ('Gardena');
INSERT INTO cities (name) VALUES ('Garland');
INSERT INTO cities (name) VALUES ('Garoua');
INSERT INTO cities (name) VALUES ('Garue');
INSERT INTO cities (name) VALUES ('Garut');
INSERT INTO cities (name) VALUES ('Gary');
INSERT INTO cities (name) VALUES ('Garyan');
INSERT INTO cities (name) VALUES ('Gashua');
INSERT INTO cities (name) VALUES ('Gatineau');
INSERT INTO cities (name) VALUES ('Gaya');
INSERT INTO cities (name) VALUES ('Gaza');
INSERT INTO cities (name) VALUES ('Gaziantep');
INSERT INTO cities (name) VALUES ('Gboko');
INSERT INTO cities (name) VALUES ('Gbongan');
INSERT INTO cities (name) VALUES ('Gdansk');
INSERT INTO cities (name) VALUES ('Gdynia');
INSERT INTO cities (name) VALUES ('Geelong');
INSERT INTO cities (name) VALUES ('Geita');
INSERT INTO cities (name) VALUES ('Gejiu');
INSERT INTO cities (name) VALUES ('Gelsenkirchen');
INSERT INTO cities (name) VALUES ('Gemena');
INSERT INTO cities (name) VALUES ('General Santos');
INSERT INTO cities (name) VALUES ('Geneva');
INSERT INTO cities (name) VALUES ('Genova');
INSERT INTO cities (name) VALUES ('George');
INSERT INTO cities (name) VALUES ('Georgetown');
INSERT INTO cities (name) VALUES ('GeorgeTown');
INSERT INTO cities (name) VALUES ('Gera');
INSERT INTO cities (name) VALUES ('Ghent');
INSERT INTO cities (name) VALUES ('Giandzha');
INSERT INTO cities (name) VALUES ('Gibraltar');
INSERT INTO cities (name) VALUES ('Gijon');
INSERT INTO cities (name) VALUES ('Gilbert');
INSERT INTO cities (name) VALUES ('Girardot');
INSERT INTO cities (name) VALUES ('Gitega');
INSERT INTO cities (name) VALUES ('Giza');
INSERT INTO cities (name) VALUES ('Gizo');
INSERT INTO cities (name) VALUES ('Glasgow');
INSERT INTO cities (name) VALUES ('Glazov');
INSERT INTO cities (name) VALUES ('Glendale');
INSERT INTO cities (name) VALUES ('Glendora');
INSERT INTO cities (name) VALUES ('Glenrothes');
INSERT INTO cities (name) VALUES ('Gliwice');
INSERT INTO cities (name) VALUES ('Gloucester');
INSERT INTO cities (name) VALUES ('Goiania');
INSERT INTO cities (name) VALUES ('Gojra');
INSERT INTO cities (name) VALUES ('Goldsboro');
INSERT INTO cities (name) VALUES ('GoldCoast');
INSERT INTO cities (name) VALUES ('Goleta');
INSERT INTO cities (name) VALUES ('Goma');
INSERT INTO cities (name) VALUES ('Gombe');
INSERT INTO cities (name) VALUES ('Gomel');
INSERT INTO cities (name) VALUES ('Gonaives');
INSERT INTO cities (name) VALUES ('Gonbad-eQabus');
INSERT INTO cities (name) VALUES ('Gondar');
INSERT INTO cities (name) VALUES ('Gongzhuling');
INSERT INTO cities (name) VALUES ('Goodlands');
INSERT INTO cities (name) VALUES ('GooseGreen');
INSERT INTO cities (name) VALUES ('Gorakhpur');
INSERT INTO cities (name) VALUES ('Gorazde');
INSERT INTO cities (name) VALUES ('Gorgan');
INSERT INTO cities (name) VALUES ('Gorontalo');
INSERT INTO cities (name) VALUES ('Gorzow Wielkopolski');
INSERT INTO cities (name) VALUES ('Gosport');
INSERT INTO cities (name) VALUES ('Goteborg');
INSERT INTO cities (name) VALUES ('Gottingen');
INSERT INTO cities (name) VALUES ('Gouda');
INSERT INTO cities (name) VALUES ('Gouyave');
INSERT INTO cities (name) VALUES ('GovernadorValadares');
INSERT INTO cities (name) VALUES ('Granada');
INSERT INTO cities (name) VALUES ('Grand Forks');
INSERT INTO cities (name) VALUES ('Grand Island');
INSERT INTO cities (name) VALUES ('Grand Junction');
INSERT INTO cities (name) VALUES ('Grand Prairie');
INSERT INTO cities (name) VALUES ('Grand Rapids');
INSERT INTO cities (name) VALUES ('Grande Prairie');
INSERT INTO cities (name) VALUES ('GrandBassam');
INSERT INTO cities (name) VALUES ('GrandBay');
INSERT INTO cities (name) VALUES ('Grantham');
INSERT INTO cities (name) VALUES ('Gravatai');
INSERT INTO cities (name) VALUES ('Graz');
INSERT INTO cities (name) VALUES ('Great Falls');
INSERT INTO cities (name) VALUES ('Great Malvern');
INSERT INTO cities (name) VALUES ('Great Yarmouth');
INSERT INTO cities (name) VALUES ('Greeley');
INSERT INTO cities (name) VALUES ('Green Bay');
INSERT INTO cities (name) VALUES ('Greensboro');
INSERT INTO cities (name) VALUES ('Greenville');
INSERT INTO cities (name) VALUES ('Greenwich');
INSERT INTO cities (name) VALUES ('Grenoble');
INSERT INTO cities (name) VALUES ('Grenville');
INSERT INTO cities (name) VALUES ('Gresham');
INSERT INTO cities (name) VALUES ('Grimsby');
INSERT INTO cities (name) VALUES ('Groningen');
INSERT INTO cities (name) VALUES ('GrovePlace');
INSERT INTO cities (name) VALUES ('Groznyy');
INSERT INTO cities (name) VALUES ('Grudziadz');
INSERT INTO cities (name) VALUES ('Grytviken');
INSERT INTO cities (name) VALUES ('Guacara');
INSERT INTO cities (name) VALUES ('Guadalajara');
INSERT INTO cities (name) VALUES ('Guadalupe');
INSERT INTO cities (name) VALUES ('Guagua');
INSERT INTO cities (name) VALUES ('Guanare');
INSERT INTO cities (name) VALUES ('Guangshui');
INSERT INTO cities (name) VALUES ('Guangyuan');
INSERT INTO cities (name) VALUES ('Guangzhou');
INSERT INTO cities (name) VALUES ('Guantanamo');
INSERT INTO cities (name) VALUES ('Guarapuava');
INSERT INTO cities (name) VALUES ('Guaruja');
INSERT INTO cities (name) VALUES ('Guarulhos');
INSERT INTO cities (name) VALUES ('Guatemala');
INSERT INTO cities (name) VALUES ('Guayama');
INSERT INTO cities (name) VALUES ('Guayaquil');
INSERT INTO cities (name) VALUES ('Guaymas');
INSERT INTO cities (name) VALUES ('Guelph');
INSERT INTO cities (name) VALUES ('Guilin');
INSERT INTO cities (name) VALUES ('Guiyang');
INSERT INTO cities (name) VALUES ('Guizhong');
INSERT INTO cities (name) VALUES ('Gujranwala');
INSERT INTO cities (name) VALUES ('Gujrat');
INSERT INTO cities (name) VALUES ('Gulbarga');
INSERT INTO cities (name) VALUES ('Gulfport');
INSERT INTO cities (name) VALUES ('Guntur');
INSERT INTO cities (name) VALUES ('Gusau');
INSERT INTO cities (name) VALUES ('Guwahati');
INSERT INTO cities (name) VALUES ('Gwalior');
INSERT INTO cities (name) VALUES ('Gweru');
INSERT INTO cities (name) VALUES ('Gyor');
INSERT INTO cities (name) VALUES ('Gyumri');
INSERT INTO cities (name) VALUES ('Haarlem');
INSERT INTO cities (name) VALUES ('Haarlemmermeer');
INSERT INTO cities (name) VALUES ('Habbush');
INSERT INTO cities (name) VALUES ('Hachinohe');
INSERT INTO cities (name) VALUES ('Hachioji');
INSERT INTO cities (name) VALUES ('Hacienda Heights');
INSERT INTO cities (name) VALUES ('Hackensack');
INSERT INTO cities (name) VALUES ('Hadera');
INSERT INTO cities (name) VALUES ('Haeju');
INSERT INTO cities (name) VALUES ('Hafar-al-Batin');
INSERT INTO cities (name) VALUES ('Hafizabad');
INSERT INTO cities (name) VALUES ('Hafnarfjordur');
INSERT INTO cities (name) VALUES ('Hagen');
INSERT INTO cities (name) VALUES ('Haibowan');
INSERT INTO cities (name) VALUES ('Haicheng');
INSERT INTO cities (name) VALUES ('Haifa');
INSERT INTO cities (name) VALUES ('Haikou');
INSERT INTO cities (name) VALUES ('Hail');
INSERT INTO cities (name) VALUES ('Hailar');
INSERT INTO cities (name) VALUES ('Hailun');
INSERT INTO cities (name) VALUES ('Haiphong');
INSERT INTO cities (name) VALUES ('Hakodate');
INSERT INTO cities (name) VALUES ('Halifax');
INSERT INTO cities (name) VALUES ('Halle');
INSERT INTO cities (name) VALUES ('Halley');
INSERT INTO cities (name) VALUES ('Halton Hills');
INSERT INTO cities (name) VALUES ('Hamadan');
INSERT INTO cities (name) VALUES ('Hamah');
INSERT INTO cities (name) VALUES ('Hamamatsu');
INSERT INTO cities (name) VALUES ('Hamburg');
INSERT INTO cities (name) VALUES ('Hamhung');
INSERT INTO cities (name) VALUES ('Hami');
INSERT INTO cities (name) VALUES ('Hamilton');
INSERT INTO cities (name) VALUES ('Hamm');
INSERT INTO cities (name) VALUES ('Hammond');
INSERT INTO cities (name) VALUES ('Hampton');
INSERT INTO cities (name) VALUES ('Handan');
INSERT INTO cities (name) VALUES ('Hanford');
INSERT INTO cities (name) VALUES ('Hangu');
INSERT INTO cities (name) VALUES ('Hangzhou');
INSERT INTO cities (name) VALUES ('Hannover');
INSERT INTO cities (name) VALUES ('Hanoi');
INSERT INTO cities (name) VALUES ('Hanzhong');
INSERT INTO cities (name) VALUES ('Haora');
INSERT INTO cities (name) VALUES ('Hapur');
INSERT INTO cities (name) VALUES ('Harare');
INSERT INTO cities (name) VALUES ('Harbin');
INSERT INTO cities (name) VALUES ('Harer');
INSERT INTO cities (name) VALUES ('Hargeysa');
INSERT INTO cities (name) VALUES ('Harrisburg');
INSERT INTO cities (name) VALUES ('Harrisonburg');
INSERT INTO cities (name) VALUES ('Harrogate');
INSERT INTO cities (name) VALUES ('Hartford');
INSERT INTO cities (name) VALUES ('Hartlepool');
INSERT INTO cities (name) VALUES ('Hasakah');
INSERT INTO cities (name) VALUES ('Hastings');
INSERT INTO cities (name) VALUES ('Hatohobei');
INSERT INTO cities (name) VALUES ('Hattiesburg');
INSERT INTO cities (name) VALUES ('HatYai');
INSERT INTO cities (name) VALUES ('Havana');
INSERT INTO cities (name) VALUES ('Havant');
INSERT INTO cities (name) VALUES ('Hawalli');
INSERT INTO cities (name) VALUES ('Hawthorne');
INSERT INTO cities (name) VALUES ('Hayward');
INSERT INTO cities (name) VALUES ('Haywards Heath');
INSERT INTO cities (name) VALUES ('Hebi');
INSERT INTO cities (name) VALUES ('Hebron');
INSERT INTO cities (name) VALUES ('Heerlen');
INSERT INTO cities (name) VALUES ('Hefei');
INSERT INTO cities (name) VALUES ('Hegang');
INSERT INTO cities (name) VALUES ('Heidelberg');
INSERT INTO cities (name) VALUES ('Heilbronn');
INSERT INTO cities (name) VALUES ('Hekou');
INSERT INTO cities (name) VALUES ('Helena');
INSERT INTO cities (name) VALUES ('Helmond');
INSERT INTO cities (name) VALUES ('Helsingborg');
INSERT INTO cities (name) VALUES ('Helsinki');
INSERT INTO cities (name) VALUES ('Hemet');
INSERT INTO cities (name) VALUES ('Henderson');
INSERT INTO cities (name) VALUES ('Hengelo');
INSERT INTO cities (name) VALUES ('Hengshan');
INSERT INTO cities (name) VALUES ('Hengshui');
INSERT INTO cities (name) VALUES ('Hengyang');
INSERT INTO cities (name) VALUES ('Henzada');
INSERT INTO cities (name) VALUES ('Herat');
INSERT INTO cities (name) VALUES ('Hereford');
INSERT INTO cities (name) VALUES ('Hermosillo');
INSERT INTO cities (name) VALUES ('Herne');
INSERT INTO cities (name) VALUES ('Herne Bay');
INSERT INTO cities (name) VALUES ('Hesperia');
INSERT INTO cities (name) VALUES ('Hetauda');
INSERT INTO cities (name) VALUES ('Heze');
INSERT INTO cities (name) VALUES ('Hialeah');
INSERT INTO cities (name) VALUES ('Hidalgo');
INSERT INTO cities (name) VALUES ('Higashiosaka');
INSERT INTO cities (name) VALUES ('High Point');
INSERT INTO cities (name) VALUES ('Hillah');
INSERT INTO cities (name) VALUES ('Hilo');
INSERT INTO cities (name) VALUES ('Hilversum');
INSERT INTO cities (name) VALUES ('Himeji');
INSERT INTO cities (name) VALUES ('Himki');
INSERT INTO cities (name) VALUES ('Hinnavaru');
INSERT INTO cities (name) VALUES ('Hirosaki');
INSERT INTO cities (name) VALUES ('Hiroshima');
INSERT INTO cities (name) VALUES ('Hisar');
INSERT INTO cities (name) VALUES ('Hitachi');
INSERT INTO cities (name) VALUES ('Hithadhoo');
INSERT INTO cities (name) VALUES ('Ho Chi Minh City (Saigon)');
INSERT INTO cities (name) VALUES ('Hoarafushi');
INSERT INTO cities (name) VALUES ('Hobart');
INSERT INTO cities (name) VALUES ('Hoboken');
INSERT INTO cities (name) VALUES ('Hohhot');
INSERT INTO cities (name) VALUES ('Holguin');
INSERT INTO cities (name) VALUES ('Holhudhoo');
INSERT INTO cities (name) VALUES ('Hollywood');
INSERT INTO cities (name) VALUES ('Homs');
INSERT INTO cities (name) VALUES ('Hong Kong');
INSERT INTO cities (name) VALUES ('Honggang');
INSERT INTO cities (name) VALUES ('Honghu');
INSERT INTO cities (name) VALUES ('HongGai');
INSERT INTO cities (name) VALUES ('Honiara');
INSERT INTO cities (name) VALUES ('Honolulu');
INSERT INTO cities (name) VALUES ('Horlivka');
INSERT INTO cities (name) VALUES ('Hortolandia');
INSERT INTO cities (name) VALUES ('Houailu');
INSERT INTO cities (name) VALUES ('Houston');
INSERT INTO cities (name) VALUES ('Hove');
INSERT INTO cities (name) VALUES ('HradecKralove');
INSERT INTO cities (name) VALUES ('Hrazdan');
INSERT INTO cities (name) VALUES ('Hrodno');
INSERT INTO cities (name) VALUES ('Hsinchu');
INSERT INTO cities (name) VALUES ('Hsinchuang');
INSERT INTO cities (name) VALUES ('Hsintien');
INSERT INTO cities (name) VALUES ('Huadian');
INSERT INTO cities (name) VALUES ('Huaibei');
INSERT INTO cities (name) VALUES ('Huaihua');
INSERT INTO cities (name) VALUES ('Huaiyin');
INSERT INTO cities (name) VALUES ('Hualian');
INSERT INTO cities (name) VALUES ('Huambo');
INSERT INTO cities (name) VALUES ('Huancayo');
INSERT INTO cities (name) VALUES ('Huanggang');
INSERT INTO cities (name) VALUES ('Huangshi');
INSERT INTO cities (name) VALUES ('Huangyan');
INSERT INTO cities (name) VALUES ('Huangzhou');
INSERT INTO cities (name) VALUES ('Huanuco');
INSERT INTO cities (name) VALUES ('Hubli');
INSERT INTO cities (name) VALUES ('Hudaydah');
INSERT INTO cities (name) VALUES ('Huddersfield');
INSERT INTO cities (name) VALUES ('Hue');
INSERT INTO cities (name) VALUES ('Huelva');
INSERT INTO cities (name) VALUES ('Huicheng');
INSERT INTO cities (name) VALUES ('Huizhou');
INSERT INTO cities (name) VALUES ('Hulan');
INSERT INTO cities (name) VALUES ('Hulan Ergi');
INSERT INTO cities (name) VALUES ('Humacao');
INSERT INTO cities (name) VALUES ('Humen');
INSERT INTO cities (name) VALUES ('Huntington');
INSERT INTO cities (name) VALUES ('Huntington Beach');
INSERT INTO cities (name) VALUES ('Huntington Park');
INSERT INTO cities (name) VALUES ('Huntsville');
INSERT INTO cities (name) VALUES ('Huribgah');
INSERT INTO cities (name) VALUES ('Hutchinson');
INSERT INTO cities (name) VALUES ('Huzhou');
INSERT INTO cities (name) VALUES ('Hwange');
INSERT INTO cities (name) VALUES ('Hyderabad');
INSERT INTO cities (name) VALUES ('Hyeres');
INSERT INTO cities (name) VALUES ('Hyesan');
INSERT INTO cities (name) VALUES ('Iasi');
INSERT INTO cities (name) VALUES ('Ibadan');
INSERT INTO cities (name) VALUES ('Ibague');
INSERT INTO cities (name) VALUES ('Ibarra');
INSERT INTO cities (name) VALUES ('Ibb');
INSERT INTO cities (name) VALUES ('Ibirite');
INSERT INTO cities (name) VALUES ('Ibri');
INSERT INTO cities (name) VALUES ('Ica');
INSERT INTO cities (name) VALUES ('Ichalkaranji');
INSERT INTO cities (name) VALUES ('Ichikawa');
INSERT INTO cities (name) VALUES ('Idaho Falls');
INSERT INTO cities (name) VALUES ('Idku');
INSERT INTO cities (name) VALUES ('Ife');
INSERT INTO cities (name) VALUES ('Igboho');
INSERT INTO cities (name) VALUES ('Iguala');
INSERT INTO cities (name) VALUES ('Ihavandhoo');
INSERT INTO cities (name) VALUES ('Ijaki');
INSERT INTO cities (name) VALUES ('Ijebu Ode');
INSERT INTO cities (name) VALUES ('Ijero');
INSERT INTO cities (name) VALUES ('Ikamiut');
INSERT INTO cities (name) VALUES ('Ikare');
INSERT INTO cities (name) VALUES ('Ikerasaarsuk');
INSERT INTO cities (name) VALUES ('Ikerasak');
INSERT INTO cities (name) VALUES ('Ikot Ekpene');
INSERT INTO cities (name) VALUES ('Iksan');
INSERT INTO cities (name) VALUES ('Ila');
INSERT INTO cities (name) VALUES ('Ilam');
INSERT INTO cities (name) VALUES ('Ilawe');
INSERT INTO cities (name) VALUES ('Ilebo');
INSERT INTO cities (name) VALUES ('Ilesha');
INSERT INTO cities (name) VALUES ('Ilheus');
INSERT INTO cities (name) VALUES ('Illoqqortoormiut');
INSERT INTO cities (name) VALUES ('Illorsuit');
INSERT INTO cities (name) VALUES ('Iloilo');
INSERT INTO cities (name) VALUES ('Ilorin');
INSERT INTO cities (name) VALUES ('Ilulissat');
INSERT INTO cities (name) VALUES ('Imeong');
INSERT INTO cities (name) VALUES ('Imperatriz');
INSERT INTO cities (name) VALUES ('Imphal');
INSERT INTO cities (name) VALUES ('Incheon');
INSERT INTO cities (name) VALUES ('Indaiatuba');
INSERT INTO cities (name) VALUES ('Independence');
INSERT INTO cities (name) VALUES ('Indianapolis');
INSERT INTO cities (name) VALUES ('Indio');
INSERT INTO cities (name) VALUES ('Indore');
INSERT INTO cities (name) VALUES ('Inegol');
INSERT INTO cities (name) VALUES ('Inglewood');
INSERT INTO cities (name) VALUES ('Ingolstadt');
INSERT INTO cities (name) VALUES ('Innsbruck');
INSERT INTO cities (name) VALUES ('Invercargill');
INSERT INTO cities (name) VALUES ('Inverness');
INSERT INTO cities (name) VALUES ('Iowa City');
INSERT INTO cities (name) VALUES ('Ipatinga');
INSERT INTO cities (name) VALUES ('Ipoh');
INSERT INTO cities (name) VALUES ('Ipswich');
INSERT INTO cities (name) VALUES ('Iqaluit');
INSERT INTO cities (name) VALUES ('Iquique');
INSERT INTO cities (name) VALUES ('Iquitos');
INSERT INTO cities (name) VALUES ('Iraklion');
INSERT INTO cities (name) VALUES ('Irapuato');
INSERT INTO cities (name) VALUES ('Irbid');
INSERT INTO cities (name) VALUES ('Irbil');
INSERT INTO cities (name) VALUES ('Iringa');
INSERT INTO cities (name) VALUES ('Irkutsk');
INSERT INTO cities (name) VALUES ('Iruna');
INSERT INTO cities (name) VALUES ('Irvine');
INSERT INTO cities (name) VALUES ('Irving');
INSERT INTO cities (name) VALUES ('Isa');
INSERT INTO cities (name) VALUES ('Isangel');
INSERT INTO cities (name) VALUES ('Ise');
INSERT INTO cities (name) VALUES ('Iserlohn');
INSERT INTO cities (name) VALUES ('Iseyin');
INSERT INTO cities (name) VALUES ('Isiro');
INSERT INTO cities (name) VALUES ('Iskenderun');
INSERT INTO cities (name) VALUES ('Islamabad');
INSERT INTO cities (name) VALUES ('Ismailia');
INSERT INTO cities (name) VALUES ('Isortoq');
INSERT INTO cities (name) VALUES ('Isparta');
INSERT INTO cities (name) VALUES ('Istanbul');
INSERT INTO cities (name) VALUES ('Itaborai');
INSERT INTO cities (name) VALUES ('Itabuna');
INSERT INTO cities (name) VALUES ('Itagui');
INSERT INTO cities (name) VALUES ('Itajai');
INSERT INTO cities (name) VALUES ('ItapecericadaSerra');
INSERT INTO cities (name) VALUES ('Itapetininga');
INSERT INTO cities (name) VALUES ('Itapevi');
INSERT INTO cities (name) VALUES ('Itaquaquecetuba');
INSERT INTO cities (name) VALUES ('Itilleq');
INSERT INTO cities (name) VALUES ('Itu');
INSERT INTO cities (name) VALUES ('Ivano-Frankivsk');
INSERT INTO cities (name) VALUES ('Ivanovo');
INSERT INTO cities (name) VALUES ('Ivittuut');
INSERT INTO cities (name) VALUES ('Iwaki');
INSERT INTO cities (name) VALUES ('Izhevsk');
INSERT INTO cities (name) VALUES ('Izmir');
INSERT INTO cities (name) VALUES ('Izmit');
INSERT INTO cities (name) VALUES ('Jabaliyah');
INSERT INTO cities (name) VALUES ('Jabalpur');
INSERT INTO cities (name) VALUES ('Jaboatao');
INSERT INTO cities (name) VALUES ('Jacarei');
INSERT INTO cities (name) VALUES ('Jackson');
INSERT INTO cities (name) VALUES ('Jacksonville');
INSERT INTO cities (name) VALUES ('Jacmel');
INSERT INTO cities (name) VALUES ('Jacobabad');
INSERT INTO cities (name) VALUES ('Jaen');
INSERT INTO cities (name) VALUES ('Jaffna');
INSERT INTO cities (name) VALUES ('Jahrom');
INSERT INTO cities (name) VALUES ('Jaipur');
INSERT INTO cities (name) VALUES ('Jakarta');
INSERT INTO cities (name) VALUES ('Jalal-Abad');
INSERT INTO cities (name) VALUES ('Jalalabad');
INSERT INTO cities (name) VALUES ('Jalandhar');
INSERT INTO cities (name) VALUES ('Jalgaon');
INSERT INTO cities (name) VALUES ('Jalib as-Suyuh');
INSERT INTO cities (name) VALUES ('Jalingo');
INSERT INTO cities (name) VALUES ('Jalna');
INSERT INTO cities (name) VALUES ('Jaluit');
INSERT INTO cities (name) VALUES ('Jamame');
INSERT INTO cities (name) VALUES ('Jambi');
INSERT INTO cities (name) VALUES ('Jamestown');
INSERT INTO cities (name) VALUES ('Jammu');
INSERT INTO cities (name) VALUES ('Jamnagar');
INSERT INTO cities (name) VALUES ('Jamshedpur');
INSERT INTO cities (name) VALUES ('Janakpur');
INSERT INTO cities (name) VALUES ('Janesville');
INSERT INTO cities (name) VALUES ('JaraguadoSul');
INSERT INTO cities (name) VALUES ('Jaranwala');
INSERT INTO cities (name) VALUES ('Jastrzebie-Zdroj');
INSERT INTO cities (name) VALUES ('Jau');
INSERT INTO cities (name) VALUES ('Jawhar');
INSERT INTO cities (name) VALUES ('Jaya Pura');
INSERT INTO cities (name) VALUES ('Jefferson City');
INSERT INTO cities (name) VALUES ('Jelgava');
INSERT INTO cities (name) VALUES ('Jember');
INSERT INTO cities (name) VALUES ('Jena');
INSERT INTO cities (name) VALUES ('Jequie');
INSERT INTO cities (name) VALUES ('Jeremie');
INSERT INTO cities (name) VALUES ('Jerez');
INSERT INTO cities (name) VALUES ('Jersey City');
INSERT INTO cities (name) VALUES ('Jerusalem (ISR)');
INSERT INTO cities (name) VALUES ('Jerusalem (PAL)');
INSERT INTO cities (name) VALUES ('Jessor');
INSERT INTO cities (name) VALUES ('Jhang');
INSERT INTO cities (name) VALUES ('Jhansi');
INSERT INTO cities (name) VALUES ('Jhelum');
INSERT INTO cities (name) VALUES ('Jiagedagi');
INSERT INTO cities (name) VALUES ('Jiamusi');
INSERT INTO cities (name) VALUES ('Jian');
INSERT INTO cities (name) VALUES ('Jiangmen');
INSERT INTO cities (name) VALUES ('Jiangyin');
INSERT INTO cities (name) VALUES ('Jiangyou');
INSERT INTO cities (name) VALUES ('Jiaohe');
INSERT INTO cities (name) VALUES ('Jiaozhou');
INSERT INTO cities (name) VALUES ('Jiaozuo');
INSERT INTO cities (name) VALUES ('Jiaxing');
INSERT INTO cities (name) VALUES ('Jiayuguan');
INSERT INTO cities (name) VALUES ('Jiddah');
INSERT INTO cities (name) VALUES ('Jieshi');
INSERT INTO cities (name) VALUES ('Jieshou');
INSERT INTO cities (name) VALUES ('Jilin');
INSERT INTO cities (name) VALUES ('Jimeta');
INSERT INTO cities (name) VALUES ('Jimma');
INSERT INTO cities (name) VALUES ('Jinan');
INSERT INTO cities (name) VALUES ('Jinchang');
INSERT INTO cities (name) VALUES ('Jincheng');
INSERT INTO cities (name) VALUES ('Jinchengjiang');
INSERT INTO cities (name) VALUES ('Jingdezhen');
INSERT INTO cities (name) VALUES ('Jingmen');
INSERT INTO cities (name) VALUES ('Jingzhou');
INSERT INTO cities (name) VALUES ('Jinhua');
INSERT INTO cities (name) VALUES ('Jining');
INSERT INTO cities (name) VALUES ('Jinja');
INSERT INTO cities (name) VALUES ('Jinma');
INSERT INTO cities (name) VALUES ('Jinxi');
INSERT INTO cities (name) VALUES ('Jinzhou');
INSERT INTO cities (name) VALUES ('Jirja');
INSERT INTO cities (name) VALUES ('Jishu');
INSERT INTO cities (name) VALUES ('Jiujiang');
INSERT INTO cities (name) VALUES ('Jiupu');
INSERT INTO cities (name) VALUES ('Jiutai');
INSERT INTO cities (name) VALUES ('Jixi');
INSERT INTO cities (name) VALUES ('JoaoPessoa');
INSERT INTO cities (name) VALUES ('Jodhpur');
INSERT INTO cities (name) VALUES ('Johannesburg');
INSERT INTO cities (name) VALUES ('Johnson City');
INSERT INTO cities (name) VALUES ('JohorBahru');
INSERT INTO cities (name) VALUES ('Joinville');
INSERT INTO cities (name) VALUES ('Joliet');
INSERT INTO cities (name) VALUES ('Jonesboro');
INSERT INTO cities (name) VALUES ('Jonkoping');
INSERT INTO cities (name) VALUES ('Jonquire');
INSERT INTO cities (name) VALUES ('Joplin');
INSERT INTO cities (name) VALUES ('Jos');
INSERT INTO cities (name) VALUES ('Juazeiro');
INSERT INTO cities (name) VALUES ('JuazeirodoNorte');
INSERT INTO cities (name) VALUES ('Juba');
INSERT INTO cities (name) VALUES ('Jubail');
INSERT INTO cities (name) VALUES ('Juigalpa');
INSERT INTO cities (name) VALUES ('JuizdeFora');
INSERT INTO cities (name) VALUES ('Jujuy');
INSERT INTO cities (name) VALUES ('Juliaca');
INSERT INTO cities (name) VALUES ('Jundiai');
INSERT INTO cities (name) VALUES ('Juneau');
INSERT INTO cities (name) VALUES ('Jyvaskyla');
INSERT INTO cities (name) VALUES ('Kabul');
INSERT INTO cities (name) VALUES ('Kabwe');
INSERT INTO cities (name) VALUES ('Kadholhudhoo');
INSERT INTO cities (name) VALUES ('Kadoma');
INSERT INTO cities (name) VALUES ('Kaduna');
INSERT INTO cities (name) VALUES ('Kaesong');
INSERT INTO cities (name) VALUES ('Kafr el Sheikh');
INSERT INTO cities (name) VALUES ('Kaga-Bandoro');
INSERT INTO cities (name) VALUES ('Kagoshima');
INSERT INTO cities (name) VALUES ('Kahramanmaras');
INSERT INTO cities (name) VALUES ('Kaifeng');
INSERT INTO cities (name) VALUES ('Kaili');
INSERT INTO cities (name) VALUES ('Kairouan');
INSERT INTO cities (name) VALUES ('Kaiserslautern');
INSERT INTO cities (name) VALUES ('Kaiyuan');
INSERT INTO cities (name) VALUES ('Kajang-SungaiChua');
INSERT INTO cities (name) VALUES ('Kakamega');
INSERT INTO cities (name) VALUES ('Kakinada');
INSERT INTO cities (name) VALUES ('Kalamazoo');
INSERT INTO cities (name) VALUES ('Kalemie');
INSERT INTO cities (name) VALUES ('Kaliningrad');
INSERT INTO cities (name) VALUES ('Kalisz');
INSERT INTO cities (name) VALUES ('Kaluga');
INSERT INTO cities (name) VALUES ('Kamalia');
INSERT INTO cities (name) VALUES ('Kamarhati');
INSERT INTO cities (name) VALUES ('Kamensk-Uralskiy');
INSERT INTO cities (name) VALUES ('Kamloops');
INSERT INTO cities (name) VALUES ('Kamoke');
INSERT INTO cities (name) VALUES ('Kampala');
INSERT INTO cities (name) VALUES ('Kamyanets-Podilskyy');
INSERT INTO cities (name) VALUES ('Kamyshin');
INSERT INTO cities (name) VALUES ('Kananga');
INSERT INTO cities (name) VALUES ('Kanata');
INSERT INTO cities (name) VALUES ('Kanazawa');
INSERT INTO cities (name) VALUES ('Kandahar');
INSERT INTO cities (name) VALUES ('Kandi');
INSERT INTO cities (name) VALUES ('Kandy');
INSERT INTO cities (name) VALUES ('Kangaamiut');
INSERT INTO cities (name) VALUES ('Kangaatsiaq');
INSERT INTO cities (name) VALUES ('Kangerlussuaq');
INSERT INTO cities (name) VALUES ('Kangersuatsiaq');
INSERT INTO cities (name) VALUES ('Kanggye');
INSERT INTO cities (name) VALUES ('Kangilinnguit');
INSERT INTO cities (name) VALUES ('Kangneung');
INSERT INTO cities (name) VALUES ('Kankan');
INSERT INTO cities (name) VALUES ('Kano');
INSERT INTO cities (name) VALUES ('Kanpur');
INSERT INTO cities (name) VALUES ('Kansas City');
INSERT INTO cities (name) VALUES ('Kansk');
INSERT INTO cities (name) VALUES ('Kaohsiung');
INSERT INTO cities (name) VALUES ('Kaolack');
INSERT INTO cities (name) VALUES ('Kapisillit');
INSERT INTO cities (name) VALUES ('Kara');
INSERT INTO cities (name) VALUES ('Karabalta');
INSERT INTO cities (name) VALUES ('Karab?k');
INSERT INTO cities (name) VALUES ('Karachi');
INSERT INTO cities (name) VALUES ('Karaganda');
INSERT INTO cities (name) VALUES ('Karakol');
INSERT INTO cities (name) VALUES ('Karaman');
INSERT INTO cities (name) VALUES ('Karamay');
INSERT INTO cities (name) VALUES ('Karawang');
INSERT INTO cities (name) VALUES ('Karbala');
INSERT INTO cities (name) VALUES ('Karimnagar');
INSERT INTO cities (name) VALUES ('Karlovac');
INSERT INTO cities (name) VALUES ('Karlsruhe');
INSERT INTO cities (name) VALUES ('Karmiel');
INSERT INTO cities (name) VALUES ('Karnal');
INSERT INTO cities (name) VALUES ('Karnul');
INSERT INTO cities (name) VALUES ('Kars');
INSERT INTO cities (name) VALUES ('Kasama');
INSERT INTO cities (name) VALUES ('Kashan');
INSERT INTO cities (name) VALUES ('Kashi');
INSERT INTO cities (name) VALUES ('Kassala');
INSERT INTO cities (name) VALUES ('Kassel');
INSERT INTO cities (name) VALUES ('Kasur');
INSERT INTO cities (name) VALUES ('Kathmandu');
INSERT INTO cities (name) VALUES ('KatimaMulilo');
INSERT INTO cities (name) VALUES ('Katowice');
INSERT INTO cities (name) VALUES ('Katsina');
INSERT INTO cities (name) VALUES ('Katsuta');
INSERT INTO cities (name) VALUES ('Kaunas');
INSERT INTO cities (name) VALUES ('Kawaguchi');
INSERT INTO cities (name) VALUES ('Kawasaki');
INSERT INTO cities (name) VALUES ('Kayangel');
INSERT INTO cities (name) VALUES ('Kayes');
INSERT INTO cities (name) VALUES ('Kayhaydi');
INSERT INTO cities (name) VALUES ('Kayseri');
INSERT INTO cities (name) VALUES ('Kazan');
INSERT INTO cities (name) VALUES ('Kecskemet');
INSERT INTO cities (name) VALUES ('Kediri');
INSERT INTO cities (name) VALUES ('Keelung (Chilung)');
INSERT INTO cities (name) VALUES ('Keetmanshoop');
INSERT INTO cities (name) VALUES ('Keflavk');
INSERT INTO cities (name) VALUES ('Kelo');
INSERT INTO cities (name) VALUES ('Kelowna');
INSERT INTO cities (name) VALUES ('Kembolcha');
INSERT INTO cities (name) VALUES ('Kemerovo');
INSERT INTO cities (name) VALUES ('Kendal');
INSERT INTO cities (name) VALUES ('Kendall');
INSERT INTO cities (name) VALUES ('Kendari');
INSERT INTO cities (name) VALUES ('Kenema');
INSERT INTO cities (name) VALUES ('Kenitra');
INSERT INTO cities (name) VALUES ('Kennewick');
INSERT INTO cities (name) VALUES ('Kenosha');
INSERT INTO cities (name) VALUES ('Kerch');
INSERT INTO cities (name) VALUES ('Keren');
INSERT INTO cities (name) VALUES ('Kerman');
INSERT INTO cities (name) VALUES ('Kermanshah');
INSERT INTO cities (name) VALUES ('Kettering');
INSERT INTO cities (name) VALUES ('Khabarovsk');
INSERT INTO cities (name) VALUES ('Khairpur');
INSERT INTO cities (name) VALUES ('Khamis Mushayt');
INSERT INTO cities (name) VALUES ('Khamissat');
INSERT INTO cities (name) VALUES ('Khanewal');
INSERT INTO cities (name) VALUES ('Khanfar');
INSERT INTO cities (name) VALUES ('Khanpur');
INSERT INTO cities (name) VALUES ('Khanyunis');
INSERT INTO cities (name) VALUES ('Kharagpur');
INSERT INTO cities (name) VALUES ('Kharj');
INSERT INTO cities (name) VALUES ('Kharkiv');
INSERT INTO cities (name) VALUES ('Khartoum');
INSERT INTO cities (name) VALUES ('Kherson');
INSERT INTO cities (name) VALUES ('Khmel\'nytskyy');
INSERT INTO cities (name) VALUES ('Khodzheyli');
INSERT INTO cities (name) VALUES ('KhonKaen');
INSERT INTO cities (name) VALUES ('Khorramabad');
INSERT INTO cities (name) VALUES ('Khorramshahr');
INSERT INTO cities (name) VALUES ('Khoy');
INSERT INTO cities (name) VALUES ('Khujand');
INSERT INTO cities (name) VALUES ('Khulna');
INSERT INTO cities (name) VALUES ('Khuzdar');
INSERT INTO cities (name) VALUES ('Kidsgrove');
INSERT INTO cities (name) VALUES ('Kiel');
INSERT INTO cities (name) VALUES ('Kielce');
INSERT INTO cities (name) VALUES ('Kifa');
INSERT INTO cities (name) VALUES ('Kigali');
INSERT INTO cities (name) VALUES ('Kigoma');
INSERT INTO cities (name) VALUES ('Kikwit');
INSERT INTO cities (name) VALUES ('Kilkenny');
INSERT INTO cities (name) VALUES ('Killeen');
INSERT INTO cities (name) VALUES ('Kilmarnock');
INSERT INTO cities (name) VALUES ('Kilosa');
INSERT INTO cities (name) VALUES ('Kimberley');
INSERT INTO cities (name) VALUES ('Kimchaek');
INSERT INTO cities (name) VALUES ('Kindu');
INSERT INTO cities (name) VALUES ('Kineshma');
INSERT INTO cities (name) VALUES ('King Williams Town');
INSERT INTO cities (name) VALUES ('King\'s Lynn');
INSERT INTO cities (name) VALUES ('Kingsport');
INSERT INTO cities (name) VALUES ('Kingston');
INSERT INTO cities (name) VALUES ('Kingston upon Hull');
INSERT INTO cities (name) VALUES ('Kingstown');
INSERT INTO cities (name) VALUES ('KingSejong');
INSERT INTO cities (name) VALUES ('Kinkala');
INSERT INTO cities (name) VALUES ('Kinshasa');
INSERT INTO cities (name) VALUES ('Kirakira');
INSERT INTO cities (name) VALUES ('Kirby in Ashfield');
INSERT INTO cities (name) VALUES ('Kirkcaldy');
INSERT INTO cities (name) VALUES ('Kirkuk');
INSERT INTO cities (name) VALUES ('Kirov');
INSERT INTO cities (name) VALUES ('Kirovohrad');
INSERT INTO cities (name) VALUES ('Kirrikkale');
INSERT INTO cities (name) VALUES ('Kisangani');
INSERT INTO cities (name) VALUES ('Kiselyovsk');
INSERT INTO cities (name) VALUES ('Kishi');
INSERT INTO cities (name) VALUES ('Kisimayo');
INSERT INTO cities (name) VALUES ('Kislovodsk');
INSERT INTO cities (name) VALUES ('Kissimmee');
INSERT INTO cities (name) VALUES ('Kisumu');
INSERT INTO cities (name) VALUES ('Kitakyushu');
INSERT INTO cities (name) VALUES ('Kitale');
INSERT INTO cities (name) VALUES ('Kitchener');
INSERT INTO cities (name) VALUES ('Kitsissuarsuit');
INSERT INTO cities (name) VALUES ('Kitwe');
INSERT INTO cities (name) VALUES ('Kiziltepe');
INSERT INTO cities (name) VALUES ('Klagenfurt');
INSERT INTO cities (name) VALUES ('Klaipeda');
INSERT INTO cities (name) VALUES ('Klaksvik');
INSERT INTO cities (name) VALUES ('Klaten');
INSERT INTO cities (name) VALUES ('Klerksdorp');
INSERT INTO cities (name) VALUES ('Kloulklubed');
INSERT INTO cities (name) VALUES ('Kluang');
INSERT INTO cities (name) VALUES ('Knoxville');
INSERT INTO cities (name) VALUES ('Kobe');
INSERT INTO cities (name) VALUES ('Koblenz');
INSERT INTO cities (name) VALUES ('Koblerville');
INSERT INTO cities (name) VALUES ('Kochi');
INSERT INTO cities (name) VALUES ('Kochi (Cochin)');
INSERT INTO cities (name) VALUES ('Koforidua');
INSERT INTO cities (name) VALUES ('Kofu');
INSERT INTO cities (name) VALUES ('Kohat');
INSERT INTO cities (name) VALUES ('Kohtla-Jarve');
INSERT INTO cities (name) VALUES ('Koidu');
INSERT INTO cities (name) VALUES ('Kokomo');
INSERT INTO cities (name) VALUES ('Koksetau');
INSERT INTO cities (name) VALUES ('Kolhapur');
INSERT INTO cities (name) VALUES ('Kollam');
INSERT INTO cities (name) VALUES ('Koln-Ruhr Area');
INSERT INTO cities (name) VALUES ('Kolomna');
INSERT INTO cities (name) VALUES ('Kolonia');
INSERT INTO cities (name) VALUES ('Kolpino');
INSERT INTO cities (name) VALUES ('Kolwezi');
INSERT INTO cities (name) VALUES ('Komilla');
INSERT INTO cities (name) VALUES ('Komsomolsk-na-Amure');
INSERT INTO cities (name) VALUES ('Konya');
INSERT INTO cities (name) VALUES ('Kopavogur');
INSERT INTO cities (name) VALUES ('Korat');
INSERT INTO cities (name) VALUES ('Korba');
INSERT INTO cities (name) VALUES ('Korhogo');
INSERT INTO cities (name) VALUES ('Koriyama');
INSERT INTO cities (name) VALUES ('Korla');
INSERT INTO cities (name) VALUES ('Korogwe');
INSERT INTO cities (name) VALUES ('Korolyov');
INSERT INTO cities (name) VALUES ('Koror');
INSERT INTO cities (name) VALUES ('Koshice');
INSERT INTO cities (name) VALUES ('Kostroma');
INSERT INTO cities (name) VALUES ('Koszalin');
INSERT INTO cities (name) VALUES ('Kota');
INSERT INTO cities (name) VALUES ('KotaBahru');
INSERT INTO cities (name) VALUES ('KotaKinabalu');
INSERT INTO cities (name) VALUES ('Koudougou');
INSERT INTO cities (name) VALUES ('Koungou');
INSERT INTO cities (name) VALUES ('Kourou');
INSERT INTO cities (name) VALUES ('Koutiala');
INSERT INTO cities (name) VALUES ('Kovrov');
INSERT INTO cities (name) VALUES ('Kowloon');
INSERT INTO cities (name) VALUES ('Koyampattur');
INSERT INTO cities (name) VALUES ('Koyang');
INSERT INTO cities (name) VALUES ('Kozhikkod');
INSERT INTO cities (name) VALUES ('Kpalime');
INSERT INTO cities (name) VALUES ('Kragujevac');
INSERT INTO cities (name) VALUES ('Krakow');
INSERT INTO cities (name) VALUES ('Kralendijk');
INSERT INTO cities (name) VALUES ('Kramatorsk');
INSERT INTO cities (name) VALUES ('Krasnodar');
INSERT INTO cities (name) VALUES ('Krasnoyarsk');
INSERT INTO cities (name) VALUES ('Krasnyy Luch');
INSERT INTO cities (name) VALUES ('Krefeld');
INSERT INTO cities (name) VALUES ('Kremenchuh');
INSERT INTO cities (name) VALUES ('Kroonstad');
INSERT INTO cities (name) VALUES ('Kryvyy Rih');
INSERT INTO cities (name) VALUES ('Ksar el Kabir');
INSERT INTO cities (name) VALUES ('Kua');
INSERT INTO cities (name) VALUES ('KualaBelait');
INSERT INTO cities (name) VALUES ('KualaLumpur');
INSERT INTO cities (name) VALUES ('KualaTerengganu');
INSERT INTO cities (name) VALUES ('Kuantan');
INSERT INTO cities (name) VALUES ('Kuching');
INSERT INTO cities (name) VALUES ('Kudus');
INSERT INTO cities (name) VALUES ('Kuisebmond');
INSERT INTO cities (name) VALUES ('Kuito');
INSERT INTO cities (name) VALUES ('Kukon');
INSERT INTO cities (name) VALUES ('Kulhudhuffushi');
INSERT INTO cities (name) VALUES ('Kullorsuaq');
INSERT INTO cities (name) VALUES ('Kulti');
INSERT INTO cities (name) VALUES ('Kulusuk');
INSERT INTO cities (name) VALUES ('Kulyab');
INSERT INTO cities (name) VALUES ('Kumamoto');
INSERT INTO cities (name) VALUES ('Kumanovo');
INSERT INTO cities (name) VALUES ('Kumasi');
INSERT INTO cities (name) VALUES ('Kumba');
INSERT INTO cities (name) VALUES ('Kumi');
INSERT INTO cities (name) VALUES ('Kuningan');
INSERT INTO cities (name) VALUES ('Kunming');
INSERT INTO cities (name) VALUES ('Kunsan');
INSERT INTO cities (name) VALUES ('Kuopio');
INSERT INTO cities (name) VALUES ('Kupang');
INSERT INTO cities (name) VALUES ('Kure');
INSERT INTO cities (name) VALUES ('Kurgan');
INSERT INTO cities (name) VALUES ('Kursk');
INSERT INTO cities (name) VALUES ('Kurume');
INSERT INTO cities (name) VALUES ('Kushiro');
INSERT INTO cities (name) VALUES ('Kusti');
INSERT INTO cities (name) VALUES ('Kutahya');
INSERT INTO cities (name) VALUES ('Kutaisi');
INSERT INTO cities (name) VALUES ('Kuumiut');
INSERT INTO cities (name) VALUES ('Kuwait City');
INSERT INTO cities (name) VALUES ('Kuznetsk');
INSERT INTO cities (name) VALUES ('Kwangju');
INSERT INTO cities (name) VALUES ('Kwangmyeong');
INSERT INTO cities (name) VALUES ('Kwekwe');
INSERT INTO cities (name) VALUES ('Kyeongju');
INSERT INTO cities (name) VALUES ('Kyoto');
INSERT INTO cities (name) VALUES ('Kyyiv');
INSERT INTO cities (name) VALUES ('Kyzyl');
INSERT INTO cities (name) VALUES ('L\'viv');
INSERT INTO cities (name) VALUES ('La Crosse');
INSERT INTO cities (name) VALUES ('La Habra');
INSERT INTO cities (name) VALUES ('La Paz');
INSERT INTO cities (name) VALUES ('La Roche-sur-Yon');
INSERT INTO cities (name) VALUES ('La Rochelle');
INSERT INTO cities (name) VALUES ('La Serena');
INSERT INTO cities (name) VALUES ('La Spezia');
INSERT INTO cities (name) VALUES ('Laayoune');
INSERT INTO cities (name) VALUES ('Labasa');
INSERT INTO cities (name) VALUES ('Labe');
INSERT INTO cities (name) VALUES ('Ladhiqiyah');
INSERT INTO cities (name) VALUES ('Ladysmith');
INSERT INTO cities (name) VALUES ('Lae');
INSERT INTO cities (name) VALUES ('Lafayette');
INSERT INTO cities (name) VALUES ('Lafia');
INSERT INTO cities (name) VALUES ('Lages');
INSERT INTO cities (name) VALUES ('Lagos');
INSERT INTO cities (name) VALUES ('Laguna Niguel');
INSERT INTO cities (name) VALUES ('Lahore');
INSERT INTO cities (name) VALUES ('Lahti');
INSERT INTO cities (name) VALUES ('Laiwu');
INSERT INTO cities (name) VALUES ('Laiyang');
INSERT INTO cities (name) VALUES ('Lakatoro');
INSERT INTO cities (name) VALUES ('Lake Charles');
INSERT INTO cities (name) VALUES ('Lake Forest');
INSERT INTO cities (name) VALUES ('Lake Havasu City');
INSERT INTO cities (name) VALUES ('Lakeland');
INSERT INTO cities (name) VALUES ('Lakewood');
INSERT INTO cities (name) VALUES ('Lalbahadur Nagar');
INSERT INTO cities (name) VALUES ('Lalitpur');
INSERT INTO cities (name) VALUES ('Lampang');
INSERT INTO cities (name) VALUES ('Lancaster');
INSERT INTO cities (name) VALUES ('Langfang');
INSERT INTO cities (name) VALUES ('Lansing');
INSERT INTO cities (name) VALUES ('Lanzhou');
INSERT INTO cities (name) VALUES ('Laohekou');
INSERT INTO cities (name) VALUES ('Laredo');
INSERT INTO cities (name) VALUES ('Larissa');
INSERT INTO cities (name) VALUES ('Larkana');
INSERT INTO cities (name) VALUES ('Larnaca');
INSERT INTO cities (name) VALUES ('Las Cruces');
INSERT INTO cities (name) VALUES ('Las Palmas');
INSERT INTO cities (name) VALUES ('Las Vegas');
INSERT INTO cities (name) VALUES ('Lasa');
INSERT INTO cities (name) VALUES ('Lashio');
INSERT INTO cities (name) VALUES ('LasTunas');
INSERT INTO cities (name) VALUES ('Lata');
INSERT INTO cities (name) VALUES ('Latina');
INSERT INTO cities (name) VALUES ('Launceston');
INSERT INTO cities (name) VALUES ('LaurodeFreitas');
INSERT INTO cities (name) VALUES ('Lausanne');
INSERT INTO cities (name) VALUES ('Lautoka');
INSERT INTO cities (name) VALUES ('Laval');
INSERT INTO cities (name) VALUES ('Lawrence');
INSERT INTO cities (name) VALUES ('Lawton');
INSERT INTO cities (name) VALUES ('Layton');
INSERT INTO cities (name) VALUES ('LaCeiba');
INSERT INTO cities (name) VALUES ('LaChorrera');
INSERT INTO cities (name) VALUES ('LaPaz');
INSERT INTO cities (name) VALUES ('LaPlata');
INSERT INTO cities (name) VALUES ('LaRioja');
INSERT INTO cities (name) VALUES ('LaRomana');
INSERT INTO cities (name) VALUES ('LaVega');
INSERT INTO cities (name) VALUES ('Le Havre');
INSERT INTO cities (name) VALUES ('Le Mans');
INSERT INTO cities (name) VALUES ('Lecce');
INSERT INTO cities (name) VALUES ('Leeds');
INSERT INTO cities (name) VALUES ('Leeuwarden');
INSERT INTO cities (name) VALUES ('Legnica');
INSERT INTO cities (name) VALUES ('Leicester');
INSERT INTO cities (name) VALUES ('Leiden');
INSERT INTO cities (name) VALUES ('Leipzig');
INSERT INTO cities (name) VALUES ('Leiyang');
INSERT INTO cities (name) VALUES ('Lelu');
INSERT INTO cities (name) VALUES ('Lelydorp');
INSERT INTO cities (name) VALUES ('Lengshuijiang');
INSERT INTO cities (name) VALUES ('Leninsk-Kuzneckiy');
INSERT INTO cities (name) VALUES ('Leon');
INSERT INTO cities (name) VALUES ('Leshan');
INSERT INTO cities (name) VALUES ('LesAbymes');
INSERT INTO cities (name) VALUES ('LesCayes');
INSERT INTO cities (name) VALUES ('Lethbridge');
INSERT INTO cities (name) VALUES ('Leverkusen');
INSERT INTO cities (name) VALUES ('Lexington-Fayette');
INSERT INTO cities (name) VALUES ('Leyland');
INSERT INTO cities (name) VALUES ('LeFrancois');
INSERT INTO cities (name) VALUES ('LeGosier');
INSERT INTO cities (name) VALUES ('LeMoule');
INSERT INTO cities (name) VALUES ('LeRobert');
INSERT INTO cities (name) VALUES ('LeTampon');
INSERT INTO cities (name) VALUES ('Lhokseumawe');
INSERT INTO cities (name) VALUES ('Lianran');
INSERT INTO cities (name) VALUES ('Liaocheng');
INSERT INTO cities (name) VALUES ('Liaoyang');
INSERT INTO cities (name) VALUES ('Liaoyuan');
INSERT INTO cities (name) VALUES ('Liberec');
INSERT INTO cities (name) VALUES ('Liberia');
INSERT INTO cities (name) VALUES ('Libreville');
INSERT INTO cities (name) VALUES ('Licheng');
INSERT INTO cities (name) VALUES ('Lichinga');
INSERT INTO cities (name) VALUES ('Liege');
INSERT INTO cities (name) VALUES ('Liepaja');
INSERT INTO cities (name) VALUES ('Likasi');
INSERT INTO cities (name) VALUES ('Lille');
INSERT INTO cities (name) VALUES ('Lilongwe');
INSERT INTO cities (name) VALUES ('Lima');
INSERT INTO cities (name) VALUES ('Limeira');
INSERT INTO cities (name) VALUES ('Limerick');
INSERT INTO cities (name) VALUES ('Limmasol');
INSERT INTO cities (name) VALUES ('Limoges');
INSERT INTO cities (name) VALUES ('Limon');
INSERT INTO cities (name) VALUES ('Linchuan');
INSERT INTO cities (name) VALUES ('Lincoln');
INSERT INTO cities (name) VALUES ('Linden');
INSERT INTO cities (name) VALUES ('Linfen');
INSERT INTO cities (name) VALUES ('Linhai');
INSERT INTO cities (name) VALUES ('Linhe');
INSERT INTO cities (name) VALUES ('Linkoping');
INSERT INTO cities (name) VALUES ('Linqing');
INSERT INTO cities (name) VALUES ('Linshui');
INSERT INTO cities (name) VALUES ('Linxi');
INSERT INTO cities (name) VALUES ('Linyi');
INSERT INTO cities (name) VALUES ('Linz');
INSERT INTO cities (name) VALUES ('Lipetsk');
INSERT INTO cities (name) VALUES ('Lisboa');
INSERT INTO cities (name) VALUES ('Lishui');
INSERT INTO cities (name) VALUES ('Little Rock');
INSERT INTO cities (name) VALUES ('Littlehampton');
INSERT INTO cities (name) VALUES ('Liupanshui');
INSERT INTO cities (name) VALUES ('Liuzhou');
INSERT INTO cities (name) VALUES ('Livermore');
INSERT INTO cities (name) VALUES ('Liverpool');
INSERT INTO cities (name) VALUES ('Livingstone');
INSERT INTO cities (name) VALUES ('Livonia');
INSERT INTO cities (name) VALUES ('Livorno');
INSERT INTO cities (name) VALUES ('Ljubljana');
INSERT INTO cities (name) VALUES ('Llanelli');
INSERT INTO cities (name) VALUES ('Lleida');
INSERT INTO cities (name) VALUES ('Lobamba');
INSERT INTO cities (name) VALUES ('Lobito');
INSERT INTO cities (name) VALUES ('Locks Heath');
INSERT INTO cities (name) VALUES ('Lodz');
INSERT INTO cities (name) VALUES ('Logan');
INSERT INTO cities (name) VALUES ('Logrono');
INSERT INTO cities (name) VALUES ('Loja');
INSERT INTO cities (name) VALUES ('Lolua');
INSERT INTO cities (name) VALUES ('Lome');
INSERT INTO cities (name) VALUES ('Lompoc');
INSERT INTO cities (name) VALUES ('London');
INSERT INTO cities (name) VALUES ('Londonderry');
INSERT INTO cities (name) VALUES ('Londrina');
INSERT INTO cities (name) VALUES ('Long Beach');
INSERT INTO cities (name) VALUES ('Longana');
INSERT INTO cities (name) VALUES ('Longfeng');
INSERT INTO cities (name) VALUES ('Longjiang');
INSERT INTO cities (name) VALUES ('Longueuil');
INSERT INTO cities (name) VALUES ('Longview');
INSERT INTO cities (name) VALUES ('Longyan');
INSERT INTO cities (name) VALUES ('Longyearbyen');
INSERT INTO cities (name) VALUES ('LongXuyen');
INSERT INTO cities (name) VALUES ('Lorient');
INSERT INTO cities (name) VALUES ('Los Angeles');
INSERT INTO cities (name) VALUES ('Los Baos');
INSERT INTO cities (name) VALUES ('Los Mochis');
INSERT INTO cities (name) VALUES ('Loubomo');
INSERT INTO cities (name) VALUES ('Loudi');
INSERT INTO cities (name) VALUES ('Loughborough');
INSERT INTO cities (name) VALUES ('Louisville');
INSERT INTO cities (name) VALUES ('Lowell');
INSERT INTO cities (name) VALUES ('Lower Merion');
INSERT INTO cities (name) VALUES ('Lowestoft');
INSERT INTO cities (name) VALUES ('Luan');
INSERT INTO cities (name) VALUES ('Luanda');
INSERT INTO cities (name) VALUES ('LuangPrabang');
INSERT INTO cities (name) VALUES ('Luanshya');
INSERT INTO cities (name) VALUES ('Lubango');
INSERT INTO cities (name) VALUES ('Lubbock');
INSERT INTO cities (name) VALUES ('Lubeck');
INSERT INTO cities (name) VALUES ('Lublin');
INSERT INTO cities (name) VALUES ('Lubumbashi');
INSERT INTO cities (name) VALUES ('Lucena');
INSERT INTO cities (name) VALUES ('Lucerne');
INSERT INTO cities (name) VALUES ('Lucknow');
INSERT INTO cities (name) VALUES ('Ludhiana');
INSERT INTO cities (name) VALUES ('Ludwigshafen');
INSERT INTO cities (name) VALUES ('Luganville');
INSERT INTO cities (name) VALUES ('Luhansk');
INSERT INTO cities (name) VALUES ('Luohe');
INSERT INTO cities (name) VALUES ('Luoyang');
INSERT INTO cities (name) VALUES ('Lusaka');
INSERT INTO cities (name) VALUES ('Luton');
INSERT INTO cities (name) VALUES ('Lutsk');
INSERT INTO cities (name) VALUES ('Luxembourg');
INSERT INTO cities (name) VALUES ('Luxor');
INSERT INTO cities (name) VALUES ('Luzhou');
INSERT INTO cities (name) VALUES ('Luziania');
INSERT INTO cities (name) VALUES ('Lynchburg');
INSERT INTO cities (name) VALUES ('Lynwood');
INSERT INTO cities (name) VALUES ('Lyon');
INSERT INTO cities (name) VALUES ('Lysychansk');
INSERT INTO cities (name) VALUES ('Lytham Saint Anne\'s');
INSERT INTO cities (name) VALUES ('Lyubercy');
INSERT INTO cities (name) VALUES ('M\'banza-Kongo');
INSERT INTO cities (name) VALUES ('Ma\'an');
INSERT INTO cities (name) VALUES ('Maanshan');
INSERT INTO cities (name) VALUES ('Maastricht');
INSERT INTO cities (name) VALUES ('Macae');
INSERT INTO cities (name) VALUES ('MacAllen');
INSERT INTO cities (name) VALUES ('Macapa');
INSERT INTO cities (name) VALUES ('Macclesfield');
INSERT INTO cities (name) VALUES ('Maceio');
INSERT INTO cities (name) VALUES ('Machakos');
INSERT INTO cities (name) VALUES ('Machala');
INSERT INTO cities (name) VALUES ('Macheng');
INSERT INTO cities (name) VALUES ('MacMurdo');
INSERT INTO cities (name) VALUES ('Macon');
INSERT INTO cities (name) VALUES ('Madaba');
INSERT INTO cities (name) VALUES ('Madang');
INSERT INTO cities (name) VALUES ('Madison');
INSERT INTO cities (name) VALUES ('Madiun');
INSERT INTO cities (name) VALUES ('Madrid');
INSERT INTO cities (name) VALUES ('Madurai');
INSERT INTO cities (name) VALUES ('Maebashi');
INSERT INTO cities (name) VALUES ('Mafeteng');
INSERT INTO cities (name) VALUES ('Mafraw');
INSERT INTO cities (name) VALUES ('Magadan');
INSERT INTO cities (name) VALUES ('Magdeburg');
INSERT INTO cities (name) VALUES ('Mage');
INSERT INTO cities (name) VALUES ('Magelang');
INSERT INTO cities (name) VALUES ('Magnitogorsk');
INSERT INTO cities (name) VALUES ('Mahabad');
INSERT INTO cities (name) VALUES ('Mahackala');
INSERT INTO cities (name) VALUES ('Mahajanga');
INSERT INTO cities (name) VALUES ('Mahebourg');
INSERT INTO cities (name) VALUES ('Mahendranagar');
INSERT INTO cities (name) VALUES ('Maheshtala');
INSERT INTO cities (name) VALUES ('Maicao');
INSERT INTO cities (name) VALUES ('Maiduguri');
INSERT INTO cities (name) VALUES ('Mainz');
INSERT INTO cities (name) VALUES ('Majie');
INSERT INTO cities (name) VALUES ('Makeni');
INSERT INTO cities (name) VALUES ('Makin');
INSERT INTO cities (name) VALUES ('Makiyivka');
INSERT INTO cities (name) VALUES ('Makurdi');
INSERT INTO cities (name) VALUES ('Malabo');
INSERT INTO cities (name) VALUES ('Malaga');
INSERT INTO cities (name) VALUES ('Malakal');
INSERT INTO cities (name) VALUES ('Malang');
INSERT INTO cities (name) VALUES ('Malanje');
INSERT INTO cities (name) VALUES ('Malatya');
INSERT INTO cities (name) VALUES ('Malayer');
INSERT INTO cities (name) VALUES ('Male');
INSERT INTO cities (name) VALUES ('Malegaon');
INSERT INTO cities (name) VALUES ('Maliana');
INSERT INTO cities (name) VALUES ('Mallawi');
INSERT INTO cities (name) VALUES ('Malmo');
INSERT INTO cities (name) VALUES ('Malolos');
INSERT INTO cities (name) VALUES ('Mamou');
INSERT INTO cities (name) VALUES ('Mamoudzou');
INSERT INTO cities (name) VALUES ('Man');
INSERT INTO cities (name) VALUES ('Manado');
INSERT INTO cities (name) VALUES ('Managua');
INSERT INTO cities (name) VALUES ('Manamah');
INSERT INTO cities (name) VALUES ('Manaus');
INSERT INTO cities (name) VALUES ('Manbij');
INSERT INTO cities (name) VALUES ('Manchester');
INSERT INTO cities (name) VALUES ('Mandalay');
INSERT INTO cities (name) VALUES ('Mandi Bahauddin');
INSERT INTO cities (name) VALUES ('Mangaluru');
INSERT INTO cities (name) VALUES ('Mangilao');
INSERT INTO cities (name) VALUES ('Manhattan');
INSERT INTO cities (name) VALUES ('Maniitsoq');
INSERT INTO cities (name) VALUES ('Manila');
INSERT INTO cities (name) VALUES ('Manisa');
INSERT INTO cities (name) VALUES ('Manizales');
INSERT INTO cities (name) VALUES ('Mannheim');
INSERT INTO cities (name) VALUES ('Mansfield');
INSERT INTO cities (name) VALUES ('Mansura');
INSERT INTO cities (name) VALUES ('Manta');
INSERT INTO cities (name) VALUES ('Manzanillo');
INSERT INTO cities (name) VALUES ('Manzini');
INSERT INTO cities (name) VALUES ('Maoming');
INSERT INTO cities (name) VALUES ('Maputo');
INSERT INTO cities (name) VALUES ('Maputsoa');
INSERT INTO cities (name) VALUES ('Maraba');
INSERT INTO cities (name) VALUES ('Maracaibo');
INSERT INTO cities (name) VALUES ('Maracanau');
INSERT INTO cities (name) VALUES ('Maracay');
INSERT INTO cities (name) VALUES ('Maradhoo');
INSERT INTO cities (name) VALUES ('Maradi');
INSERT INTO cities (name) VALUES ('Maragheh');
INSERT INTO cities (name) VALUES ('Marawi');
INSERT INTO cities (name) VALUES ('Marbella');
INSERT INTO cities (name) VALUES ('Mardan');
INSERT INTO cities (name) VALUES ('Margate');
INSERT INTO cities (name) VALUES ('Maribor');
INSERT INTO cities (name) VALUES ('Marigot');
INSERT INTO cities (name) VALUES ('Marilia');
INSERT INTO cities (name) VALUES ('Maringa');
INSERT INTO cities (name) VALUES ('Mariupol\'');
INSERT INTO cities (name) VALUES ('Marka');
INSERT INTO cities (name) VALUES ('Markham');
INSERT INTO cities (name) VALUES ('Marondera');
INSERT INTO cities (name) VALUES ('Maroua');
INSERT INTO cities (name) VALUES ('Marrakech');
INSERT INTO cities (name) VALUES ('Marseille');
INSERT INTO cities (name) VALUES ('Martapura');
INSERT INTO cities (name) VALUES ('Martin-de-Vivies');
INSERT INTO cities (name) VALUES ('MarvDasht');
INSERT INTO cities (name) VALUES ('Mary');
INSERT INTO cities (name) VALUES ('MardelPlata');
INSERT INTO cities (name) VALUES ('Masaka');
INSERT INTO cities (name) VALUES ('Masan');
INSERT INTO cities (name) VALUES ('Masaya');
INSERT INTO cities (name) VALUES ('Maseru');
INSERT INTO cities (name) VALUES ('Mashhad');
INSERT INTO cities (name) VALUES ('Masjed-eSoleyman');
INSERT INTO cities (name) VALUES ('Masqat-Matrah');
INSERT INTO cities (name) VALUES ('Masuku');
INSERT INTO cities (name) VALUES ('Masvingo');
INSERT INTO cities (name) VALUES ('Mata Utu');
INSERT INTO cities (name) VALUES ('Matadi');
INSERT INTO cities (name) VALUES ('Matagalpa');
INSERT INTO cities (name) VALUES ('Matamoros');
INSERT INTO cities (name) VALUES ('Matanzas');
INSERT INTO cities (name) VALUES ('Mataram');
INSERT INTO cities (name) VALUES ('Matola');
INSERT INTO cities (name) VALUES ('Matoury');
INSERT INTO cities (name) VALUES ('Matsudo');
INSERT INTO cities (name) VALUES ('Matsumoto');
INSERT INTO cities (name) VALUES ('Matsuyama');
INSERT INTO cities (name) VALUES ('Maturin');
INSERT INTO cities (name) VALUES ('Maua');
INSERT INTO cities (name) VALUES ('Maun');
INSERT INTO cities (name) VALUES ('Mawlamyine');
INSERT INTO cities (name) VALUES ('Mawson');
INSERT INTO cities (name) VALUES ('Maxixe');
INSERT INTO cities (name) VALUES ('Mayaguez');
INSERT INTO cities (name) VALUES ('Maykop');
INSERT INTO cities (name) VALUES ('Maymyo');
INSERT INTO cities (name) VALUES ('MayPen');
INSERT INTO cities (name) VALUES ('Mazar-I-Sharif');
INSERT INTO cities (name) VALUES ('Mazatlan');
INSERT INTO cities (name) VALUES ('Mbabane');
INSERT INTO cities (name) VALUES ('Mbale');
INSERT INTO cities (name) VALUES ('Mbandaka');
INSERT INTO cities (name) VALUES ('Mbeya');
INSERT INTO cities (name) VALUES ('Mbour');
INSERT INTO cities (name) VALUES ('Mbuji-Mayi');
INSERT INTO cities (name) VALUES ('Mecca');
INSERT INTO cities (name) VALUES ('Medan');
INSERT INTO cities (name) VALUES ('Medellin');
INSERT INTO cities (name) VALUES ('Medford');
INSERT INTO cities (name) VALUES ('Medicine Hat');
INSERT INTO cities (name) VALUES ('Medinah');
INSERT INTO cities (name) VALUES ('Meedhoo');
INSERT INTO cities (name) VALUES ('Meihekou');
INSERT INTO cities (name) VALUES ('Meiktila');
INSERT INTO cities (name) VALUES ('Meizhou');
INSERT INTO cities (name) VALUES ('Mek\'ele');
INSERT INTO cities (name) VALUES ('Meknes');
INSERT INTO cities (name) VALUES ('Melaka');
INSERT INTO cities (name) VALUES ('Melbourne');
INSERT INTO cities (name) VALUES ('Melekeok');
INSERT INTO cities (name) VALUES ('Melipilla');
INSERT INTO cities (name) VALUES ('Melitpol\'');
INSERT INTO cities (name) VALUES ('Melo');
INSERT INTO cities (name) VALUES ('Melton Mowbray');
INSERT INTO cities (name) VALUES ('Memphis');
INSERT INTO cities (name) VALUES ('Mendoza');
INSERT INTO cities (name) VALUES ('Mentougou');
INSERT INTO cities (name) VALUES ('Merced');
INSERT INTO cities (name) VALUES ('Mercedes');
INSERT INTO cities (name) VALUES ('Mergui');
INSERT INTO cities (name) VALUES ('Merida');
INSERT INTO cities (name) VALUES ('Meriden');
INSERT INTO cities (name) VALUES ('Meridian');
INSERT INTO cities (name) VALUES ('Mersin');
INSERT INTO cities (name) VALUES ('Merthyr Tydfil');
INSERT INTO cities (name) VALUES ('Meru');
INSERT INTO cities (name) VALUES ('Mesa');
INSERT INTO cities (name) VALUES ('Mesquite');
INSERT INTO cities (name) VALUES ('Messina');
INSERT INTO cities (name) VALUES ('Metairie');
INSERT INTO cities (name) VALUES ('Metro');
INSERT INTO cities (name) VALUES ('Metz');
INSERT INTO cities (name) VALUES ('Mexicali');
INSERT INTO cities (name) VALUES ('Mexico City');
INSERT INTO cities (name) VALUES ('Meyungs');
INSERT INTO cities (name) VALUES ('Mezhdurechensk');
INSERT INTO cities (name) VALUES ('Mhlume');
INSERT INTO cities (name) VALUES ('Miami');
INSERT INTO cities (name) VALUES ('Miami Beach');
INSERT INTO cities (name) VALUES ('Mianchang');
INSERT INTO cities (name) VALUES ('Miandoab');
INSERT INTO cities (name) VALUES ('Mianyang');
INSERT INTO cities (name) VALUES ('Miass');
INSERT INTO cities (name) VALUES ('Michurinsk');
INSERT INTO cities (name) VALUES ('Micoud');
INSERT INTO cities (name) VALUES ('Middelburg');
INSERT INTO cities (name) VALUES ('Middlesbrough');
INSERT INTO cities (name) VALUES ('Midland');
INSERT INTO cities (name) VALUES ('Milagro');
INSERT INTO cities (name) VALUES ('Milano');
INSERT INTO cities (name) VALUES ('Milpitas');
INSERT INTO cities (name) VALUES ('Milton Keynes');
INSERT INTO cities (name) VALUES ('Milwaukee');
INSERT INTO cities (name) VALUES ('Mindelo');
INSERT INTO cities (name) VALUES ('Mingaora');
INSERT INTO cities (name) VALUES ('Mingshui');
INSERT INTO cities (name) VALUES ('Minhang');
INSERT INTO cities (name) VALUES ('Minna');
INSERT INTO cities (name) VALUES ('Minneapolis');
INSERT INTO cities (name) VALUES ('Minnetonka');
INSERT INTO cities (name) VALUES ('Minsk');
INSERT INTO cities (name) VALUES ('Miquelon');
INSERT INTO cities (name) VALUES ('Mira Bhayandar');
INSERT INTO cities (name) VALUES ('Miramar');
INSERT INTO cities (name) VALUES ('Mirat');
INSERT INTO cities (name) VALUES ('Miri');
INSERT INTO cities (name) VALUES ('Mirnyj');
INSERT INTO cities (name) VALUES ('Mirpur Khas');
INSERT INTO cities (name) VALUES ('Mirzapur');
INSERT INTO cities (name) VALUES ('Mishawaka');
INSERT INTO cities (name) VALUES ('Miskolc');
INSERT INTO cities (name) VALUES ('Misratah');
INSERT INTO cities (name) VALUES ('Mission Viejo');
INSERT INTO cities (name) VALUES ('Mississauga');
INSERT INTO cities (name) VALUES ('Missoula');
INSERT INTO cities (name) VALUES ('Mit Gamr');
INSERT INTO cities (name) VALUES ('Mito');
INSERT INTO cities (name) VALUES ('Mitsamiouli');
INSERT INTO cities (name) VALUES ('Miyazaki');
INSERT INTO cities (name) VALUES ('Moanda');
INSERT INTO cities (name) VALUES ('Mobile');
INSERT INTO cities (name) VALUES ('Mocuba');
INSERT INTO cities (name) VALUES ('Modakeke');
INSERT INTO cities (name) VALUES ('Modena');
INSERT INTO cities (name) VALUES ('Modesto');
INSERT INTO cities (name) VALUES ('Moers');
INSERT INTO cities (name) VALUES ('Mogadishu');
INSERT INTO cities (name) VALUES ('Mogilev');
INSERT INTO cities (name) VALUES ('MogiGuacu');
INSERT INTO cities (name) VALUES ('MojidasCruzes');
INSERT INTO cities (name) VALUES ('Mojokerto');
INSERT INTO cities (name) VALUES ('Mokpo');
INSERT INTO cities (name) VALUES ('Molepolole');
INSERT INTO cities (name) VALUES ('Moline');
INSERT INTO cities (name) VALUES ('Molodezhnaja');
INSERT INTO cities (name) VALUES ('Mombasa');
INSERT INTO cities (name) VALUES ('Monchengladbach');
INSERT INTO cities (name) VALUES ('Monclova');
INSERT INTO cities (name) VALUES ('Moncton');
INSERT INTO cities (name) VALUES ('Monroe');
INSERT INTO cities (name) VALUES ('Monrovia');
INSERT INTO cities (name) VALUES ('Montauban');
INSERT INTO cities (name) VALUES ('Montebello');
INSERT INTO cities (name) VALUES ('MontegoBay');
INSERT INTO cities (name) VALUES ('Monterey Park');
INSERT INTO cities (name) VALUES ('Monteria');
INSERT INTO cities (name) VALUES ('Monterrey');
INSERT INTO cities (name) VALUES ('MontesClaros');
INSERT INTO cities (name) VALUES ('Montevideo');
INSERT INTO cities (name) VALUES ('MonteCarlo');
INSERT INTO cities (name) VALUES ('Montgomery');
INSERT INTO cities (name) VALUES ('Montpelier');
INSERT INTO cities (name) VALUES ('Montpellier');
INSERT INTO cities (name) VALUES ('Montreal');
INSERT INTO cities (name) VALUES ('Monywa');
INSERT INTO cities (name) VALUES ('Moose Jaw');
INSERT INTO cities (name) VALUES ('Mopti');
INSERT INTO cities (name) VALUES ('Moradabad');
INSERT INTO cities (name) VALUES ('Morecambe');
INSERT INTO cities (name) VALUES ('Morelia');
INSERT INTO cities (name) VALUES ('Moreno Valley');
INSERT INTO cities (name) VALUES ('Morioka');
INSERT INTO cities (name) VALUES ('Morne-a-l\'Eau');
INSERT INTO cities (name) VALUES ('Morogoro');
INSERT INTO cities (name) VALUES ('Moroni');
INSERT INTO cities (name) VALUES ('Moshi');
INSERT INTO cities (name) VALUES ('Moskva');
INSERT INTO cities (name) VALUES ('Mossendjo');
INSERT INTO cities (name) VALUES ('Mossoro');
INSERT INTO cities (name) VALUES ('Mosta');
INSERT INTO cities (name) VALUES ('Mostar');
INSERT INTO cities (name) VALUES ('Mosul');
INSERT INTO cities (name) VALUES ('Moundou');
INSERT INTO cities (name) VALUES ('Mountain View');
INSERT INTO cities (name) VALUES ('Mozir');
INSERT INTO cities (name) VALUES ('Mtwara');
INSERT INTO cities (name) VALUES ('Mu\'a');
INSERT INTO cities (name) VALUES ('Mubi');
INSERT INTO cities (name) VALUES ('Mudanjiang');
INSERT INTO cities (name) VALUES ('Muenster');
INSERT INTO cities (name) VALUES ('Mufulira');
INSERT INTO cities (name) VALUES ('Mukalla');
INSERT INTO cities (name) VALUES ('Mulheim');
INSERT INTO cities (name) VALUES ('Mulhouse');
INSERT INTO cities (name) VALUES ('Multan');
INSERT INTO cities (name) VALUES ('Mumbai (Bombay)');
INSERT INTO cities (name) VALUES ('Muncie');
INSERT INTO cities (name) VALUES ('Munich');
INSERT INTO cities (name) VALUES ('Murcia');
INSERT INTO cities (name) VALUES ('Murfreesboro');
INSERT INTO cities (name) VALUES ('Muridke');
INSERT INTO cities (name) VALUES ('Murmansk');
INSERT INTO cities (name) VALUES ('Murom');
INSERT INTO cities (name) VALUES ('Murzuq');
INSERT INTO cities (name) VALUES ('Musayid');
INSERT INTO cities (name) VALUES ('Musoma');
INSERT INTO cities (name) VALUES ('Mutare');
INSERT INTO cities (name) VALUES ('Mutsamudu');
INSERT INTO cities (name) VALUES ('Muyinga');
INSERT INTO cities (name) VALUES ('Muzaffargarh');
INSERT INTO cities (name) VALUES ('Muzaffarnagar');
INSERT INTO cities (name) VALUES ('Muzaffarpur');
INSERT INTO cities (name) VALUES ('Mwanza');
INSERT INTO cities (name) VALUES ('Mwene-Ditu');
INSERT INTO cities (name) VALUES ('Myingyan');
INSERT INTO cities (name) VALUES ('Mykolayiv');
INSERT INTO cities (name) VALUES ('Mysore');
INSERT INTO cities (name) VALUES ('Mytishchi');
INSERT INTO cities (name) VALUES ('Mzuzu');
INSERT INTO cities (name) VALUES ('N\'Djamena');
INSERT INTO cities (name) VALUES ('Nabari');
INSERT INTO cities (name) VALUES ('Naberezhnyye Chelny');
INSERT INTO cities (name) VALUES ('Nablus');
INSERT INTO cities (name) VALUES ('Nacala');
INSERT INTO cities (name) VALUES ('Nadi');
INSERT INTO cities (name) VALUES ('Nador');
INSERT INTO cities (name) VALUES ('Naga');
INSERT INTO cities (name) VALUES ('Nagano');
INSERT INTO cities (name) VALUES ('Nagaoka');
INSERT INTO cities (name) VALUES ('Nagasaki');
INSERT INTO cities (name) VALUES ('Nagercoil');
INSERT INTO cities (name) VALUES ('Nagoya');
INSERT INTO cities (name) VALUES ('Nagpur');
INSERT INTO cities (name) VALUES ('Naha');
INSERT INTO cities (name) VALUES ('Nahodka');
INSERT INTO cities (name) VALUES ('Naifaru');
INSERT INTO cities (name) VALUES ('Nairobi');
INSERT INTO cities (name) VALUES ('Najran');
INSERT INTO cities (name) VALUES ('NakhonPathom');
INSERT INTO cities (name) VALUES ('NakhonSiThammarat');
INSERT INTO cities (name) VALUES ('Nakuru');
INSERT INTO cities (name) VALUES ('Nalchik');
INSERT INTO cities (name) VALUES ('Namangan');
INSERT INTO cities (name) VALUES ('Namibe');
INSERT INTO cities (name) VALUES ('Nampo');
INSERT INTO cities (name) VALUES ('Nampula');
INSERT INTO cities (name) VALUES ('Namur');
INSERT INTO cities (name) VALUES ('NamDinh');
INSERT INTO cities (name) VALUES ('Nanaimo');
INSERT INTO cities (name) VALUES ('Nancha');
INSERT INTO cities (name) VALUES ('Nanchang');
INSERT INTO cities (name) VALUES ('Nanchong');
INSERT INTO cities (name) VALUES ('Nancy');
INSERT INTO cities (name) VALUES ('Nanded');
INSERT INTO cities (name) VALUES ('Nanjing');
INSERT INTO cities (name) VALUES ('Nanning');
INSERT INTO cities (name) VALUES ('Nanortalik');
INSERT INTO cities (name) VALUES ('Nanping');
INSERT INTO cities (name) VALUES ('Nantes');
INSERT INTO cities (name) VALUES ('Nantong');
INSERT INTO cities (name) VALUES ('Nantou');
INSERT INTO cities (name) VALUES ('Nanyang');
INSERT INTO cities (name) VALUES ('Napa');
INSERT INTO cities (name) VALUES ('Napasoq');
INSERT INTO cities (name) VALUES ('Naperville');
INSERT INTO cities (name) VALUES ('Napier');
INSERT INTO cities (name) VALUES ('Napoli');
INSERT INTO cities (name) VALUES ('Narayanganj');
INSERT INTO cities (name) VALUES ('Narsaq');
INSERT INTO cities (name) VALUES ('Narsarmijit');
INSERT INTO cities (name) VALUES ('Narsarsuaq');
INSERT INTO cities (name) VALUES ('Narsingdi');
INSERT INTO cities (name) VALUES ('Narva');
INSERT INTO cities (name) VALUES ('Naryn');
INSERT INTO cities (name) VALUES ('Nashik');
INSERT INTO cities (name) VALUES ('Nashville');
INSERT INTO cities (name) VALUES ('Nasirabad');
INSERT INTO cities (name) VALUES ('Nasirah');
INSERT INTO cities (name) VALUES ('Nassau');
INSERT INTO cities (name) VALUES ('Natal');
INSERT INTO cities (name) VALUES ('Natitingou');
INSERT INTO cities (name) VALUES ('Naucalpan');
INSERT INTO cities (name) VALUES ('Navajoa');
INSERT INTO cities (name) VALUES ('Navi Mumbai');
INSERT INTO cities (name) VALUES ('Navoi');
INSERT INTO cities (name) VALUES ('Nawabganj');
INSERT INTO cities (name) VALUES ('Nawabshah');
INSERT INTO cities (name) VALUES ('Nazerat');
INSERT INTO cities (name) VALUES ('Nazerat Illit');
INSERT INTO cities (name) VALUES ('Nazilli');
INSERT INTO cities (name) VALUES ('Nazran');
INSERT INTO cities (name) VALUES ('Nazret');
INSERT INTO cities (name) VALUES ('Ndola');
INSERT INTO cities (name) VALUES ('Neath');
INSERT INTO cities (name) VALUES ('Neftekamsk');
INSERT INTO cities (name) VALUES ('Nefteyugansk');
INSERT INTO cities (name) VALUES ('Nehe');
INSERT INTO cities (name) VALUES ('Neiafu');
INSERT INTO cities (name) VALUES ('Neijiang');
INSERT INTO cities (name) VALUES ('Neiva');
INSERT INTO cities (name) VALUES ('Nek\'emte');
INSERT INTO cities (name) VALUES ('Nelson');
INSERT INTO cities (name) VALUES ('Nelspruit');
INSERT INTO cities (name) VALUES ('Nepean');
INSERT INTO cities (name) VALUES ('Neumayer');
INSERT INTO cities (name) VALUES ('Neuquen');
INSERT INTO cities (name) VALUES ('Neuss');
INSERT INTO cities (name) VALUES ('Nevinnomyssk');
INSERT INTO cities (name) VALUES ('New Haven');
INSERT INTO cities (name) VALUES ('New Milton-Barton-on-Sea');
INSERT INTO cities (name) VALUES ('New Orleans');
INSERT INTO cities (name) VALUES ('New Westminster');
INSERT INTO cities (name) VALUES ('New York');
INSERT INTO cities (name) VALUES ('Newark');
INSERT INTO cities (name) VALUES ('Newark-on-Trent');
INSERT INTO cities (name) VALUES ('Newbury');
INSERT INTO cities (name) VALUES ('Newcastle');
INSERT INTO cities (name) VALUES ('Newcastle upon Tyne');
INSERT INTO cities (name) VALUES ('Newcastle-under-Lyme');
INSERT INTO cities (name) VALUES ('Newport');
INSERT INTO cities (name) VALUES ('Newport Beach');
INSERT INTO cities (name) VALUES ('Newport News');
INSERT INTO cities (name) VALUES ('Newry');
INSERT INTO cities (name) VALUES ('Newton Abbot');
INSERT INTO cities (name) VALUES ('NewAmsterdam');
INSERT INTO cities (name) VALUES ('NewPlymouth');
INSERT INTO cities (name) VALUES ('Neyshabur');
INSERT INTO cities (name) VALUES ('Nezahualcoyotl');
INSERT INTO cities (name) VALUES ('Ngaramash');
INSERT INTO cities (name) VALUES ('Ngerkeai');
INSERT INTO cities (name) VALUES ('Ngermechau');
INSERT INTO cities (name) VALUES ('Ngetkip');
INSERT INTO cities (name) VALUES ('Ngozi');
INSERT INTO cities (name) VALUES ('NhaTrang');
INSERT INTO cities (name) VALUES ('Ni Dilli');
INSERT INTO cities (name) VALUES ('Niamey');
INSERT INTO cities (name) VALUES ('Niaqornaarsuk');
INSERT INTO cities (name) VALUES ('Nice');
INSERT INTO cities (name) VALUES ('Nicosia');
INSERT INTO cities (name) VALUES ('NieuwNickerie');
INSERT INTO cities (name) VALUES ('Nigel');
INSERT INTO cities (name) VALUES ('Niigata');
INSERT INTO cities (name) VALUES ('Nijmegen');
INSERT INTO cities (name) VALUES ('Nikopol\'');
INSERT INTO cities (name) VALUES ('Nilopolis');
INSERT INTO cities (name) VALUES ('Nimes');
INSERT INTO cities (name) VALUES ('Ningbo');
INSERT INTO cities (name) VALUES ('Niort');
INSERT INTO cities (name) VALUES ('Nish');
INSERT INTO cities (name) VALUES ('Niteroi');
INSERT INTO cities (name) VALUES ('Nitra');
INSERT INTO cities (name) VALUES ('Nizamabad');
INSERT INTO cities (name) VALUES ('Nizhnekamsk');
INSERT INTO cities (name) VALUES ('Nizhnevartovsk');
INSERT INTO cities (name) VALUES ('Nizhniy Novgorod');
INSERT INTO cities (name) VALUES ('Nizhniy Tagil');
INSERT INTO cities (name) VALUES ('Nizwa');
INSERT INTO cities (name) VALUES ('Nkayi');
INSERT INTO cities (name) VALUES ('Nkongsamba');
INSERT INTO cities (name) VALUES ('Nogales');
INSERT INTO cities (name) VALUES ('Nola');
INSERT INTO cities (name) VALUES ('Nongan');
INSERT INTO cities (name) VALUES ('Nonthaburi');
INSERT INTO cities (name) VALUES ('Norfolk');
INSERT INTO cities (name) VALUES ('Norilsk');
INSERT INTO cities (name) VALUES ('Normal');
INSERT INTO cities (name) VALUES ('Norman');
INSERT INTO cities (name) VALUES ('Norrkoping');
INSERT INTO cities (name) VALUES ('Norsup');
INSERT INTO cities (name) VALUES ('North Fort Myers');
INSERT INTO cities (name) VALUES ('Northampton');
INSERT INTO cities (name) VALUES ('Northwich');
INSERT INTO cities (name) VALUES ('Norwalk');
INSERT INTO cities (name) VALUES ('Norwich');
INSERT INTO cities (name) VALUES ('NossaSenhoradoSocorro');
INSERT INTO cities (name) VALUES ('Nottingham');
INSERT INTO cities (name) VALUES ('Nouadhibou');
INSERT INTO cities (name) VALUES ('Nouakchott');
INSERT INTO cities (name) VALUES ('Noumea');
INSERT INTO cities (name) VALUES ('Novara');
INSERT INTO cities (name) VALUES ('NovaFriburgo');
INSERT INTO cities (name) VALUES ('NovaIguacu');
INSERT INTO cities (name) VALUES ('NoviSad');
INSERT INTO cities (name) VALUES ('Novocheboksarsk');
INSERT INTO cities (name) VALUES ('Novocherkassk');
INSERT INTO cities (name) VALUES ('Novokuznetsk');
INSERT INTO cities (name) VALUES ('Novolazarovskaja');
INSERT INTO cities (name) VALUES ('Novomoskovsk');
INSERT INTO cities (name) VALUES ('Novorossiysk');
INSERT INTO cities (name) VALUES ('Novoshahtinsk');
INSERT INTO cities (name) VALUES ('Novosibirsk');
INSERT INTO cities (name) VALUES ('Novotroitsk');
INSERT INTO cities (name) VALUES ('Novouralsk');
INSERT INTO cities (name) VALUES ('NovoHamburgo');
INSERT INTO cities (name) VALUES ('Noyabrsk');
INSERT INTO cities (name) VALUES ('Nsukka');
INSERT INTO cities (name) VALUES ('Nu\'uuli');
INSERT INTO cities (name) VALUES ('Nuevo Laredo');
INSERT INTO cities (name) VALUES ('Nuku\'alofa');
INSERT INTO cities (name) VALUES ('Nukunonu');
INSERT INTO cities (name) VALUES ('Nukus');
INSERT INTO cities (name) VALUES ('Numazu');
INSERT INTO cities (name) VALUES ('Nurnberg');
INSERT INTO cities (name) VALUES ('Nuugaatsiaq');
INSERT INTO cities (name) VALUES ('Nuuk');
INSERT INTO cities (name) VALUES ('Nuusuaq');
INSERT INTO cities (name) VALUES ('Ny-Alesund');
INSERT INTO cities (name) VALUES ('Nyala');
INSERT INTO cities (name) VALUES ('Nyeri');
INSERT INTO cities (name) VALUES ('Nyiregyhaza');
INSERT INTO cities (name) VALUES ('Oakengates-Donnington');
INSERT INTO cities (name) VALUES ('Oakland');
INSERT INTO cities (name) VALUES ('Oakville');
INSERT INTO cities (name) VALUES ('Oaxaca');
INSERT INTO cities (name) VALUES ('Oberhausen');
INSERT INTO cities (name) VALUES ('Obihiro');
INSERT INTO cities (name) VALUES ('Obninsk');
INSERT INTO cities (name) VALUES ('Obuasi');
INSERT INTO cities (name) VALUES ('Ocala');
INSERT INTO cities (name) VALUES ('Oceanside');
INSERT INTO cities (name) VALUES ('Ocumare del Tuy');
INSERT INTO cities (name) VALUES ('Odawara');
INSERT INTO cities (name) VALUES ('Odense');
INSERT INTO cities (name) VALUES ('Odesa');
INSERT INTO cities (name) VALUES ('Odessa');
INSERT INTO cities (name) VALUES ('Odincovo');
INSERT INTO cities (name) VALUES ('Offa');
INSERT INTO cities (name) VALUES ('Offenbach');
INSERT INTO cities (name) VALUES ('Ogbomosho');
INSERT INTO cities (name) VALUES ('Ogden');
INSERT INTO cities (name) VALUES ('Oikul');
INSERT INTO cities (name) VALUES ('Oita');
INSERT INTO cities (name) VALUES ('Okara');
INSERT INTO cities (name) VALUES ('Okayama');
INSERT INTO cities (name) VALUES ('Okene');
INSERT INTO cities (name) VALUES ('Okigwe');
INSERT INTO cities (name) VALUES ('Oklahoma City');
INSERT INTO cities (name) VALUES ('Okpoko');
INSERT INTO cities (name) VALUES ('Oktyabrskiy');
INSERT INTO cities (name) VALUES ('Olathe');
INSERT INTO cities (name) VALUES ('Oldenburg');
INSERT INTO cities (name) VALUES ('Oldham');
INSERT INTO cities (name) VALUES ('OldRoad');
INSERT INTO cities (name) VALUES ('Olinda');
INSERT INTO cities (name) VALUES ('Ollei');
INSERT INTO cities (name) VALUES ('Olmaliq');
INSERT INTO cities (name) VALUES ('Olomouc');
INSERT INTO cities (name) VALUES ('Olongapo');
INSERT INTO cities (name) VALUES ('Olsztyn');
INSERT INTO cities (name) VALUES ('Olympia');
INSERT INTO cities (name) VALUES ('Omaha');
INSERT INTO cities (name) VALUES ('Omdurman');
INSERT INTO cities (name) VALUES ('Omiya');
INSERT INTO cities (name) VALUES ('Omsk');
INSERT INTO cities (name) VALUES ('Omuta');
INSERT INTO cities (name) VALUES ('Onchan');
INSERT INTO cities (name) VALUES ('Ondo');
INSERT INTO cities (name) VALUES ('Onitsha');
INSERT INTO cities (name) VALUES ('Ontario');
INSERT INTO cities (name) VALUES ('Ooma');
INSERT INTO cities (name) VALUES ('Opole');
INSERT INTO cities (name) VALUES ('Oradea');
INSERT INTO cities (name) VALUES ('Oral');
INSERT INTO cities (name) VALUES ('Oran');
INSERT INTO cities (name) VALUES ('Orange');
INSERT INTO cities (name) VALUES ('OrangeWalk');
INSERT INTO cities (name) VALUES ('Oranjestad');
INSERT INTO cities (name) VALUES ('Ordu');
INSERT INTO cities (name) VALUES ('Orebro');
INSERT INTO cities (name) VALUES ('Orehovo-Zuyevo');
INSERT INTO cities (name) VALUES ('Orem');
INSERT INTO cities (name) VALUES ('Orenburg');
INSERT INTO cities (name) VALUES ('Orizaba');
INSERT INTO cities (name) VALUES ('Orlando');
INSERT INTO cities (name) VALUES ('Orsa');
INSERT INTO cities (name) VALUES ('Orsk');
INSERT INTO cities (name) VALUES ('Orumiyeh');
INSERT INTO cities (name) VALUES ('Oruro');
INSERT INTO cities (name) VALUES ('Oryol');
INSERT INTO cities (name) VALUES ('Osaka');
INSERT INTO cities (name) VALUES ('Osasco');
INSERT INTO cities (name) VALUES ('Osh');
INSERT INTO cities (name) VALUES ('Oshakati');
INSERT INTO cities (name) VALUES ('Oshawa');
INSERT INTO cities (name) VALUES ('Oshkosh');
INSERT INTO cities (name) VALUES ('Oshogbo');
INSERT INTO cities (name) VALUES ('Osijek');
INSERT INTO cities (name) VALUES ('Oskemen');
INSERT INTO cities (name) VALUES ('Oslo');
INSERT INTO cities (name) VALUES ('Osmaniye');
INSERT INTO cities (name) VALUES ('Osnabruck');
INSERT INTO cities (name) VALUES ('Osorno');
INSERT INTO cities (name) VALUES ('Ostrava');
INSERT INTO cities (name) VALUES ('Ottawa');
INSERT INTO cities (name) VALUES ('Otukpo');
INSERT INTO cities (name) VALUES ('Ouagadougou');
INSERT INTO cities (name) VALUES ('Ouahigouya');
INSERT INTO cities (name) VALUES ('Ouesso');
INSERT INTO cities (name) VALUES ('Oujda');
INSERT INTO cities (name) VALUES ('Oulu');
INSERT INTO cities (name) VALUES ('Ourense');
INSERT INTO cities (name) VALUES ('Ovalle');
INSERT INTO cities (name) VALUES ('Overland Park');
INSERT INTO cities (name) VALUES ('Oviedo');
INSERT INTO cities (name) VALUES ('Owando');
INSERT INTO cities (name) VALUES ('Owensboro');
INSERT INTO cities (name) VALUES ('Owerri');
INSERT INTO cities (name) VALUES ('Owo');
INSERT INTO cities (name) VALUES ('Oxford');
INSERT INTO cities (name) VALUES ('Oxnard');
INSERT INTO cities (name) VALUES ('Oyama');
INSERT INTO cities (name) VALUES ('Oyem');
INSERT INTO cities (name) VALUES ('Oyo');
INSERT INTO cities (name) VALUES ('Paamiut');
INSERT INTO cities (name) VALUES ('Pabna');
INSERT INTO cities (name) VALUES ('Pachuca');
INSERT INTO cities (name) VALUES ('Padang');
INSERT INTO cities (name) VALUES ('Padang Sidempuan');
INSERT INTO cities (name) VALUES ('Paderborn');
INSERT INTO cities (name) VALUES ('Padova');
INSERT INTO cities (name) VALUES ('PagoPago');
INSERT INTO cities (name) VALUES ('Paignton');
INSERT INTO cities (name) VALUES ('Pakokku');
INSERT INTO cities (name) VALUES ('Pakpattan');
INSERT INTO cities (name) VALUES ('Pakxe');
INSERT INTO cities (name) VALUES ('Palangka Raya');
INSERT INTO cities (name) VALUES ('Palembang');
INSERT INTO cities (name) VALUES ('Palermo');
INSERT INTO cities (name) VALUES ('Palhoca');
INSERT INTO cities (name) VALUES ('Palikir');
INSERT INTO cities (name) VALUES ('Palm Bay');
INSERT INTO cities (name) VALUES ('Palma');
INSERT INTO cities (name) VALUES ('Palmas');
INSERT INTO cities (name) VALUES ('PalmaSoriano');
INSERT INTO cities (name) VALUES ('Palmdale');
INSERT INTO cities (name) VALUES ('Palmer');
INSERT INTO cities (name) VALUES ('PalmerstonNorth');
INSERT INTO cities (name) VALUES ('Palmira');
INSERT INTO cities (name) VALUES ('Palu');
INSERT INTO cities (name) VALUES ('Pamanzi');
INSERT INTO cities (name) VALUES ('Panama City');
INSERT INTO cities (name) VALUES ('Panchiao');
INSERT INTO cities (name) VALUES ('Panevezys');
INSERT INTO cities (name) VALUES ('Pangkal Pinang');
INSERT INTO cities (name) VALUES ('Panihati');
INSERT INTO cities (name) VALUES ('Panipat');
INSERT INTO cities (name) VALUES ('Panjin');
INSERT INTO cities (name) VALUES ('Panzhihua');
INSERT INTO cities (name) VALUES ('Papeete');
INSERT INTO cities (name) VALUES ('Paradise');
INSERT INTO cities (name) VALUES ('Parakou');
INSERT INTO cities (name) VALUES ('Paramaribo');
INSERT INTO cities (name) VALUES ('Paramount');
INSERT INTO cities (name) VALUES ('Parana');
INSERT INTO cities (name) VALUES ('Paranagua');
INSERT INTO cities (name) VALUES ('Parbhani');
INSERT INTO cities (name) VALUES ('Pardubice');
INSERT INTO cities (name) VALUES ('Paris');
INSERT INTO cities (name) VALUES ('Parma');
INSERT INTO cities (name) VALUES ('Parnaiba');
INSERT INTO cities (name) VALUES ('Parnamirim');
INSERT INTO cities (name) VALUES ('Parnu');
INSERT INTO cities (name) VALUES ('Pasadena');
INSERT INTO cities (name) VALUES ('PassoFundo');
INSERT INTO cities (name) VALUES ('Pasto');
INSERT INTO cities (name) VALUES ('Pasuruan');
INSERT INTO cities (name) VALUES ('Pate');
INSERT INTO cities (name) VALUES ('Paterson');
INSERT INTO cities (name) VALUES ('Pathein');
INSERT INTO cities (name) VALUES ('Patiala');
INSERT INTO cities (name) VALUES ('Patna');
INSERT INTO cities (name) VALUES ('PatosdeMinas');
INSERT INTO cities (name) VALUES ('Patrai');
INSERT INTO cities (name) VALUES ('Pau');
INSERT INTO cities (name) VALUES ('Paulista');
INSERT INTO cities (name) VALUES ('Pavlodar');
INSERT INTO cities (name) VALUES ('Pavlohrad');
INSERT INTO cities (name) VALUES ('Paysandu');
INSERT INTO cities (name) VALUES ('Pec');
INSERT INTO cities (name) VALUES ('Pecs');
INSERT INTO cities (name) VALUES ('PedraBadejo');
INSERT INTO cities (name) VALUES ('PedroJuanCaballero');
INSERT INTO cities (name) VALUES ('Peel');
INSERT INTO cities (name) VALUES ('Pekalongan');
INSERT INTO cities (name) VALUES ('Pekan Baru');
INSERT INTO cities (name) VALUES ('Pelotas');
INSERT INTO cities (name) VALUES ('Pemalang');
INSERT INTO cities (name) VALUES ('Pematang Siantar');
INSERT INTO cities (name) VALUES ('Pemba');
INSERT INTO cities (name) VALUES ('Pembroke Pines');
INSERT INTO cities (name) VALUES ('Pensacola');
INSERT INTO cities (name) VALUES ('Penza');
INSERT INTO cities (name) VALUES ('Peoria');
INSERT INTO cities (name) VALUES ('Pereira');
INSERT INTO cities (name) VALUES ('Perm');
INSERT INTO cities (name) VALUES ('Perpignan');
INSERT INTO cities (name) VALUES ('Perth');
INSERT INTO cities (name) VALUES ('Perugia');
INSERT INTO cities (name) VALUES ('Pervouralsk');
INSERT INTO cities (name) VALUES ('Pescara');
INSERT INTO cities (name) VALUES ('Peshawar');
INSERT INTO cities (name) VALUES ('Petare');
INSERT INTO cities (name) VALUES ('Peterborough');
INSERT INTO cities (name) VALUES ('Petit-Bourg');
INSERT INTO cities (name) VALUES ('Petrolina');
INSERT INTO cities (name) VALUES ('Petropavl');
INSERT INTO cities (name) VALUES ('Petropavlovsk-Kamchatskiy');
INSERT INTO cities (name) VALUES ('Petropolis');
INSERT INTO cities (name) VALUES ('Petrozavodsk');
INSERT INTO cities (name) VALUES ('Pforzheim');
INSERT INTO cities (name) VALUES ('PhanThiet');
INSERT INTO cities (name) VALUES ('Philadelphia');
INSERT INTO cities (name) VALUES ('Philipsburg');
INSERT INTO cities (name) VALUES ('Phnom Penh');
INSERT INTO cities (name) VALUES ('Phoenix');
INSERT INTO cities (name) VALUES ('Phuntsholing');
INSERT INTO cities (name) VALUES ('Phyongsong');
INSERT INTO cities (name) VALUES ('Piacenza');
INSERT INTO cities (name) VALUES ('Pico Rivera');
INSERT INTO cities (name) VALUES ('Piedras Negras');
INSERT INTO cities (name) VALUES ('Pierre');
INSERT INTO cities (name) VALUES ('Pietermaritzburg');
INSERT INTO cities (name) VALUES ('Pietersburg');
INSERT INTO cities (name) VALUES ('Pimpri');
INSERT INTO cities (name) VALUES ('PinardelRio');
INSERT INTO cities (name) VALUES ('Pindamonhangaba');
INSERT INTO cities (name) VALUES ('Pine Bluff');
INSERT INTO cities (name) VALUES ('Pingchen');
INSERT INTO cities (name) VALUES ('Pingdingshan');
INSERT INTO cities (name) VALUES ('Pingliang');
INSERT INTO cities (name) VALUES ('Pingtung');
INSERT INTO cities (name) VALUES ('Pingxiang');
INSERT INTO cities (name) VALUES ('Pinhais');
INSERT INTO cities (name) VALUES ('Pinsk');
INSERT INTO cities (name) VALUES ('Piracicaba');
INSERT INTO cities (name) VALUES ('Pisa');
INSERT INTO cities (name) VALUES ('Pitesti');
INSERT INTO cities (name) VALUES ('Pittsburgh');
INSERT INTO cities (name) VALUES ('Pittsfield');
INSERT INTO cities (name) VALUES ('Pituffik');
INSERT INTO cities (name) VALUES ('Piura');
INSERT INTO cities (name) VALUES ('Plano');
INSERT INTO cities (name) VALUES ('Plantation');
INSERT INTO cities (name) VALUES ('Pleasanton');
INSERT INTO cities (name) VALUES ('Pleven');
INSERT INTO cities (name) VALUES ('Plock');
INSERT INTO cities (name) VALUES ('Ploiesti');
INSERT INTO cities (name) VALUES ('Plovdiv');
INSERT INTO cities (name) VALUES ('Plymouth');
INSERT INTO cities (name) VALUES ('Plzen');
INSERT INTO cities (name) VALUES ('Pocatello');
INSERT INTO cities (name) VALUES ('Podgorica');
INSERT INTO cities (name) VALUES ('Podolsk');
INSERT INTO cities (name) VALUES ('Pohang');
INSERT INTO cities (name) VALUES ('Poindimie');
INSERT INTO cities (name) VALUES ('Point-a-Pitre');
INSERT INTO cities (name) VALUES ('PointeNoire');
INSERT INTO cities (name) VALUES ('Poitiers');
INSERT INTO cities (name) VALUES ('Pokhara');
INSERT INTO cities (name) VALUES ('Poltava');
INSERT INTO cities (name) VALUES ('Pomona');
INSERT INTO cities (name) VALUES ('Pompano Beach');
INSERT INTO cities (name) VALUES ('Ponce');
INSERT INTO cities (name) VALUES ('Pondicherry');
INSERT INTO cities (name) VALUES ('PontaGrossa');
INSERT INTO cities (name) VALUES ('Pontefract');
INSERT INTO cities (name) VALUES ('Pontiac');
INSERT INTO cities (name) VALUES ('Pontianak');
INSERT INTO cities (name) VALUES ('Pontypool');
INSERT INTO cities (name) VALUES ('Pontypridd');
INSERT INTO cities (name) VALUES ('Poole');
INSERT INTO cities (name) VALUES ('Popayan');
INSERT INTO cities (name) VALUES ('Pori');
INSERT INTO cities (name) VALUES ('Port Arthur');
INSERT INTO cities (name) VALUES ('Port Elizabeth');
INSERT INTO cities (name) VALUES ('Port Harcourt');
INSERT INTO cities (name) VALUES ('Port Said');
INSERT INTO cities (name) VALUES ('Port Saint Lucie');
INSERT INTO cities (name) VALUES ('Port Sudan');
INSERT INTO cities (name) VALUES ('Port Talbot');
INSERT INTO cities (name) VALUES ('Port-au-Prince');
INSERT INTO cities (name) VALUES ('Port-aux-Francais');
INSERT INTO cities (name) VALUES ('Port-de-Paix');
INSERT INTO cities (name) VALUES ('Port-Gentil');
INSERT INTO cities (name) VALUES ('Portage');
INSERT INTO cities (name) VALUES ('Porterville');
INSERT INTO cities (name) VALUES ('Portland');
INSERT INTO cities (name) VALUES ('Porto');
INSERT INTO cities (name) VALUES ('Portoviejo');
INSERT INTO cities (name) VALUES ('PortoAlegre');
INSERT INTO cities (name) VALUES ('PortoNovo');
INSERT INTO cities (name) VALUES ('PortoVelho');
INSERT INTO cities (name) VALUES ('Portsmouth');
INSERT INTO cities (name) VALUES ('PortErin');
INSERT INTO cities (name) VALUES ('PortHoward');
INSERT INTO cities (name) VALUES ('PortLouis');
INSERT INTO cities (name) VALUES ('PortMoresby');
INSERT INTO cities (name) VALUES ('PortofSpain');
INSERT INTO cities (name) VALUES ('PortOlry');
INSERT INTO cities (name) VALUES ('PortStanley');
INSERT INTO cities (name) VALUES ('Poryong');
INSERT INTO cities (name) VALUES ('Posadas');
INSERT INTO cities (name) VALUES ('Potchefstroom');
INSERT INTO cities (name) VALUES ('Potosi');
INSERT INTO cities (name) VALUES ('Potsdam');
INSERT INTO cities (name) VALUES ('Poughkeepsie');
INSERT INTO cities (name) VALUES ('PousoAlegre');
INSERT INTO cities (name) VALUES ('Poza Rica');
INSERT INTO cities (name) VALUES ('Poznan');
INSERT INTO cities (name) VALUES ('PoosdeCaldas');
INSERT INTO cities (name) VALUES ('Prague');
INSERT INTO cities (name) VALUES ('Praia');
INSERT INTO cities (name) VALUES ('PraiaGrande');
INSERT INTO cities (name) VALUES ('Preah Seihanu');
INSERT INTO cities (name) VALUES ('Preshov');
INSERT INTO cities (name) VALUES ('PresidenteEduardoFrei');
INSERT INTO cities (name) VALUES ('PresidentePrudente');
INSERT INTO cities (name) VALUES ('Preston');
INSERT INTO cities (name) VALUES ('Pretoria');
INSERT INTO cities (name) VALUES ('Prilep');
INSERT INTO cities (name) VALUES ('Prince Albert');
INSERT INTO cities (name) VALUES ('Prince George');
INSERT INTO cities (name) VALUES ('Prishtina');
INSERT INTO cities (name) VALUES ('Prizren');
INSERT INTO cities (name) VALUES ('Probolinggo');
INSERT INTO cities (name) VALUES ('Prokopyevsk');
INSERT INTO cities (name) VALUES ('Providence');
INSERT INTO cities (name) VALUES ('Provo');
INSERT INTO cities (name) VALUES ('Pskov');
INSERT INTO cities (name) VALUES ('Pucallpa');
INSERT INTO cities (name) VALUES ('Pucheon');
INSERT INTO cities (name) VALUES ('Puebla');
INSERT INTO cities (name) VALUES ('Pueblo');
INSERT INTO cities (name) VALUES ('Puente Alto');
INSERT INTO cities (name) VALUES ('Puerto Cabello');
INSERT INTO cities (name) VALUES ('Puerto la Cruz');
INSERT INTO cities (name) VALUES ('Puerto Montt');
INSERT INTO cities (name) VALUES ('Puerto Vallarta');
INSERT INTO cities (name) VALUES ('PuertoPlata');
INSERT INTO cities (name) VALUES ('Pula');
INSERT INTO cities (name) VALUES ('Pulandian');
INSERT INTO cities (name) VALUES ('Pune');
INSERT INTO cities (name) VALUES ('Punta Arenas');
INSERT INTO cities (name) VALUES ('Puntarenas');
INSERT INTO cities (name) VALUES ('Punto Fijo');
INSERT INTO cities (name) VALUES ('Puqi');
INSERT INTO cities (name) VALUES ('Purwakarta');
INSERT INTO cities (name) VALUES ('Purwokerto');
INSERT INTO cities (name) VALUES ('Pusan');
INSERT INTO cities (name) VALUES ('Puyang');
INSERT INTO cities (name) VALUES ('Pyatigorsk');
INSERT INTO cities (name) VALUES ('Pyay');
INSERT INTO cities (name) VALUES ('Pyeongtaek');
INSERT INTO cities (name) VALUES ('Pyongyang');
INSERT INTO cities (name) VALUES ('PyramidPoint');
INSERT INTO cities (name) VALUES ('Qaanaaq');
INSERT INTO cities (name) VALUES ('Qaarsut');
INSERT INTO cities (name) VALUES ('Qacemshahr');
INSERT INTO cities (name) VALUES ('Qadarif');
INSERT INTO cities (name) VALUES ('Qamishly');
INSERT INTO cities (name) VALUES ('Qaqortoq');
INSERT INTO cities (name) VALUES ('Qarsi');
INSERT INTO cities (name) VALUES ('Qasigiannguit');
INSERT INTO cities (name) VALUES ('Qazvin');
INSERT INTO cities (name) VALUES ('Qeqertaq');
INSERT INTO cities (name) VALUES ('Qeqertarsuaq');
INSERT INTO cities (name) VALUES ('Qeqertarsuatsiaat');
INSERT INTO cities (name) VALUES ('Qianguo');
INSERT INTO cities (name) VALUES ('Qianjiang');
INSERT INTO cities (name) VALUES ('Qina');
INSERT INTO cities (name) VALUES ('Qincheng');
INSERT INTO cities (name) VALUES ('Qingdao');
INSERT INTO cities (name) VALUES ('Qingyi');
INSERT INTO cities (name) VALUES ('Qingyuan');
INSERT INTO cities (name) VALUES ('Qinhuangdao');
INSERT INTO cities (name) VALUES ('Qiongguanou');
INSERT INTO cities (name) VALUES ('Qiqihar');
INSERT INTO cities (name) VALUES ('QiryatGat');
INSERT INTO cities (name) VALUES ('Qitaihe');
INSERT INTO cities (name) VALUES ('Qom');
INSERT INTO cities (name) VALUES ('Qoryoley');
INSERT INTO cities (name) VALUES ('Qostanay');
INSERT INTO cities (name) VALUES ('Quanwan');
INSERT INTO cities (name) VALUES ('Quanzhou');
INSERT INTO cities (name) VALUES ('Quebec');
INSERT INTO cities (name) VALUES ('Queimados');
INSERT INTO cities (name) VALUES ('Quelimane');
INSERT INTO cities (name) VALUES ('Queluz');
INSERT INTO cities (name) VALUES ('Queretaro');
INSERT INTO cities (name) VALUES ('Quesada');
INSERT INTO cities (name) VALUES ('Quetta');
INSERT INTO cities (name) VALUES ('Quevedo');
INSERT INTO cities (name) VALUES ('Quezaltenango');
INSERT INTO cities (name) VALUES ('Quillacollo');
INSERT INTO cities (name) VALUES ('Quimper');
INSERT INTO cities (name) VALUES ('Quincy');
INSERT INTO cities (name) VALUES ('Quito');
INSERT INTO cities (name) VALUES ('QuiNhon');
INSERT INTO cities (name) VALUES ('Qujing');
INSERT INTO cities (name) VALUES ('Qurgonteppa');
INSERT INTO cities (name) VALUES ('Rabat');
INSERT INTO cities (name) VALUES ('Rabnita');
INSERT INTO cities (name) VALUES ('RachGia');
INSERT INTO cities (name) VALUES ('Racine');
INSERT INTO cities (name) VALUES ('Radom');
INSERT INTO cities (name) VALUES ('Rafah');
INSERT INTO cities (name) VALUES ('Rafsanjan');
INSERT INTO cities (name) VALUES ('Rahim Yar Khan');
INSERT INTO cities (name) VALUES ('Raichur');
INSERT INTO cities (name) VALUES ('Raipur');
INSERT INTO cities (name) VALUES ('Rajamahendri');
INSERT INTO cities (name) VALUES ('Rajarhat Gopalpur');
INSERT INTO cities (name) VALUES ('Rajkot');
INSERT INTO cities (name) VALUES ('Rajpur');
INSERT INTO cities (name) VALUES ('Rajshahi');
INSERT INTO cities (name) VALUES ('Rakkah');
INSERT INTO cities (name) VALUES ('Raleigh');
INSERT INTO cities (name) VALUES ('Ramagundam');
INSERT INTO cities (name) VALUES ('Rampur');
INSERT INTO cities (name) VALUES ('Ramsey');
INSERT INTO cities (name) VALUES ('Ramsgate');
INSERT INTO cities (name) VALUES ('Rancagua');
INSERT INTO cities (name) VALUES ('Ranchi');
INSERT INTO cities (name) VALUES ('Rancho Cucamonga');
INSERT INTO cities (name) VALUES ('Ranghulu');
INSERT INTO cities (name) VALUES ('Rangpur');
INSERT INTO cities (name) VALUES ('Rapid City');
INSERT INTO cities (name) VALUES ('Ras al Khaymah');
INSERT INTO cities (name) VALUES ('Rasht');
INSERT INTO cities (name) VALUES ('Ratlam');
INSERT INTO cities (name) VALUES ('Raurkela');
INSERT INTO cities (name) VALUES ('Ravenna');
INSERT INTO cities (name) VALUES ('Rawalpindi');
INSERT INTO cities (name) VALUES ('Rawannawi');
INSERT INTO cities (name) VALUES ('Rayong');
INSERT INTO cities (name) VALUES ('Reading');
INSERT INTO cities (name) VALUES ('Recife');
INSERT INTO cities (name) VALUES ('Recklinghausen');
INSERT INTO cities (name) VALUES ('Red Deer');
INSERT INTO cities (name) VALUES ('Redding');
INSERT INTO cities (name) VALUES ('Redlands');
INSERT INTO cities (name) VALUES ('Redondo Beach');
INSERT INTO cities (name) VALUES ('Redwood City');
INSERT INTO cities (name) VALUES ('Regensburg');
INSERT INTO cities (name) VALUES ('Reggio di Calabria');
INSERT INTO cities (name) VALUES ('Reggio nell\'Emilia');
INSERT INTO cities (name) VALUES ('Regina');
INSERT INTO cities (name) VALUES ('Rehoboth');
INSERT INTO cities (name) VALUES ('Reims');
INSERT INTO cities (name) VALUES ('Remire-Montjoly');
INSERT INTO cities (name) VALUES ('Remscheid');
INSERT INTO cities (name) VALUES ('Rennes');
INSERT INTO cities (name) VALUES ('Reno');
INSERT INTO cities (name) VALUES ('Resende');
INSERT INTO cities (name) VALUES ('Resistencia');
INSERT INTO cities (name) VALUES ('Reutlingen');
INSERT INTO cities (name) VALUES ('Reykjavk');
INSERT INTO cities (name) VALUES ('Reynosa');
INSERT INTO cities (name) VALUES ('Rhondda');
INSERT INTO cities (name) VALUES ('Rhyl');
INSERT INTO cities (name) VALUES ('Rialto');
INSERT INTO cities (name) VALUES ('RibeiraodasNeves');
INSERT INTO cities (name) VALUES ('RibeiraoPires');
INSERT INTO cities (name) VALUES ('RibeiraoPreto');
INSERT INTO cities (name) VALUES ('Richardson');
INSERT INTO cities (name) VALUES ('Richmond');
INSERT INTO cities (name) VALUES ('Richmond Hill');
INSERT INTO cities (name) VALUES ('Ridah');
INSERT INTO cities (name) VALUES ('Rifa');
INSERT INTO cities (name) VALUES ('Riga');
INSERT INTO cities (name) VALUES ('Rijeka');
INSERT INTO cities (name) VALUES ('Rimini');
INSERT INTO cities (name) VALUES ('Riobamba');
INSERT INTO cities (name) VALUES ('RioBranco');
INSERT INTO cities (name) VALUES ('RioClaro');
INSERT INTO cities (name) VALUES ('RioCuarto');
INSERT INTO cities (name) VALUES ('RiodeJaneiro');
INSERT INTO cities (name) VALUES ('RioGrande');
INSERT INTO cities (name) VALUES ('RioVerde');
INSERT INTO cities (name) VALUES ('Rita');
INSERT INTO cities (name) VALUES ('Rivera');
INSERT INTO cities (name) VALUES ('Riverside');
INSERT INTO cities (name) VALUES ('Rivne');
INSERT INTO cities (name) VALUES ('Riyadh');
INSERT INTO cities (name) VALUES ('Rizhao');
INSERT INTO cities (name) VALUES ('Road Town');
INSERT INTO cities (name) VALUES ('Roanoke');
INSERT INTO cities (name) VALUES ('Rochester');
INSERT INTO cities (name) VALUES ('Rochester Hills');
INSERT INTO cities (name) VALUES ('Rockford');
INSERT INTO cities (name) VALUES ('Rocky Mount');
INSERT INTO cities (name) VALUES ('Rohtak');
INSERT INTO cities (name) VALUES ('Roma');
INSERT INTO cities (name) VALUES ('Rondonopolis');
INSERT INTO cities (name) VALUES ('Rongcheng');
INSERT INTO cities (name) VALUES ('Roosendaal');
INSERT INTO cities (name) VALUES ('Roreti');
INSERT INTO cities (name) VALUES ('Rosario');
INSERT INTO cities (name) VALUES ('Roseau');
INSERT INTO cities (name) VALUES ('Rosemead');
INSERT INTO cities (name) VALUES ('Rostock');
INSERT INTO cities (name) VALUES ('Rostov');
INSERT INTO cities (name) VALUES ('Roswell');
INSERT INTO cities (name) VALUES ('Rotherham');
INSERT INTO cities (name) VALUES ('Rotorua');
INSERT INTO cities (name) VALUES ('Rotterdam');
INSERT INTO cities (name) VALUES ('Rouen');
INSERT INTO cities (name) VALUES ('Round Rock');
INSERT INTO cities (name) VALUES ('Rowland Heights');
INSERT INTO cities (name) VALUES ('Roxas');
INSERT INTO cities (name) VALUES ('Rubtsovsk');
INSERT INTO cities (name) VALUES ('Rudni');
INSERT INTO cities (name) VALUES ('Ruhengeri');
INSERT INTO cities (name) VALUES ('Ruian');
INSERT INTO cities (name) VALUES ('Rumadiya');
INSERT INTO cities (name) VALUES ('Runavik');
INSERT INTO cities (name) VALUES ('Rundu');
INSERT INTO cities (name) VALUES ('Rungata');
INSERT INTO cities (name) VALUES ('Ruse');
INSERT INTO cities (name) VALUES ('Rushden');
INSERT INTO cities (name) VALUES ('Rustaq');
INSERT INTO cities (name) VALUES ('Rustenburg');
INSERT INTO cities (name) VALUES ('Rusu');
INSERT INTO cities (name) VALUES ('Ruyigi');
INSERT INTO cities (name) VALUES ('Ryazan');
INSERT INTO cities (name) VALUES ('Rybinsk');
INSERT INTO cities (name) VALUES ('Rybnik');
INSERT INTO cities (name) VALUES ('Rzeszow');
INSERT INTO cities (name) VALUES ('Saanich');
INSERT INTO cities (name) VALUES ('Saarbrucken');
INSERT INTO cities (name) VALUES ('Saattut');
INSERT INTO cities (name) VALUES ('Sabara');
INSERT INTO cities (name) VALUES ('Sabhah');
INSERT INTO cities (name) VALUES ('Sabratah');
INSERT INTO cities (name) VALUES ('Sabzevar');
INSERT INTO cities (name) VALUES ('Sacramento');
INSERT INTO cities (name) VALUES ('Sada');
INSERT INTO cities (name) VALUES ('Sadiqabad');
INSERT INTO cities (name) VALUES ('Saertu');
INSERT INTO cities (name) VALUES ('Saga');
INSERT INTO cities (name) VALUES ('Sagamihara');
INSERT INTO cities (name) VALUES ('Sagar');
INSERT INTO cities (name) VALUES ('Saginaw');
INSERT INTO cities (name) VALUES ('Sahab');
INSERT INTO cities (name) VALUES ('Saham');
INSERT INTO cities (name) VALUES ('Saharanpur');
INSERT INTO cities (name) VALUES ('Sahiwal');
INSERT INTO cities (name) VALUES ('Sahrisabz');
INSERT INTO cities (name) VALUES ('Saint Catharines');
INSERT INTO cities (name) VALUES ('Saint Cloud');
INSERT INTO cities (name) VALUES ('Saint George');
INSERT INTO cities (name) VALUES ('Saint Helens');
INSERT INTO cities (name) VALUES ('Saint John\'s');
INSERT INTO cities (name) VALUES ('Saint Joseph');
INSERT INTO cities (name) VALUES ('Saint Louis');
INSERT INTO cities (name) VALUES ('Saint Paul');
INSERT INTO cities (name) VALUES ('Saint Petersburg');
INSERT INTO cities (name) VALUES ('Saint-Denis');
INSERT INTO cities (name) VALUES ('Saint-Etienne');
INSERT INTO cities (name) VALUES ('Saint-Hubert');
INSERT INTO cities (name) VALUES ('Saint-Laurent-du-Maroni');
INSERT INTO cities (name) VALUES ('Saint-Louis');
INSERT INTO cities (name) VALUES ('Saint-Malo');
INSERT INTO cities (name) VALUES ('Saint-Marc');
INSERT INTO cities (name) VALUES ('Saint-Nazaire');
INSERT INTO cities (name) VALUES ('Saint-Paul');
INSERT INTO cities (name) VALUES ('Saint-Pierre');
INSERT INTO cities (name) VALUES ('Saint-Quentin');
INSERT INTO cities (name) VALUES ('Sainte-Anne');
INSERT INTO cities (name) VALUES ('Sainte-Julie');
INSERT INTO cities (name) VALUES ('Sainte-Marie');
INSERT INTO cities (name) VALUES ('Sainte-Rose');
INSERT INTO cities (name) VALUES ('SaintGeorge');
INSERT INTO cities (name) VALUES ('SaintGeorge\'s');
INSERT INTO cities (name) VALUES ('SaintHelier');
INSERT INTO cities (name) VALUES ('SaintJohn\'s');
INSERT INTO cities (name) VALUES ('SaintJoseph');
INSERT INTO cities (name) VALUES ('SaintPauls');
INSERT INTO cities (name) VALUES ('SaintPeterPort');
INSERT INTO cities (name) VALUES ('Sakai');
INSERT INTO cities (name) VALUES ('Sakakah');
INSERT INTO cities (name) VALUES ('Sakikdah');
INSERT INTO cities (name) VALUES ('Salalah');
INSERT INTO cities (name) VALUES ('Salamanca');
INSERT INTO cities (name) VALUES ('Salatiga');
INSERT INTO cities (name) VALUES ('Salavat');
INSERT INTO cities (name) VALUES ('Salem');
INSERT INTO cities (name) VALUES ('Salerno');
INSERT INTO cities (name) VALUES ('Salina');
INSERT INTO cities (name) VALUES ('Salinas');
INSERT INTO cities (name) VALUES ('Salisbury');
INSERT INTO cities (name) VALUES ('Salt');
INSERT INTO cities (name) VALUES ('Salt Lake City');
INSERT INTO cities (name) VALUES ('Salta');
INSERT INTO cities (name) VALUES ('Saltillo');
INSERT INTO cities (name) VALUES ('Salto');
INSERT INTO cities (name) VALUES ('Salvador');
INSERT INTO cities (name) VALUES ('Salzburg');
INSERT INTO cities (name) VALUES ('Salzgitter');
INSERT INTO cities (name) VALUES ('Samara');
INSERT INTO cities (name) VALUES ('Samarah');
INSERT INTO cities (name) VALUES ('Samarinda');
INSERT INTO cities (name) VALUES ('Samarkand');
INSERT INTO cities (name) VALUES ('Samarra');
INSERT INTO cities (name) VALUES ('Samsun');
INSERT INTO cities (name) VALUES ('SamutPrakan');
INSERT INTO cities (name) VALUES ('San Angelo');
INSERT INTO cities (name) VALUES ('San Antonio');
INSERT INTO cities (name) VALUES ('San Bernardino');
INSERT INTO cities (name) VALUES ('San Bernardo');
INSERT INTO cities (name) VALUES ('San Buenaventura');
INSERT INTO cities (name) VALUES ('San Clemente');
INSERT INTO cities (name) VALUES ('San Cristobal');
INSERT INTO cities (name) VALUES ('San Cristobal de la Laguna');
INSERT INTO cities (name) VALUES ('San Cristobal de las Casas');
INSERT INTO cities (name) VALUES ('San Diego');
INSERT INTO cities (name) VALUES ('San Fernando');
INSERT INTO cities (name) VALUES ('San Francisco');
INSERT INTO cities (name) VALUES ('San Jose');
INSERT INTO cities (name) VALUES ('San Leandro');
INSERT INTO cities (name) VALUES ('San Luis Obispo');
INSERT INTO cities (name) VALUES ('San Luis Potos');
INSERT INTO cities (name) VALUES ('San Luis Ro Colorado');
INSERT INTO cities (name) VALUES ('San Mateo');
INSERT INTO cities (name) VALUES ('San Pablo');
INSERT INTO cities (name) VALUES ('San Pedro');
INSERT INTO cities (name) VALUES ('Sana\'a');
INSERT INTO cities (name) VALUES ('SanaeIV');
INSERT INTO cities (name) VALUES ('Sanandaj');
INSERT INTO cities (name) VALUES ('Sanbu');
INSERT INTO cities (name) VALUES ('Sanchung');
INSERT INTO cities (name) VALUES ('SanctiSpiritus');
INSERT INTO cities (name) VALUES ('Sandakan');
INSERT INTO cities (name) VALUES ('Sandy');
INSERT INTO cities (name) VALUES ('SandyGround');
INSERT INTO cities (name) VALUES ('Sangli-Miraj');
INSERT INTO cities (name) VALUES ('Sankt Peterburg');
INSERT INTO cities (name) VALUES ('SanktGallen');
INSERT INTO cities (name) VALUES ('SanktPlten');
INSERT INTO cities (name) VALUES ('Sanmenxia');
INSERT INTO cities (name) VALUES ('Sanming');
INSERT INTO cities (name) VALUES ('Santa Ana');
INSERT INTO cities (name) VALUES ('Santa Barbara');
INSERT INTO cities (name) VALUES ('Santa Clara');
INSERT INTO cities (name) VALUES ('Santa Clarita');
INSERT INTO cities (name) VALUES ('Santa Cruz');
INSERT INTO cities (name) VALUES ('Santa Cruz de Tenerife');
INSERT INTO cities (name) VALUES ('Santa Fe');
INSERT INTO cities (name) VALUES ('Santa Maria');
INSERT INTO cities (name) VALUES ('Santa Monica');
INSERT INTO cities (name) VALUES ('Santa Rosa');
INSERT INTO cities (name) VALUES ('Santa Teresa');
INSERT INTO cities (name) VALUES ('Santander');
INSERT INTO cities (name) VALUES ('Santarem');
INSERT INTO cities (name) VALUES ('SantaAna');
INSERT INTO cities (name) VALUES ('SantaBarbarad\'Oeste');
INSERT INTO cities (name) VALUES ('SantaClara');
INSERT INTO cities (name) VALUES ('SantaCruz');
INSERT INTO cities (name) VALUES ('SantaFe');
INSERT INTO cities (name) VALUES ('SantaLuzia');
INSERT INTO cities (name) VALUES ('SantaMaria');
INSERT INTO cities (name) VALUES ('SantaMarta');
INSERT INTO cities (name) VALUES ('SantaRita');
INSERT INTO cities (name) VALUES ('Santiago');
INSERT INTO cities (name) VALUES ('Santiago de Compostela');
INSERT INTO cities (name) VALUES ('SantiagodelEstero');
INSERT INTO cities (name) VALUES ('SantiagodeCuba');
INSERT INTO cities (name) VALUES ('Santos');
INSERT INTO cities (name) VALUES ('SantoAndre');
INSERT INTO cities (name) VALUES ('SantoDomingo');
INSERT INTO cities (name) VALUES ('SanAntonio');
INSERT INTO cities (name) VALUES ('SanCarlosdeBariloche');
INSERT INTO cities (name) VALUES ('SanCristobal');
INSERT INTO cities (name) VALUES ('SanFernando');
INSERT INTO cities (name) VALUES ('SanFranciscodeMacoris');
INSERT INTO cities (name) VALUES ('SanIgnacio');
INSERT INTO cities (name) VALUES ('SanIsidro');
INSERT INTO cities (name) VALUES ('SanJose');
INSERT INTO cities (name) VALUES ('SanJuan');
INSERT INTO cities (name) VALUES ('SanLuis');
INSERT INTO cities (name) VALUES ('SanMarino');
INSERT INTO cities (name) VALUES ('SanMiguel');
INSERT INTO cities (name) VALUES ('SanNicolas');
INSERT INTO cities (name) VALUES ('SanPedrodeMacoris');
INSERT INTO cities (name) VALUES ('SanPedroSula');
INSERT INTO cities (name) VALUES ('SanRafael');
INSERT INTO cities (name) VALUES ('SanSalvador');
INSERT INTO cities (name) VALUES ('Sao Paulo');
INSERT INTO cities (name) VALUES ('Sao Tome');
INSERT INTO cities (name) VALUES ('SaoBernardodoCampo');
INSERT INTO cities (name) VALUES ('SaoCaetanodoSul');
INSERT INTO cities (name) VALUES ('SaoCarlos');
INSERT INTO cities (name) VALUES ('SaoFilipe');
INSERT INTO cities (name) VALUES ('SaoGoncalo');
INSERT INTO cities (name) VALUES ('SaoJoaodeMeriti');
INSERT INTO cities (name) VALUES ('SaoJose');
INSERT INTO cities (name) VALUES ('SaoJosedosCampos');
INSERT INTO cities (name) VALUES ('SaoJosedosPinhais');
INSERT INTO cities (name) VALUES ('SaoJosedoRioPreto');
INSERT INTO cities (name) VALUES ('SaoLeopoldo');
INSERT INTO cities (name) VALUES ('SaoLuis');
INSERT INTO cities (name) VALUES ('SaoVicente');
INSERT INTO cities (name) VALUES ('Sapele');
INSERT INTO cities (name) VALUES ('Sapporo');
INSERT INTO cities (name) VALUES ('Sapucaia');
INSERT INTO cities (name) VALUES ('Saqqaq');
INSERT INTO cities (name) VALUES ('Saqqez');
INSERT INTO cities (name) VALUES ('Sarajevo');
INSERT INTO cities (name) VALUES ('Saransk');
INSERT INTO cities (name) VALUES ('Sarapul');
INSERT INTO cities (name) VALUES ('Sarasota');
INSERT INTO cities (name) VALUES ('Saratov');
INSERT INTO cities (name) VALUES ('Saravan');
INSERT INTO cities (name) VALUES ('Sarfannguit');
INSERT INTO cities (name) VALUES ('Sargodha');
INSERT INTO cities (name) VALUES ('Sarh');
INSERT INTO cities (name) VALUES ('Sari');
INSERT INTO cities (name) VALUES ('Sariwon');
INSERT INTO cities (name) VALUES ('Sarnia');
INSERT INTO cities (name) VALUES ('Sasebo');
INSERT INTO cities (name) VALUES ('Saskatoon');
INSERT INTO cities (name) VALUES ('Sassari');
INSERT INTO cities (name) VALUES ('Satna');
INSERT INTO cities (name) VALUES ('Sattat');
INSERT INTO cities (name) VALUES ('SatuMare');
INSERT INTO cities (name) VALUES ('Savannah');
INSERT INTO cities (name) VALUES ('Savannakhet');
INSERT INTO cities (name) VALUES ('Savave');
INSERT INTO cities (name) VALUES ('Saveh');
INSERT INTO cities (name) VALUES ('Sayda');
INSERT INTO cities (name) VALUES ('Sayyan');
INSERT INTO cities (name) VALUES ('Scarborough');
INSERT INTO cities (name) VALUES ('Schaan');
INSERT INTO cities (name) VALUES ('Schenectady');
INSERT INTO cities (name) VALUES ('Schwerin');
INSERT INTO cities (name) VALUES ('Scott');
INSERT INTO cities (name) VALUES ('Scottsdale');
INSERT INTO cities (name) VALUES ('Scranton');
INSERT INTO cities (name) VALUES ('Scunthorpe');
INSERT INTO cities (name) VALUES ('Seattle');
INSERT INTO cities (name) VALUES ('Segou');
INSERT INTO cities (name) VALUES ('Sekondi');
INSERT INTO cities (name) VALUES ('Selam');
INSERT INTO cities (name) VALUES ('SelebiPhikwe');
INSERT INTO cities (name) VALUES ('Semarang');
INSERT INTO cities (name) VALUES ('Semey (Semipalatinsk)');
INSERT INTO cities (name) VALUES ('Semnan');
INSERT INTO cities (name) VALUES ('Sendai');
INSERT INTO cities (name) VALUES ('Seongnam');
INSERT INTO cities (name) VALUES ('Seoul');
INSERT INTO cities (name) VALUES ('Sept-Iles');
INSERT INTO cities (name) VALUES ('Serang');
INSERT INTO cities (name) VALUES ('Serekunda');
INSERT INTO cities (name) VALUES ('Seremban');
INSERT INTO cities (name) VALUES ('Sergiyev Posad');
INSERT INTO cities (name) VALUES ('Seria');
INSERT INTO cities (name) VALUES ('Sermiligaaq');
INSERT INTO cities (name) VALUES ('Serov');
INSERT INTO cities (name) VALUES ('Serpuhov');
INSERT INTO cities (name) VALUES ('Serra');
INSERT INTO cities (name) VALUES ('Serravalle');
INSERT INTO cities (name) VALUES ('SeteLagoas');
INSERT INTO cities (name) VALUES ('Setif');
INSERT INTO cities (name) VALUES ('Setubal');
INSERT INTO cities (name) VALUES ('Sevastopol\'');
INSERT INTO cities (name) VALUES ('Severodvinsk');
INSERT INTO cities (name) VALUES ('Sevilla');
INSERT INTO cities (name) VALUES ('Sfax');
INSERT INTO cities (name) VALUES ('sGravenhage');
INSERT INTO cities (name) VALUES ('Shagamu');
INSERT INTO cities (name) VALUES ('Shahe');
INSERT INTO cities (name) VALUES ('Shahr-eKord');
INSERT INTO cities (name) VALUES ('Shahrud');
INSERT INTO cities (name) VALUES ('Shahty');
INSERT INTO cities (name) VALUES ('Shaki');
INSERT INTO cities (name) VALUES ('Shanghai');
INSERT INTO cities (name) VALUES ('Shangqiu');
INSERT INTO cities (name) VALUES ('Shangrao');
INSERT INTO cities (name) VALUES ('Shantou');
INSERT INTO cities (name) VALUES ('Shanwei');
INSERT INTO cities (name) VALUES ('Shaoguan');
INSERT INTO cities (name) VALUES ('Shaowu');
INSERT INTO cities (name) VALUES ('Shaoxing');
INSERT INTO cities (name) VALUES ('Shaoyang');
INSERT INTO cities (name) VALUES ('Sharjah');
INSERT INTO cities (name) VALUES ('Shashemenne');
INSERT INTO cities (name) VALUES ('Shashi');
INSERT INTO cities (name) VALUES ('Shatin');
INSERT INTO cities (name) VALUES ('Sheboygan');
INSERT INTO cities (name) VALUES ('Sheffield');
INSERT INTO cities (name) VALUES ('Shekhupura');
INSERT INTO cities (name) VALUES ('Shenjiamen');
INSERT INTO cities (name) VALUES ('Shenyang');
INSERT INTO cities (name) VALUES ('Shenzhen');
INSERT INTO cities (name) VALUES ('sHertogenbosch');
INSERT INTO cities (name) VALUES ('Shibin el Kom');
INSERT INTO cities (name) VALUES ('Shihezi');
INSERT INTO cities (name) VALUES ('Shijiazhuang');
INSERT INTO cities (name) VALUES ('Shikarpur');
INSERT INTO cities (name) VALUES ('Shimoga');
INSERT INTO cities (name) VALUES ('Shinyanga');
INSERT INTO cities (name) VALUES ('Shiongshui');
INSERT INTO cities (name) VALUES ('Shiqiao');
INSERT INTO cities (name) VALUES ('Shiraz');
INSERT INTO cities (name) VALUES ('Shishou');
INSERT INTO cities (name) VALUES ('Shiyan');
INSERT INTO cities (name) VALUES ('Shizuishan');
INSERT INTO cities (name) VALUES ('Shizuoka');
INSERT INTO cities (name) VALUES ('Sholapur');
INSERT INTO cities (name) VALUES ('Shreveport');
INSERT INTO cities (name) VALUES ('Shrewsbury');
INSERT INTO cities (name) VALUES ('Shuangcheng');
INSERT INTO cities (name) VALUES ('Shuangyashan');
INSERT INTO cities (name) VALUES ('Shuimogou');
INSERT INTO cities (name) VALUES ('Shunyi');
INSERT INTO cities (name) VALUES ('Sialkot');
INSERT INTO cities (name) VALUES ('Siauliai');
INSERT INTO cities (name) VALUES ('Sibiu');
INSERT INTO cities (name) VALUES ('Sibu');
INSERT INTO cities (name) VALUES ('Sidi bel Abbes');
INSERT INTO cities (name) VALUES ('Siegen');
INSERT INTO cities (name) VALUES ('Siem Riep');
INSERT INTO cities (name) VALUES ('Siguiri');
INSERT INTO cities (name) VALUES ('Siirt');
INSERT INTO cities (name) VALUES ('Sikandarabad');
INSERT INTO cities (name) VALUES ('Sikasso');
INSERT INTO cities (name) VALUES ('Silhat');
INSERT INTO cities (name) VALUES ('Simferopol\'');
INSERT INTO cities (name) VALUES ('Simi Valley');
INSERT INTO cities (name) VALUES ('Sincelejo');
INSERT INTO cities (name) VALUES ('Singapore');
INSERT INTO cities (name) VALUES ('Sinnar');
INSERT INTO cities (name) VALUES ('SintNicolas');
INSERT INTO cities (name) VALUES ('Sinuiju');
INSERT INTO cities (name) VALUES ('Sioux City');
INSERT INTO cities (name) VALUES ('Sioux Falls');
INSERT INTO cities (name) VALUES ('Siping');
INSERT INTO cities (name) VALUES ('Siquirres');
INSERT INTO cities (name) VALUES ('Siracusa');
INSERT INTO cities (name) VALUES ('Sirjan');
INSERT INTO cities (name) VALUES ('Sisimiut');
INSERT INTO cities (name) VALUES ('Sitrah');
INSERT INTO cities (name) VALUES ('Sittingbourne');
INSERT INTO cities (name) VALUES ('Sivas');
INSERT INTO cities (name) VALUES ('SiRacha');
INSERT INTO cities (name) VALUES ('Skopje');
INSERT INTO cities (name) VALUES ('SlavonskiBrod');
INSERT INTO cities (name) VALUES ('Sliema');
INSERT INTO cities (name) VALUES ('Sliven');
INSERT INTO cities (name) VALUES ('Slough');
INSERT INTO cities (name) VALUES ('Slupsk');
INSERT INTO cities (name) VALUES ('Smolensk');
INSERT INTO cities (name) VALUES ('Soacha');
INSERT INTO cities (name) VALUES ('Sobral');
INSERT INTO cities (name) VALUES ('Sochi');
INSERT INTO cities (name) VALUES ('Sofia');
INSERT INTO cities (name) VALUES ('Sogamoso');
INSERT INTO cities (name) VALUES ('Sokode');
INSERT INTO cities (name) VALUES ('Sokoto');
INSERT INTO cities (name) VALUES ('Sola');
INSERT INTO cities (name) VALUES ('Soledad');
INSERT INTO cities (name) VALUES ('Solegorsk');
INSERT INTO cities (name) VALUES ('Solikamsk');
INSERT INTO cities (name) VALUES ('Solingen');
INSERT INTO cities (name) VALUES ('Sompting-Lancing');
INSERT INTO cities (name) VALUES ('Songea');
INSERT INTO cities (name) VALUES ('Songjiang');
INSERT INTO cities (name) VALUES ('Songkhla');
INSERT INTO cities (name) VALUES ('Songnim');
INSERT INTO cities (name) VALUES ('Songsong');
INSERT INTO cities (name) VALUES ('Sonsonate');
INSERT INTO cities (name) VALUES ('Sorocaba');
INSERT INTO cities (name) VALUES ('Sorong');
INSERT INTO cities (name) VALUES ('Sosnowiec');
INSERT INTO cities (name) VALUES ('Soufriere');
INSERT INTO cities (name) VALUES ('South Bend');
INSERT INTO cities (name) VALUES ('South Dum Dum');
INSERT INTO cities (name) VALUES ('South Gate');
INSERT INTO cities (name) VALUES ('South Whittier');
INSERT INTO cities (name) VALUES ('Southampton');
INSERT INTO cities (name) VALUES ('Southend-on-Sea');
INSERT INTO cities (name) VALUES ('Southfield');
INSERT INTO cities (name) VALUES ('Soweto');
INSERT INTO cities (name) VALUES ('Spanish Fork');
INSERT INTO cities (name) VALUES ('Sparks');
INSERT INTO cities (name) VALUES ('Speightstown');
INSERT INTO cities (name) VALUES ('Split');
INSERT INTO cities (name) VALUES ('Spokane');
INSERT INTO cities (name) VALUES ('Spring Valley');
INSERT INTO cities (name) VALUES ('Springdale');
INSERT INTO cities (name) VALUES ('Springfield');
INSERT INTO cities (name) VALUES ('Srinagar');
INSERT INTO cities (name) VALUES ('Stafford');
INSERT INTO cities (name) VALUES ('Stakhanov');
INSERT INTO cities (name) VALUES ('Stamford');
INSERT INTO cities (name) VALUES ('StaraZagora');
INSERT INTO cities (name) VALUES ('Staryy Oskol');
INSERT INTO cities (name) VALUES ('Stavanger');
INSERT INTO cities (name) VALUES ('Stavropol');
INSERT INTO cities (name) VALUES ('Sterling Heights');
INSERT INTO cities (name) VALUES ('Sterlitamak');
INSERT INTO cities (name) VALUES ('Stirling');
INSERT INTO cities (name) VALUES ('Stockholm');
INSERT INTO cities (name) VALUES ('Stockport');
INSERT INTO cities (name) VALUES ('Stockton');
INSERT INTO cities (name) VALUES ('Stoke-on-Trent');
INSERT INTO cities (name) VALUES ('Strasbourg');
INSERT INTO cities (name) VALUES ('Strathcona');
INSERT INTO cities (name) VALUES ('Stroud');
INSERT INTO cities (name) VALUES ('Strovolos');
INSERT INTO cities (name) VALUES ('Stuttgart');
INSERT INTO cities (name) VALUES ('Subang');
INSERT INTO cities (name) VALUES ('Subotica');
INSERT INTO cities (name) VALUES ('Subra al-Haymah');
INSERT INTO cities (name) VALUES ('Sucheng');
INSERT INTO cities (name) VALUES ('Sucre');
INSERT INTO cities (name) VALUES ('Sudbury');
INSERT INTO cities (name) VALUES ('Suez');
INSERT INTO cities (name) VALUES ('Suffolk');
INSERT INTO cities (name) VALUES ('Suhaj');
INSERT INTO cities (name) VALUES ('Suhar');
INSERT INTO cities (name) VALUES ('Suihua');
INSERT INTO cities (name) VALUES ('Suining');
INSERT INTO cities (name) VALUES ('Suizhou');
INSERT INTO cities (name) VALUES ('Sujiatun');
INSERT INTO cities (name) VALUES ('Sukabumi');
INSERT INTO cities (name) VALUES ('Sukkur');
INSERT INTO cities (name) VALUES ('Suleymaniye');
INSERT INTO cities (name) VALUES ('Sullana');
INSERT INTO cities (name) VALUES ('Sumare');
INSERT INTO cities (name) VALUES ('Sumbawanga');
INSERT INTO cities (name) VALUES ('Sumgait');
INSERT INTO cities (name) VALUES ('Sumter');
INSERT INTO cities (name) VALUES ('Sumy');
INSERT INTO cities (name) VALUES ('Suncheon');
INSERT INTO cities (name) VALUES ('Sunderland');
INSERT INTO cities (name) VALUES ('SungaiPetani');
INSERT INTO cities (name) VALUES ('Sunnyvale');
INSERT INTO cities (name) VALUES ('Sunrise');
INSERT INTO cities (name) VALUES ('Sunrise Manor');
INSERT INTO cities (name) VALUES ('SunshineCoast');
INSERT INTO cities (name) VALUES ('Sur');
INSERT INTO cities (name) VALUES ('Surabaya');
INSERT INTO cities (name) VALUES ('Surakarta');
INSERT INTO cities (name) VALUES ('Surat');
INSERT INTO cities (name) VALUES ('SuratThani');
INSERT INTO cities (name) VALUES ('Surgut');
INSERT INTO cities (name) VALUES ('Surrey');
INSERT INTO cities (name) VALUES ('Surt');
INSERT INTO cities (name) VALUES ('Susah');
INSERT INTO cities (name) VALUES ('Susupe');
INSERT INTO cities (name) VALUES ('Sutton in Ashfield');
INSERT INTO cities (name) VALUES ('Suva');
INSERT INTO cities (name) VALUES ('Suwayq');
INSERT INTO cities (name) VALUES ('Suweon');
INSERT INTO cities (name) VALUES ('Suzano');
INSERT INTO cities (name) VALUES ('Suzhou');
INSERT INTO cities (name) VALUES ('Suzuka');
INSERT INTO cities (name) VALUES ('Swadlincote');
INSERT INTO cities (name) VALUES ('Swakopmund');
INSERT INTO cities (name) VALUES ('Swansea');
INSERT INTO cities (name) VALUES ('Swindon');
INSERT INTO cities (name) VALUES ('Swords');
INSERT INTO cities (name) VALUES ('Sydney');
INSERT INTO cities (name) VALUES ('Syeverodonetsk');
INSERT INTO cities (name) VALUES ('Syktyvkar');
INSERT INTO cities (name) VALUES ('Syowa');
INSERT INTO cities (name) VALUES ('Syracuse');
INSERT INTO cities (name) VALUES ('Syzran');
INSERT INTO cities (name) VALUES ('Szczecin');
INSERT INTO cities (name) VALUES ('Szeged');
INSERT INTO cities (name) VALUES ('Szekesfehervar');
INSERT INTO cities (name) VALUES ('Szombathely');
INSERT INTO cities (name) VALUES ('Tabiauea');
INSERT INTO cities (name) VALUES ('TaboaodaSerra');
INSERT INTO cities (name) VALUES ('Tabontebike');
INSERT INTO cities (name) VALUES ('Tabora');
INSERT INTO cities (name) VALUES ('Tabriz');
INSERT INTO cities (name) VALUES ('Tabuk');
INSERT INTO cities (name) VALUES ('Tabukiniberu');
INSERT INTO cities (name) VALUES ('Taburao');
INSERT INTO cities (name) VALUES ('Tacloban');
INSERT INTO cities (name) VALUES ('Tacna');
INSERT INTO cities (name) VALUES ('Tacoma');
INSERT INTO cities (name) VALUES ('Tacuarembo');
INSERT INTO cities (name) VALUES ('Tadine');
INSERT INTO cities (name) VALUES ('Taegu');
INSERT INTO cities (name) VALUES ('Taejeon');
INSERT INTO cities (name) VALUES ('Tafuna');
INSERT INTO cities (name) VALUES ('Taganrog');
INSERT INTO cities (name) VALUES ('Tagum');
INSERT INTO cities (name) VALUES ('Tahoua');
INSERT INTO cities (name) VALUES ('Taian');
INSERT INTO cities (name) VALUES ('Taicheng');
INSERT INTO cities (name) VALUES ('Taichung');
INSERT INTO cities (name) VALUES ('Tainan');
INSERT INTO cities (name) VALUES ('Taipei');
INSERT INTO cities (name) VALUES ('Taiping');
INSERT INTO cities (name) VALUES ('Taipo');
INSERT INTO cities (name) VALUES ('Taitung');
INSERT INTO cities (name) VALUES ('Taiyuan');
INSERT INTO cities (name) VALUES ('Taizz');
INSERT INTO cities (name) VALUES ('Takaeang');
INSERT INTO cities (name) VALUES ('Takamatsu');
INSERT INTO cities (name) VALUES ('Takaoka');
INSERT INTO cities (name) VALUES ('Takasaki');
INSERT INTO cities (name) VALUES ('Takoradi');
INSERT INTO cities (name) VALUES ('Talara');
INSERT INTO cities (name) VALUES ('Talas');
INSERT INTO cities (name) VALUES ('Talca');
INSERT INTO cities (name) VALUES ('Talcahuano');
INSERT INTO cities (name) VALUES ('Taldykugan');
INSERT INTO cities (name) VALUES ('Tallahassee');
INSERT INTO cities (name) VALUES ('Tallinn');
INSERT INTO cities (name) VALUES ('Tamale');
INSERT INTO cities (name) VALUES ('Tambacounda');
INSERT INTO cities (name) VALUES ('Tambov');
INSERT INTO cities (name) VALUES ('Tampa');
INSERT INTO cities (name) VALUES ('Tampere');
INSERT INTO cities (name) VALUES ('Tampico');
INSERT INTO cities (name) VALUES ('Tamuning');
INSERT INTO cities (name) VALUES ('Tandil');
INSERT INTO cities (name) VALUES ('Tando adam');
INSERT INTO cities (name) VALUES ('Tanga');
INSERT INTO cities (name) VALUES ('Tangayal');
INSERT INTO cities (name) VALUES ('Tanggu');
INSERT INTO cities (name) VALUES ('Tangiers');
INSERT INTO cities (name) VALUES ('Tangshan');
INSERT INTO cities (name) VALUES ('Tanjung Pinang');
INSERT INTO cities (name) VALUES ('Tanrake');
INSERT INTO cities (name) VALUES ('Tanta');
INSERT INTO cities (name) VALUES ('Taonan');
INSERT INTO cities (name) VALUES ('Taoyuan');
INSERT INTO cities (name) VALUES ('Tapachula');
INSERT INTO cities (name) VALUES ('Tarakan');
INSERT INTO cities (name) VALUES ('Taranto');
INSERT INTO cities (name) VALUES ('Tarapoto');
INSERT INTO cities (name) VALUES ('Taraz (Dzhambul)');
INSERT INTO cities (name) VALUES ('Targu Mures');
INSERT INTO cities (name) VALUES ('Tarhuna');
INSERT INTO cities (name) VALUES ('Tarija');
INSERT INTO cities (name) VALUES ('Tarnow');
INSERT INTO cities (name) VALUES ('Tarrafal');
INSERT INTO cities (name) VALUES ('Tarragona');
INSERT INTO cities (name) VALUES ('Tarsus');
INSERT INTO cities (name) VALUES ('Tartu');
INSERT INTO cities (name) VALUES ('Tartus');
INSERT INTO cities (name) VALUES ('Tashkent');
INSERT INTO cities (name) VALUES ('Tasiilaq');
INSERT INTO cities (name) VALUES ('Tasikmalaya');
INSERT INTO cities (name) VALUES ('Tasiusaq');
INSERT INTO cities (name) VALUES ('Taubate');
INSERT INTO cities (name) VALUES ('Taunggyi');
INSERT INTO cities (name) VALUES ('Taunton');
INSERT INTO cities (name) VALUES ('Tauranga');
INSERT INTO cities (name) VALUES ('Tawau');
INSERT INTO cities (name) VALUES ('Tazah');
INSERT INTO cities (name) VALUES ('Tbilisi');
INSERT INTO cities (name) VALUES ('Tchamba');
INSERT INTO cities (name) VALUES ('Tebessa');
INSERT INTO cities (name) VALUES ('Tebingtinggi');
INSERT INTO cities (name) VALUES ('Tegal');
INSERT INTO cities (name) VALUES ('Tegucigalpa');
INSERT INTO cities (name) VALUES ('Tehran');
INSERT INTO cities (name) VALUES ('Tehuacan');
INSERT INTO cities (name) VALUES ('TeixeiradeFreitas');
INSERT INTO cities (name) VALUES ('Tekirdag');
INSERT INTO cities (name) VALUES ('Telford');
INSERT INTO cities (name) VALUES ('TelAviv-Yafo');
INSERT INTO cities (name) VALUES ('Temaraia');
INSERT INTO cities (name) VALUES ('Temecula');
INSERT INTO cities (name) VALUES ('Temirtau');
INSERT INTO cities (name) VALUES ('Tempe');
INSERT INTO cities (name) VALUES ('Temple');
INSERT INTO cities (name) VALUES ('Temuco');
INSERT INTO cities (name) VALUES ('Tengzhou');
INSERT INTO cities (name) VALUES ('TeofiloOtoni');
INSERT INTO cities (name) VALUES ('Tepic');
INSERT INTO cities (name) VALUES ('Teresina');
INSERT INTO cities (name) VALUES ('Teresopolis');
INSERT INTO cities (name) VALUES ('Termez');
INSERT INTO cities (name) VALUES ('Ternate');
INSERT INTO cities (name) VALUES ('Terni');
INSERT INTO cities (name) VALUES ('Ternopil\'');
INSERT INTO cities (name) VALUES ('Terre Haute');
INSERT INTO cities (name) VALUES ('Tessaoua');
INSERT INTO cities (name) VALUES ('Tete');
INSERT INTO cities (name) VALUES ('Tetouan');
INSERT INTO cities (name) VALUES ('Tetovo');
INSERT INTO cities (name) VALUES ('ThaiNguyen');
INSERT INTO cities (name) VALUES ('Thanjavur');
INSERT INTO cities (name) VALUES ('Thaton');
INSERT INTO cities (name) VALUES ('Thawrah');
INSERT INTO cities (name) VALUES ('The Hammocks');
INSERT INTO cities (name) VALUES ('The Woodlands');
INSERT INTO cities (name) VALUES ('Thessaloniki');
INSERT INTO cities (name) VALUES ('TheBottom');
INSERT INTO cities (name) VALUES ('TheSettlement');
INSERT INTO cities (name) VALUES ('TheValley');
INSERT INTO cities (name) VALUES ('Thies');
INSERT INTO cities (name) VALUES ('Thinadhoo');
INSERT INTO cities (name) VALUES ('Thiruvananthapuram');
INSERT INTO cities (name) VALUES ('Thornton');
INSERT INTO cities (name) VALUES ('Thornton-Cleveleys');
INSERT INTO cities (name) VALUES ('Thousand Oaks');
INSERT INTO cities (name) VALUES ('Thrissur');
INSERT INTO cities (name) VALUES ('Thulhaadhoo');
INSERT INTO cities (name) VALUES ('Thunder Bay');
INSERT INTO cities (name) VALUES ('Thuthukkudi');
INSERT INTO cities (name) VALUES ('Tiangjiaan');
INSERT INTO cities (name) VALUES ('Tianjin');
INSERT INTO cities (name) VALUES ('Tianmen');
INSERT INTO cities (name) VALUES ('Tiberias');
INSERT INTO cities (name) VALUES ('Tieli');
INSERT INTO cities (name) VALUES ('Tieling');
INSERT INTO cities (name) VALUES ('Tighina');
INSERT INTO cities (name) VALUES ('Tijuana');
INSERT INTO cities (name) VALUES ('Tilafar');
INSERT INTO cities (name) VALUES ('Tilburg');
INSERT INTO cities (name) VALUES ('Timisoara');
INSERT INTO cities (name) VALUES ('Timon');
INSERT INTO cities (name) VALUES ('Timphu');
INSERT INTO cities (name) VALUES ('Tiniteqilaaq');
INSERT INTO cities (name) VALUES ('Tinshuiwei');
INSERT INTO cities (name) VALUES ('Tirana');
INSERT INTO cities (name) VALUES ('Tiraspol');
INSERT INTO cities (name) VALUES ('Tiruchchirappalli');
INSERT INTO cities (name) VALUES ('Tirunelveli');
INSERT INTO cities (name) VALUES ('Tirupati');
INSERT INTO cities (name) VALUES ('Tiruppur');
INSERT INTO cities (name) VALUES ('Titusville');
INSERT INTO cities (name) VALUES ('Tiyarat');
INSERT INTO cities (name) VALUES ('Tlalnepantla');
INSERT INTO cities (name) VALUES ('Tlaquepaque');
INSERT INTO cities (name) VALUES ('Toamasina');
INSERT INTO cities (name) VALUES ('Tobolsk');
INSERT INTO cities (name) VALUES ('Tobruk');
INSERT INTO cities (name) VALUES ('Tokat');
INSERT INTO cities (name) VALUES ('Tokmok');
INSERT INTO cities (name) VALUES ('Tokushima');
INSERT INTO cities (name) VALUES ('Tokyo');
INSERT INTO cities (name) VALUES ('Tol');
INSERT INTO cities (name) VALUES ('Toledo');
INSERT INTO cities (name) VALUES ('Toliary');
INSERT INTO cities (name) VALUES ('Toluca');
INSERT INTO cities (name) VALUES ('Tolyatti');
INSERT INTO cities (name) VALUES ('Tomakomai');
INSERT INTO cities (name) VALUES ('Tombouctou');
INSERT INTO cities (name) VALUES ('Toms River');
INSERT INTO cities (name) VALUES ('Tomsk');
INSERT INTO cities (name) VALUES ('Tonga');
INSERT INTO cities (name) VALUES ('Tongchuan');
INSERT INTO cities (name) VALUES ('Tonghae');
INSERT INTO cities (name) VALUES ('Tongi');
INSERT INTO cities (name) VALUES ('Tongliao');
INSERT INTO cities (name) VALUES ('Tongling');
INSERT INTO cities (name) VALUES ('Tongren');
INSERT INTO cities (name) VALUES ('Tongyong');
INSERT INTO cities (name) VALUES ('Tongzhou');
INSERT INTO cities (name) VALUES ('Tooele');
INSERT INTO cities (name) VALUES ('Topeka');
INSERT INTO cities (name) VALUES ('Torino');
INSERT INTO cities (name) VALUES ('Toronto');
INSERT INTO cities (name) VALUES ('Torquay');
INSERT INTO cities (name) VALUES ('Torrance');
INSERT INTO cities (name) VALUES ('Torreon');
INSERT INTO cities (name) VALUES ('Torshavn');
INSERT INTO cities (name) VALUES ('Torun');
INSERT INTO cities (name) VALUES ('Totton');
INSERT INTO cities (name) VALUES ('Touliu');
INSERT INTO cities (name) VALUES ('Toulon');
INSERT INTO cities (name) VALUES ('Toulouse');
INSERT INTO cities (name) VALUES ('Toungoo');
INSERT INTO cities (name) VALUES ('Tours');
INSERT INTO cities (name) VALUES ('Townsville');
INSERT INTO cities (name) VALUES ('Toyama');
INSERT INTO cities (name) VALUES ('Toyohashi');
INSERT INTO cities (name) VALUES ('Trabzon');
INSERT INTO cities (name) VALUES ('Tralee');
INSERT INTO cities (name) VALUES ('Trelew');
INSERT INTO cities (name) VALUES ('Trento');
INSERT INTO cities (name) VALUES ('Trenton');
INSERT INTO cities (name) VALUES ('Trier');
INSERT INTO cities (name) VALUES ('Triesen');
INSERT INTO cities (name) VALUES ('Trieste');
INSERT INTO cities (name) VALUES ('Trincomalee');
INSERT INTO cities (name) VALUES ('Trinidade');
INSERT INTO cities (name) VALUES ('Triolet');
INSERT INTO cities (name) VALUES ('Tripoli');
INSERT INTO cities (name) VALUES ('Trois-Rivieres');
INSERT INTO cities (name) VALUES ('Trondheim');
INSERT INTO cities (name) VALUES ('Trowbridge');
INSERT INTO cities (name) VALUES ('Troy');
INSERT INTO cities (name) VALUES ('Troyes');
INSERT INTO cities (name) VALUES ('Trujillo');
INSERT INTO cities (name) VALUES ('Tsevie');
INSERT INTO cities (name) VALUES ('Tshikapa');
INSERT INTO cities (name) VALUES ('Tsu');
INSERT INTO cities (name) VALUES ('Tsumeb');
INSERT INTO cities (name) VALUES ('Tuanmun');
INSERT INTO cities (name) VALUES ('Tucheng');
INSERT INTO cities (name) VALUES ('Tucson');
INSERT INTO cities (name) VALUES ('Tucuman');
INSERT INTO cities (name) VALUES ('Tula');
INSERT INTO cities (name) VALUES ('Tulare');
INSERT INTO cities (name) VALUES ('Tulsa');
INSERT INTO cities (name) VALUES ('Tulua');
INSERT INTO cities (name) VALUES ('Tumkur');
INSERT INTO cities (name) VALUES ('Tunis');
INSERT INTO cities (name) VALUES ('Tunja');
INSERT INTO cities (name) VALUES ('Turkmenabat');
INSERT INTO cities (name) VALUES ('Turku');
INSERT INTO cities (name) VALUES ('Turlock');
INSERT INTO cities (name) VALUES ('Turmero');
INSERT INTO cities (name) VALUES ('Turrialba');
INSERT INTO cities (name) VALUES ('Tuscaloosa');
INSERT INTO cities (name) VALUES ('Tustin');
INSERT INTO cities (name) VALUES ('Tuxtla Gutierrez');
INSERT INTO cities (name) VALUES ('Tuzla');
INSERT INTO cities (name) VALUES ('Tver');
INSERT INTO cities (name) VALUES ('Tvoroyri');
INSERT INTO cities (name) VALUES ('Tyler');
INSERT INTO cities (name) VALUES ('Tyumen');
INSERT INTO cities (name) VALUES ('Ube');
INSERT INTO cities (name) VALUES ('Uberaba');
INSERT INTO cities (name) VALUES ('Uberlandia');
INSERT INTO cities (name) VALUES ('UbonRatchathani');
INSERT INTO cities (name) VALUES ('Udaipur');
INSERT INTO cities (name) VALUES ('Udine');
INSERT INTO cities (name) VALUES ('UdonThani');
INSERT INTO cities (name) VALUES ('Ufa');
INSERT INTO cities (name) VALUES ('Ugep');
INSERT INTO cities (name) VALUES ('Ujjain');
INSERT INTO cities (name) VALUES ('Ujung Pandang');
INSERT INTO cities (name) VALUES ('Ukhta');
INSERT INTO cities (name) VALUES ('Ukkusissat');
INSERT INTO cities (name) VALUES ('Ulaanbaatar');
INSERT INTO cities (name) VALUES ('Ulaangom');
INSERT INTO cities (name) VALUES ('Ulan-Ude');
INSERT INTO cities (name) VALUES ('Ulimang');
INSERT INTO cities (name) VALUES ('Ulm');
INSERT INTO cities (name) VALUES ('Ulsan');
INSERT INTO cities (name) VALUES ('Ulubaria');
INSERT INTO cities (name) VALUES ('Ulyanovsk');
INSERT INTO cities (name) VALUES ('Umm Salal');
INSERT INTO cities (name) VALUES ('Ummal-Qaywayn');
INSERT INTO cities (name) VALUES ('Umuahia');
INSERT INTO cities (name) VALUES ('Unayzah');
INSERT INTO cities (name) VALUES ('Union City');
INSERT INTO cities (name) VALUES ('Upernavik');
INSERT INTO cities (name) VALUES ('UpernavikKujalleq');
INSERT INTO cities (name) VALUES ('Upland');
INSERT INTO cities (name) VALUES ('Uppsala');
INSERT INTO cities (name) VALUES ('Urawa');
INSERT INTO cities (name) VALUES ('Urfa');
INSERT INTO cities (name) VALUES ('Urgench');
INSERT INTO cities (name) VALUES ('Uromi');
INSERT INTO cities (name) VALUES ('Uroteppa');
INSERT INTO cities (name) VALUES ('Uruapan');
INSERT INTO cities (name) VALUES ('Uruguaiana');
INSERT INTO cities (name) VALUES ('Urumqi');
INSERT INTO cities (name) VALUES ('Usak');
INSERT INTO cities (name) VALUES ('Ushuaia');
INSERT INTO cities (name) VALUES ('Usolye-Sibirskoye');
INSERT INTO cities (name) VALUES ('Ussuriysk');
INSERT INTO cities (name) VALUES ('Ust-Ilimsk');
INSERT INTO cities (name) VALUES ('Usti');
INSERT INTO cities (name) VALUES ('Utica');
INSERT INTO cities (name) VALUES ('Utiroa');
INSERT INTO cities (name) VALUES ('Utrecht');
INSERT INTO cities (name) VALUES ('Utsunomiya');
INSERT INTO cities (name) VALUES ('Uummannaq');
INSERT INTO cities (name) VALUES ('Uvira');
INSERT INTO cities (name) VALUES ('Uyo');
INSERT INTO cities (name) VALUES ('Uzgen');
INSERT INTO cities (name) VALUES ('Uzhhorod');
INSERT INTO cities (name) VALUES ('Uge');
INSERT INTO cities (name) VALUES ('Vacaville');
INSERT INTO cities (name) VALUES ('Vadodara');
INSERT INTO cities (name) VALUES ('Vaduz');
INSERT INTO cities (name) VALUES ('Vagharshapat');
INSERT INTO cities (name) VALUES ('Vaiaku');
INSERT INTO cities (name) VALUES ('Valdivia');
INSERT INTO cities (name) VALUES ('Valdosta');
INSERT INTO cities (name) VALUES ('Valence');
INSERT INTO cities (name) VALUES ('Valencia');
INSERT INTO cities (name) VALUES ('Valera');
INSERT INTO cities (name) VALUES ('Valladolid');
INSERT INTO cities (name) VALUES ('Valledupar');
INSERT INTO cities (name) VALUES ('Vallejo');
INSERT INTO cities (name) VALUES ('Valletta');
INSERT INTO cities (name) VALUES ('Valparaso');
INSERT INTO cities (name) VALUES ('Van');
INSERT INTO cities (name) VALUES ('Vanadzor');
INSERT INTO cities (name) VALUES ('Vancouver');
INSERT INTO cities (name) VALUES ('Vannes');
INSERT INTO cities (name) VALUES ('Vantaa');
INSERT INTO cities (name) VALUES ('Varanasi');
INSERT INTO cities (name) VALUES ('Varginha');
INSERT INTO cities (name) VALUES ('Varna');
INSERT INTO cities (name) VALUES ('VarzeaGrande');
INSERT INTO cities (name) VALUES ('Vasteras');
INSERT INTO cities (name) VALUES ('Vatican');
INSERT INTO cities (name) VALUES ('Vaughan');
INSERT INTO cities (name) VALUES ('Velidhoo');
INSERT INTO cities (name) VALUES ('Velikiy Novgorod');
INSERT INTO cities (name) VALUES ('Velikiye Luki');
INSERT INTO cities (name) VALUES ('Venezia');
INSERT INTO cities (name) VALUES ('Ventspils');
INSERT INTO cities (name) VALUES ('Veracruz');
INSERT INTO cities (name) VALUES ('Vereeniging');
INSERT INTO cities (name) VALUES ('Verona');
INSERT INTO cities (name) VALUES ('Verrettes');
INSERT INTO cities (name) VALUES ('Versailles');
INSERT INTO cities (name) VALUES ('Viamao');
INSERT INTO cities (name) VALUES ('Vicenza');
INSERT INTO cities (name) VALUES ('Victoria');
INSERT INTO cities (name) VALUES ('Victorville');
INSERT INTO cities (name) VALUES ('Vienna (Wien)');
INSERT INTO cities (name) VALUES ('Vientiane');
INSERT INTO cities (name) VALUES ('VieuxFort');
INSERT INTO cities (name) VALUES ('Vigo');
INSERT INTO cities (name) VALUES ('Vihari');
INSERT INTO cities (name) VALUES ('Vijayawada');
INSERT INTO cities (name) VALUES ('Vila');
INSERT INTO cities (name) VALUES ('VilaVelha');
INSERT INTO cities (name) VALUES ('Viligili');
INSERT INTO cities (name) VALUES ('Villach');
INSERT INTO cities (name) VALUES ('Villahermosa');
INSERT INTO cities (name) VALUES ('Villavicencio');
INSERT INTO cities (name) VALUES ('VillaCarlosPaz');
INSERT INTO cities (name) VALUES ('Vilnius');
INSERT INTO cities (name) VALUES ('Vinnytsya');
INSERT INTO cities (name) VALUES ('Viransehir');
INSERT INTO cities (name) VALUES ('Virginia Beach');
INSERT INTO cities (name) VALUES ('Visakhapatnam');
INSERT INTO cities (name) VALUES ('Visalia');
INSERT INTO cities (name) VALUES ('Vista');
INSERT INTO cities (name) VALUES ('Vitebsk');
INSERT INTO cities (name) VALUES ('Vitoria');
INSERT INTO cities (name) VALUES ('VitoriadaConquista');
INSERT INTO cities (name) VALUES ('VitoriadeSantoAntao');
INSERT INTO cities (name) VALUES ('Via del Mar');
INSERT INTO cities (name) VALUES ('Vladikavkaz');
INSERT INTO cities (name) VALUES ('Vladimir');
INSERT INTO cities (name) VALUES ('Vladivostok');
INSERT INTO cities (name) VALUES ('Volgodonsk');
INSERT INTO cities (name) VALUES ('Volgograd');
INSERT INTO cities (name) VALUES ('Vologda');
INSERT INTO cities (name) VALUES ('VoltaRedonda');
INSERT INTO cities (name) VALUES ('Volzhskiy');
INSERT INTO cities (name) VALUES ('Voronezh');
INSERT INTO cities (name) VALUES ('Vostok');
INSERT INTO cities (name) VALUES ('Votkinsk');
INSERT INTO cities (name) VALUES ('VungTau');
INSERT INTO cities (name) VALUES ('Waco');
INSERT INTO cities (name) VALUES ('Wad Medaniy');
INSERT INTO cities (name) VALUES ('Wafangdian');
INSERT INTO cities (name) VALUES ('Wah');
INSERT INTO cities (name) VALUES ('Wakayama');
INSERT INTO cities (name) VALUES ('Walbrzych');
INSERT INTO cities (name) VALUES ('Walnut Creek');
INSERT INTO cities (name) VALUES ('WalvisBay');
INSERT INTO cities (name) VALUES ('Wanxian');
INSERT INTO cities (name) VALUES ('Warangal');
INSERT INTO cities (name) VALUES ('Warner Robins');
INSERT INTO cities (name) VALUES ('Warren');
INSERT INTO cities (name) VALUES ('Warri');
INSERT INTO cities (name) VALUES ('Warszawa');
INSERT INTO cities (name) VALUES ('Washington');
INSERT INTO cities (name) VALUES ('Waterbury');
INSERT INTO cities (name) VALUES ('Waterford');
INSERT INTO cities (name) VALUES ('Waterloo');
INSERT INTO cities (name) VALUES ('Waterlooville');
INSERT INTO cities (name) VALUES ('Watford');
INSERT INTO cities (name) VALUES ('Waukegan');
INSERT INTO cities (name) VALUES ('Waw');
INSERT INTO cities (name) VALUES ('We');
INSERT INTO cities (name) VALUES ('Weifang');
INSERT INTO cities (name) VALUES ('Weihai');
INSERT INTO cities (name) VALUES ('Weinan');
INSERT INTO cities (name) VALUES ('Welkom');
INSERT INTO cities (name) VALUES ('Welland');
INSERT INTO cities (name) VALUES ('Wellingborough');
INSERT INTO cities (name) VALUES ('Wellington');
INSERT INTO cities (name) VALUES ('Wels');
INSERT INTO cities (name) VALUES ('Wencheng');
INSERT INTO cities (name) VALUES ('Weno');
INSERT INTO cities (name) VALUES ('Wenzhou');
INSERT INTO cities (name) VALUES ('Weonju');
INSERT INTO cities (name) VALUES ('West Covina');
INSERT INTO cities (name) VALUES ('West Palm Beach');
INSERT INTO cities (name) VALUES ('West Valley City');
INSERT INTO cities (name) VALUES ('Westland');
INSERT INTO cities (name) VALUES ('Westminster');
INSERT INTO cities (name) VALUES ('Weston');
INSERT INTO cities (name) VALUES ('Weston-super-Mare');
INSERT INTO cities (name) VALUES ('WestBay');
INSERT INTO cities (name) VALUES ('WestIsland');
INSERT INTO cities (name) VALUES ('Wewak');
INSERT INTO cities (name) VALUES ('Weymouth');
INSERT INTO cities (name) VALUES ('Whangarei');
INSERT INTO cities (name) VALUES ('Whitehaven');
INSERT INTO cities (name) VALUES ('Whitehorse');
INSERT INTO cities (name) VALUES ('Whitstable');
INSERT INTO cities (name) VALUES ('Whittier');
INSERT INTO cities (name) VALUES ('Wichita');
INSERT INTO cities (name) VALUES ('Wichita Falls');
INSERT INTO cities (name) VALUES ('Wiesbaden');
INSERT INTO cities (name) VALUES ('Wigston');
INSERT INTO cities (name) VALUES ('Wilkes-Barre');
INSERT INTO cities (name) VALUES ('Willemstad');
INSERT INTO cities (name) VALUES ('Wilmington');
INSERT INTO cities (name) VALUES ('Wilson');
INSERT INTO cities (name) VALUES ('Winchester');
INSERT INTO cities (name) VALUES ('Windhoek');
INSERT INTO cities (name) VALUES ('Windsor');
INSERT INTO cities (name) VALUES ('Winnipeg');
INSERT INTO cities (name) VALUES ('Winsford');
INSERT INTO cities (name) VALUES ('Winston-Salem');
INSERT INTO cities (name) VALUES ('Winterthur');
INSERT INTO cities (name) VALUES ('Wisbech');
INSERT INTO cities (name) VALUES ('Witbank');
INSERT INTO cities (name) VALUES ('Witten');
INSERT INTO cities (name) VALUES ('Wloclawek');
INSERT INTO cities (name) VALUES ('Woking-Byfleet');
INSERT INTO cities (name) VALUES ('Wolfsburg');
INSERT INTO cities (name) VALUES ('Wollongong');
INSERT INTO cities (name) VALUES ('Wolverhampton');
INSERT INTO cities (name) VALUES ('Wolverton-Stony Stratford');
INSERT INTO cities (name) VALUES ('Wonsan');
INSERT INTO cities (name) VALUES ('Worcester');
INSERT INTO cities (name) VALUES ('Workington');
INSERT INTO cities (name) VALUES ('Worthing');
INSERT INTO cities (name) VALUES ('Wrexham');
INSERT INTO cities (name) VALUES ('Wroclaw');
INSERT INTO cities (name) VALUES ('Wuda');
INSERT INTO cities (name) VALUES ('Wuhan');
INSERT INTO cities (name) VALUES ('Wuhu');
INSERT INTO cities (name) VALUES ('Wulanhaote');
INSERT INTO cities (name) VALUES ('Wuning');
INSERT INTO cities (name) VALUES ('Wuppertal');
INSERT INTO cities (name) VALUES ('Wurzburg');
INSERT INTO cities (name) VALUES ('Wuwei');
INSERT INTO cities (name) VALUES ('Wuxi');
INSERT INTO cities (name) VALUES ('Wuxue');
INSERT INTO cities (name) VALUES ('Wuzhou');
INSERT INTO cities (name) VALUES ('Xai-Xai');
INSERT INTO cities (name) VALUES ('Xalapa');
INSERT INTO cities (name) VALUES ('Xiamen');
INSERT INTO cities (name) VALUES ('Xian');
INSERT INTO cities (name) VALUES ('Xiangdong');
INSERT INTO cities (name) VALUES ('Xiangfan');
INSERT INTO cities (name) VALUES ('Xianggangzi');
INSERT INTO cities (name) VALUES ('Xiangtan');
INSERT INTO cities (name) VALUES ('Xianning');
INSERT INTO cities (name) VALUES ('Xiantao');
INSERT INTO cities (name) VALUES ('Xianyang');
INSERT INTO cities (name) VALUES ('Xiaogan');
INSERT INTO cities (name) VALUES ('Xiaolan');
INSERT INTO cities (name) VALUES ('Xichang');
INSERT INTO cities (name) VALUES ('Xilinhaote');
INSERT INTO cities (name) VALUES ('Xinan');
INSERT INTO cities (name) VALUES ('Xincheng');
INSERT INTO cities (name) VALUES ('Xingcheng');
INSERT INTO cities (name) VALUES ('Xingtai');
INSERT INTO cities (name) VALUES ('Xingyi');
INSERT INTO cities (name) VALUES ('Xining');
INSERT INTO cities (name) VALUES ('Xinji');
INSERT INTO cities (name) VALUES ('Xinpu');
INSERT INTO cities (name) VALUES ('Xinshi');
INSERT INTO cities (name) VALUES ('Xintai');
INSERT INTO cities (name) VALUES ('Xinxiang');
INSERT INTO cities (name) VALUES ('Xinyang');
INSERT INTO cities (name) VALUES ('Xinzhou');
INSERT INTO cities (name) VALUES ('Xuanhua');
INSERT INTO cities (name) VALUES ('Xuanzhou');
INSERT INTO cities (name) VALUES ('Xuchang');
INSERT INTO cities (name) VALUES ('Xuzhou');
INSERT INTO cities (name) VALUES ('Yaan');
INSERT INTO cities (name) VALUES ('Yafran');
INSERT INTO cities (name) VALUES ('Yakeshi');
INSERT INTO cities (name) VALUES ('Yakima');
INSERT INTO cities (name) VALUES ('Yakutsk');
INSERT INTO cities (name) VALUES ('Yamagata');
INSERT INTO cities (name) VALUES ('Yamoussoukro');
INSERT INTO cities (name) VALUES ('Yanan');
INSERT INTO cities (name) VALUES ('Yanbu');
INSERT INTO cities (name) VALUES ('Yancheng');
INSERT INTO cities (name) VALUES ('Yangjiang');
INSERT INTO cities (name) VALUES ('Yangmei');
INSERT INTO cities (name) VALUES ('Yangon');
INSERT INTO cities (name) VALUES ('Yangquan');
INSERT INTO cities (name) VALUES ('Yangzhou');
INSERT INTO cities (name) VALUES ('Yanji');
INSERT INTO cities (name) VALUES ('Yantai');
INSERT INTO cities (name) VALUES ('Yaroslavl');
INSERT INTO cities (name) VALUES ('Yauco');
INSERT INTO cities (name) VALUES ('Yazd');
INSERT INTO cities (name) VALUES ('Yekaterinburg');
INSERT INTO cities (name) VALUES ('Yelets');
INSERT INTO cities (name) VALUES ('Yellowknife');
INSERT INTO cities (name) VALUES ('Yenangyaung');
INSERT INTO cities (name) VALUES ('Yeosu');
INSERT INTO cities (name) VALUES ('Yeovil');
INSERT INTO cities (name) VALUES ('Yerevan');
INSERT INTO cities (name) VALUES ('Yessentuki');
INSERT INTO cities (name) VALUES ('Yevpatoriya');
INSERT INTO cities (name) VALUES ('Yibin');
INSERT INTO cities (name) VALUES ('Yichang');
INSERT INTO cities (name) VALUES ('Yichun');
INSERT INTO cities (name) VALUES ('Yidu');
INSERT INTO cities (name) VALUES ('Yinchuan');
INSERT INTO cities (name) VALUES ('Yingcheng');
INSERT INTO cities (name) VALUES ('Yingkou');
INSERT INTO cities (name) VALUES ('Yingzhong');
INSERT INTO cities (name) VALUES ('Yining');
INSERT INTO cities (name) VALUES ('Yiyang');
INSERT INTO cities (name) VALUES ('Yizheng');
INSERT INTO cities (name) VALUES ('Yogyakarta');
INSERT INTO cities (name) VALUES ('Yokohama');
INSERT INTO cities (name) VALUES ('Yokosuka');
INSERT INTO cities (name) VALUES ('Yongan');
INSERT INTO cities (name) VALUES ('Yonkers');
INSERT INTO cities (name) VALUES ('Yorba Linda');
INSERT INTO cities (name) VALUES ('York');
INSERT INTO cities (name) VALUES ('Yoshkar-Ola');
INSERT INTO cities (name) VALUES ('Youngstown');
INSERT INTO cities (name) VALUES ('Yuanlin');
INSERT INTO cities (name) VALUES ('Yuanlong');
INSERT INTO cities (name) VALUES ('Yuci');
INSERT INTO cities (name) VALUES ('Yueyang');
INSERT INTO cities (name) VALUES ('Yuhong');
INSERT INTO cities (name) VALUES ('Yulin');
INSERT INTO cities (name) VALUES ('Yuma');
INSERT INTO cities (name) VALUES ('Yuncheng');
INSERT INTO cities (name) VALUES ('Yungho');
INSERT INTO cities (name) VALUES ('Yungkang');
INSERT INTO cities (name) VALUES ('Yunyang');
INSERT INTO cities (name) VALUES ('Yushan');
INSERT INTO cities (name) VALUES ('Yushu');
INSERT INTO cities (name) VALUES ('Yuyao');
INSERT INTO cities (name) VALUES ('Yuzhno-Sahalinsk');
INSERT INTO cities (name) VALUES ('Zaanstad');
INSERT INTO cities (name) VALUES ('Zabid');
INSERT INTO cities (name) VALUES ('Zabol');
INSERT INTO cities (name) VALUES ('Zacatecas');
INSERT INTO cities (name) VALUES ('Zadar');
INSERT INTO cities (name) VALUES ('Zagazig');
INSERT INTO cities (name) VALUES ('Zagreb');
INSERT INTO cities (name) VALUES ('Zahedan');
INSERT INTO cities (name) VALUES ('Zahlah');
INSERT INTO cities (name) VALUES ('Zalantun');
INSERT INTO cities (name) VALUES ('Zamboanga');
INSERT INTO cities (name) VALUES ('Zamora');
INSERT INTO cities (name) VALUES ('Zanjan');
INSERT INTO cities (name) VALUES ('Zanzibar');
INSERT INTO cities (name) VALUES ('Zaoyang');
INSERT INTO cities (name) VALUES ('Zaozhuang');
INSERT INTO cities (name) VALUES ('Zapopan');
INSERT INTO cities (name) VALUES ('Zaporizhzhya');
INSERT INTO cities (name) VALUES ('Zaragoza');
INSERT INTO cities (name) VALUES ('Zaria');
INSERT INTO cities (name) VALUES ('Zawiyah');
INSERT INTO cities (name) VALUES ('Zelyenograd');
INSERT INTO cities (name) VALUES ('Zelyonodolsk');
INSERT INTO cities (name) VALUES ('Zenica');
INSERT INTO cities (name) VALUES ('Zhalainuoer');
INSERT INTO cities (name) VALUES ('Zhangdian');
INSERT INTO cities (name) VALUES ('Zhangjiakou');
INSERT INTO cities (name) VALUES ('Zhangye');
INSERT INTO cities (name) VALUES ('Zhangzhou');
INSERT INTO cities (name) VALUES ('Zhanjiang');
INSERT INTO cities (name) VALUES ('Zhaocheng');
INSERT INTO cities (name) VALUES ('Zhaodong');
INSERT INTO cities (name) VALUES ('Zhaoqing');
INSERT INTO cities (name) VALUES ('Zhaotong');
INSERT INTO cities (name) VALUES ('Zhaoyang');
INSERT INTO cities (name) VALUES ('Zheleznogorsk');
INSERT INTO cities (name) VALUES ('Zhengzhou');
INSERT INTO cities (name) VALUES ('Zhenjiang');
INSERT INTO cities (name) VALUES ('Zhicheng');
INSERT INTO cities (name) VALUES ('Zhilina');
INSERT INTO cities (name) VALUES ('Zhizaq');
INSERT INTO cities (name) VALUES ('Zhongshan');
INSERT INTO cities (name) VALUES ('Zhoucun');
INSERT INTO cities (name) VALUES ('Zhoukou');
INSERT INTO cities (name) VALUES ('Zhucheng');
INSERT INTO cities (name) VALUES ('Zhuhai');
INSERT INTO cities (name) VALUES ('Zhuji');
INSERT INTO cities (name) VALUES ('Zhumadian');
INSERT INTO cities (name) VALUES ('Zhuozhou');
INSERT INTO cities (name) VALUES ('Zhuzhou');
INSERT INTO cities (name) VALUES ('Zhytomyr');
INSERT INTO cities (name) VALUES ('Zielona Gora');
INSERT INTO cities (name) VALUES ('Zigong');
INSERT INTO cities (name) VALUES ('Ziguinchor');
INSERT INTO cities (name) VALUES ('Zinder');
INSERT INTO cities (name) VALUES ('Zlatoust');
INSERT INTO cities (name) VALUES ('Zlitan');
INSERT INTO cities (name) VALUES ('Zoetermeer');
INSERT INTO cities (name) VALUES ('Zomba');
INSERT INTO cities (name) VALUES ('Zonguldak');
INSERT INTO cities (name) VALUES ('Zouxian');
INSERT INTO cities (name) VALUES ('Zrenjanin');
INSERT INTO cities (name) VALUES ('Zubayr');
INSERT INTO cities (name) VALUES ('Zugdidi');
INSERT INTO cities (name) VALUES ('Zunyi');
INSERT INTO cities (name) VALUES ('Zurich');
INSERT INTO cities (name) VALUES ('Zvishavane');
INSERT INTO cities (name) VALUES ('Zwedru');
INSERT INTO cities (name) VALUES ('Zwolle');

COMMIT;

-- -------------------------------------------------------------------------------------------------
-- Set the current database version number
-- -------------------------------------------------------------------------------------------------

INSERT INTO system_prefs (name, value) VALUES('DATABASE_VERSION','1.18');

-- *************************************************************************************************
--   SWISScenter Source                                                              Robert Taylor
-- *************************************************************************************************
